package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.boot.tools.excel.ExcelReader;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.distributed.print.business.bean.excel.PaperStructDto;
import com.qmth.distributed.print.business.entity.TCPaperStruct;
import com.qmth.distributed.print.business.mapper.TCPaperStructMapper;
import com.qmth.distributed.print.business.service.PrintCommonService;
import com.qmth.distributed.print.business.service.TCFinalScoreService;
import com.qmth.distributed.print.business.service.TCPaperStructService;
import com.qmth.distributed.print.business.service.TRBasicInfoService;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.FieldUniqueEnum;
import com.qmth.teachcloud.common.util.JacksonUtil;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.*;

/**
 * <p>
 * 试卷蓝图表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
@Service
public class TCPaperStructServiceImpl extends ServiceImpl<TCPaperStructMapper, TCPaperStruct> implements TCPaperStructService {
    private final static Logger log = LoggerFactory.getLogger(TCPaperStructServiceImpl.class);

    @Resource
    TCPaperStructService tcPaperStructService;

    @Resource
    TCFinalScoreService tcFinalScoreService;

    @Resource
    TRBasicInfoService trBasicInfoService;

    @Resource
    PrintCommonService printCommonService;

    /**
     * 根据考试id/课程编码/试卷编码查询数据库数据
     *
     * @param examId
     * @param courseCode
     * @param paperNumber
     * @return
     */
    @Override
    @Transactional
    public Boolean remove(Long examId, String courseCode, String paperNumber) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        return tcPaperStructService.remove(new QueryWrapper<TCPaperStruct>().lambda().eq(TCPaperStruct::getExamId, examId)
                .eq(TCPaperStruct::getCourseCode, courseCode)
                .eq(TCPaperStruct::getPaperNumber, paperNumber)
                .eq(TCPaperStruct::getCreateId, sysUser.getId()));
    }

    /**
     * 导入期末成绩-试卷结构excel
     *
     * @param file
     * @param examId
     * @param courseCode
     * @param paperNumber
     * @return
     * @throws IOException
     */
    @Override
    @Transactional
    public Map<String, String> paperStructExcelImport(MultipartFile file, Long examId, String courseCode, String paperNumber) throws IOException {
        log.debug("导入Excel开始...");
        long start = System.currentTimeMillis();
        Map<String, String> messageMap = new LinkedHashMap<>();
        try {
            MarkPaper markPaper = printCommonService.scoreImportExcelVaild(file, examId, courseCode, paperNumber);
            StringJoiner errorData = new StringJoiner("");
            StringJoiner successData = new StringJoiner("");

            ExcelReader excelReader = ExcelReader.create(ExcelType.XLSX, file.getInputStream(), 1);
            List<PaperStructDto> paperStructDtoList = excelReader.getObjectList(PaperStructDto.class);
            log.info("paperStructDtoList:{}", JacksonUtil.parseJson(paperStructDtoList));

            if (!CollectionUtils.isEmpty(paperStructDtoList)) {
                SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
                String[] columnNames = excelReader.getColumnNames();
                for (int i = 0; i < columnNames.length; i++) {
                    if (i == 0 && Objects.nonNull(columnNames[i]) && !columnNames[i].trim().contains("大题名称")) {
                        throw ExceptionResultEnum.ERROR.exception("excel表头第一行为大题名称");
                    } else if (i == 1 && Objects.nonNull(columnNames[i]) && !columnNames[i].trim().contains("大题号")) {
                        throw ExceptionResultEnum.ERROR.exception("excel表头第二行为大题号");
                    } else if (i == 2 && Objects.nonNull(columnNames[i]) && !columnNames[i].trim().contains("小题号")) {
                        throw ExceptionResultEnum.ERROR.exception("excel表头第三行为小题号");
                    } else if (i == 3 && Objects.nonNull(columnNames[i]) && !columnNames[i].trim().contains("小题满分")) {
                        throw ExceptionResultEnum.ERROR.exception("excel表头第四行为小题满分");
                    }
                }

                Double totalScore = 0.0d;
                List<PaperStructDto> paperStructDtoNewList = new ArrayList<>(paperStructDtoList.size());
                for (int i = 0; i < paperStructDtoList.size(); i++) {
                    PaperStructDto paperStructDto = paperStructDtoList.get(i);
                    boolean error = false;
                    if (Objects.isNull(paperStructDto.getMainTitle()) || Objects.equals(paperStructDto.getMainTitle().trim(), "")) {
                        errorData.add("excel第").add((i + 1) + "").add("行[").add(columnNames[0] + "]为空;").add("\r\n");
                        error = true;
                    }
                    if (Objects.isNull(paperStructDto.getMainNumber()) || Objects.equals(paperStructDto.getMainNumber(), "")) {
                        errorData.add("excel第").add((i + 1) + "").add("行[").add(columnNames[1] + "]为空;").add("\r\n");
                        error = true;
                    }
                    if (Objects.isNull(paperStructDto.getSubNumber()) || Objects.equals(paperStructDto.getSubNumber(), "")) {
                        errorData.add("excel第").add((i + 1) + "").add("行[").add(columnNames[2] + "]为空;").add("\r\n");
                        error = true;
                    }
                    if (Objects.isNull(paperStructDto.getScore()) || Objects.equals(paperStructDto.getScore(), "")) {
                        errorData.add("excel第").add((i + 1) + "").add("行[").add(columnNames[3] + "]为空;").add("\r\n");
                        error = true;
                    }
                    if (!error) {
                        totalScore = totalScore + paperStructDto.getScore();
                        paperStructDtoNewList.add(paperStructDto);
                    }
                }
                if (totalScore.doubleValue() < 100d) {
                    throw ExceptionResultEnum.ERROR.exception("试卷满分小于100分");
                }
                if (!CollectionUtils.isEmpty(paperStructDtoNewList)) {
                    successData.add("共导入").add(paperStructDtoNewList.size() + "").add("条数据");
                    TCPaperStruct tcPaperStruct = new TCPaperStruct(examId, courseCode, markPaper.getCourseName(), paperNumber, JacksonUtil.parseJson(paperStructDtoNewList), totalScore, 60d, sysUser.getId());
                    TCPaperStruct tcPaperStructDb = tcPaperStructService.queryByExamIdAndCourseCodeAndPaperNumber(examId, courseCode, paperNumber);
                    if (Objects.nonNull(tcPaperStructDb) && !tcPaperStructDb.equals(tcPaperStruct)) {
                        tcFinalScoreService.remove(examId, courseCode, paperNumber);
                        trBasicInfoService.clearReportData(examId, courseCode, paperNumber, false);
                    }
                    tcPaperStructService.remove(examId, courseCode, paperNumber);
                    tcPaperStructService.save(tcPaperStruct);
                }
            }
            messageMap.put(SystemConstant.SUCCESS, successData.length() > 0 ? successData.toString() : "无");
            messageMap.put(SystemConstant.EXCEL_ERROR, errorData.length() > 0 ? errorData.toString() : "无");
        } catch (
                Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof DuplicateKeyException) {
                String errorColumn = e.getCause().toString();
                String columnStr = errorColumn.substring(errorColumn.lastIndexOf("key") + 3, errorColumn.length()).replaceAll("'", "");
                throw ExceptionResultEnum.SQL_ERROR.exception("[" + FieldUniqueEnum.convertToTitle(columnStr) + "]数据不允许重复插入");
            } else if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, ((ApiException) e).getCode(), e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        log.debug("导入Excel结束...");
        long end = System.currentTimeMillis();
        log.info("============耗时{}秒============:", (end - start) / 1000);
        return messageMap;
    }

    /**
     * 根据考试id/科目编辑/试卷编码查询数据
     *
     * @param examId
     * @param courseCode
     * @param paperNumber
     * @return
     */
    @Override
    public TCPaperStruct queryByExamIdAndCourseCodeAndPaperNumber(Long examId, String courseCode, String paperNumber) {
        return tcPaperStructService.getOne(new QueryWrapper<TCPaperStruct>().lambda().eq(TCPaperStruct::getExamId, examId).eq(TCPaperStruct::getCourseCode, courseCode).eq(TCPaperStruct::getPaperNumber, paperNumber));
    }
}
