package com.qmth.teachcloud.mark.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.common.bean.dto.mark.MarkUser;
import com.qmth.teachcloud.common.bean.params.mark.setting.MarkClassUserParams;
import com.qmth.teachcloud.common.bean.params.mark.setting.MarkGroupClassUserParams;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkGroupClassProgressDto;
import com.qmth.teachcloud.mark.dto.mark.setting.MarkGroupDto;
import com.qmth.teachcloud.mark.dto.mark.setting.MarkUserClassDetailDto;
import com.qmth.teachcloud.mark.dto.mark.setting.MarkUserClassDto;
import com.qmth.teachcloud.mark.entity.MarkGroup;
import com.qmth.teachcloud.mark.entity.MarkUserClass;
import com.qmth.teachcloud.mark.mapper.MarkUserClassMapper;
import com.qmth.teachcloud.mark.service.*;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 评卷员班级 服务实现类
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Service
public class MarkUserClassServiceImpl extends ServiceImpl<MarkUserClassMapper, MarkUserClass> implements MarkUserClassService {
    @Resource
    MarkUserGroupService markUserGroupService;

    @Resource
    MarkStudentService markStudentService;

    @Resource
    MarkGroupService markGroupService;

    @Resource
    MarkQuestionService markQuestionService;

    @Override
    public void deleteByExamIdAndPaperNumber(Long examId, String paperNumber) {
        UpdateWrapper<MarkUserClass> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(MarkUserClass::getExamId, examId)
                .eq(MarkUserClass::getPaperNumber, paperNumber);
        this.remove(updateWrapper);
    }

    @Override
    public MarkUserClassDto listMarkUserClassByExamIdAndPaperNumber(Long examId, String paperNumber) {
        // 课程下所有班级数据
        List<String> paperNumberClassList = markStudentService.listClassByExamIdAndCourseCode(examId, paperNumber);
        // 课程下所有分组
        List<MarkGroup> markGroupList = markGroupService.listGroupByExamIdAndPaperNumber(examId, paperNumber);
        List<MarkGroupDto> markGroupDtoList = markGroupList.stream().map(m -> {
            MarkGroupDto markGroupDto = new MarkGroupDto();
            markGroupDto.setGroupNumber(m.getNumber());
            markGroupDto.setMarkers(markUserGroupService.listGroupUserByExamIdAndPaperNumberAndGroupNumberAndClassName(examId, paperNumber, m.getNumber(), null));
            markGroupDto.setGroupQuestions(markQuestionService.assembleGroupQuestionsByExamIdAndPaperNumberAndNumber(examId, paperNumber, m.getNumber()));
            return markGroupDto;
        }).collect(Collectors.toList());

        MarkUserClassDto markUserClassDto = new MarkUserClassDto();
        markUserClassDto.setClassNames(paperNumberClassList);

        List<MarkUserClassDetailDto> markUserClassDetailDtos = new ArrayList<>();
        for (MarkGroupDto markGroupDto : markGroupDtoList) {
            for (MarkUser marker : markGroupDto.getMarkers()) {
                MarkUserClassDetailDto markUserClassDetailDto = new MarkUserClassDetailDto();
                markUserClassDetailDto.setGroupNumber(markGroupDto.getGroupNumber());
                markUserClassDetailDto.setGroupQuestions(markGroupDto.getGroupQuestions());
                markUserClassDetailDto.setMarker(marker);
                List<MarkUserClass> markUserClasses = this.baseMapper.listMarkerClassByExamIdAndPaperNumberAndGroupNumberAndUserId(examId, paperNumber, markGroupDto.getGroupNumber(), marker.getUserId());
                markUserClassDetailDto.setMarkerClassList(markUserClasses.stream().map(MarkUserClass::getClassName).collect(Collectors.toList()));
                markUserClassDetailDtos.add(markUserClassDetailDto);
            }
        }
        markUserClassDto.setMarkerClass(markUserClassDetailDtos);
        return markUserClassDto;
    }

    @Override
    public List<MarkUser> listClassMarkerByExamIdAndPaperNumberAndGroupNumberAndClassName(Long examId, String paperNumber, Integer groupNumber, String className) {
        return this.baseMapper.listClassMarkerByExamIdAndPaperNumberAndGroupNumberAndClassName(examId, paperNumber, groupNumber, className);
    }

    @Transactional
    @Override
    public boolean saveMarkClassUser(MarkClassUserParams markClassUserParams) {
        Long examId = markClassUserParams.getExamId();
        String paperNumber = markClassUserParams.getPaperNumber();

        // 校验分组、班级是否绑定完成
        List<String> listUserClass = markStudentService.listClassByExamIdAndCourseCode(examId, paperNumber);
        if (CollectionUtils.isNotEmpty(listUserClass)) {
            Map<Integer, Set<String>> groupBindClassMap = new HashMap<>();
            for (MarkGroupClassUserParams groupUserClassInfo : markClassUserParams.getGroupMarkerClass()) {
                Integer groupNumber = groupUserClassInfo.getGroupNumber();
                if (!groupBindClassMap.containsKey(groupNumber)) {
                    groupBindClassMap.put(groupNumber, new HashSet<>());
                }
                groupBindClassMap.get(groupNumber).addAll(groupUserClassInfo.getMarkerClassList());
            }

            if (!groupBindClassMap.isEmpty()) {
                StringJoiner stringJoiner = new StringJoiner("；");
                for (Map.Entry<Integer, Set<String>> entry : groupBindClassMap.entrySet()) {
                    List<String> groupAllClass = listUserClass;
                    groupAllClass.removeAll(entry.getValue());
                    if (CollectionUtils.isNotEmpty(groupAllClass)) {
                        stringJoiner.add(String.format("班级[%s]在分组[%s]中未被任何评卷员绑定", String.join(",", entry.getValue()), entry.getKey()));
                    }
                }
                if (stringJoiner.toString().length() > 0) {
                    throw ExceptionResultEnum.ERROR.exception(stringJoiner.toString());
                }
            }
        }

        List<MarkUserClass> markUserClasses = new ArrayList<>();
        for (MarkGroupClassUserParams groupUserClassInfo : markClassUserParams.getGroupMarkerClass()) {
            for (String className : groupUserClassInfo.getMarkerClassList()) {
                MarkUserClass markUserClass = new MarkUserClass();
                markUserClass.setId(SystemConstant.getDbUuid());
                markUserClass.setExamId(examId);
                markUserClass.setPaperNumber(paperNumber);
                markUserClass.setGroupNumber(groupUserClassInfo.getGroupNumber());
                markUserClass.setClassName(className);
                markUserClass.setUserId(groupUserClassInfo.getMarker().getUserId());
                markUserClasses.add(markUserClass);
            }
        }
        // 科目下所有数据同时提交
        this.deleteByExamIdAndPaperNumberAndGroupNumberAndClassName(examId, paperNumber, null, null);
        this.saveBatch(markUserClasses);
        return true;
    }

    @Override
    public boolean deleteByExamIdAndPaperNumberAndGroupNumberAndClassName(Long examId, String paperNumber, Integer groupNumber, String className) {
        UpdateWrapper<MarkUserClass> updateWrapper = new UpdateWrapper<>();
        LambdaUpdateWrapper<MarkUserClass> lambdaUpdateWrapper = updateWrapper.lambda();
        lambdaUpdateWrapper.eq(MarkUserClass::getExamId, examId)
                .eq(MarkUserClass::getPaperNumber, paperNumber);
        if (groupNumber != null) {
            lambdaUpdateWrapper.eq(MarkUserClass::getGroupNumber, groupNumber);
        }
        if (StringUtils.isNotBlank(className)) {
            lambdaUpdateWrapper.eq(MarkUserClass::getClassName, className);
        }
        return this.remove(updateWrapper);
    }

    @Override
    public List<MarkUserClass> listByExamIdAndPaperNumber(Long examId, String paperNumber) {
        QueryWrapper<MarkUserClass> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkUserClass::getExamId, examId)
                .eq(MarkUserClass::getPaperNumber, paperNumber);
        return this.list(queryWrapper);
    }

    @Override
    public IPage<MarkGroupClassProgressDto> pageClassByExamIdAndPaperNumber(Page<MarkGroupClassProgressDto> page, Long examId, String paperNumber) {
        return this.baseMapper.pageClassByExamIdAndPaperNumber(page, examId, paperNumber);
    }
}
