﻿USE teachreport_db;
-- ----------------------------
-- Table structure for basic_attachment
-- ----------------------------
DROP TABLE IF EXISTS `basic_attachment`;
CREATE TABLE `basic_attachment` (
                                    `id` bigint NOT NULL,
                                    `name` varchar(100) NOT NULL COMMENT '文件名',
                                    `type` varchar(10) NOT NULL COMMENT '文件类型',
                                    `size` int NOT NULL COMMENT '单位（KB）',
                                    `md5` varchar(32) NOT NULL COMMENT '文件md5值',
                                    `path` varchar(2000) NOT NULL COMMENT '存储路径',
                                    `create_id` bigint DEFAULT NULL COMMENT '创建人',
                                    `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                    `obj_id` bigint DEFAULT NULL COMMENT '关联业务id',
                                    PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='所有附件记录表';

-- ----------------------------
-- Table structure for basic_campus
-- ----------------------------
DROP TABLE IF EXISTS `basic_campus`;
CREATE TABLE `basic_campus` (
                                `id` bigint NOT NULL COMMENT '校区id',
                                `school_id` bigint NOT NULL COMMENT '学校id',
                                `campus_name` varchar(64) DEFAULT NULL COMMENT '校区名称',
                                `campus_code` varchar(64) DEFAULT NULL COMMENT '校区编码',
                                `print_house_id` bigint DEFAULT NULL COMMENT '印刷室id',
                                `enable` tinyint DEFAULT '1' COMMENT '是否可用',
                                `org_id` bigint DEFAULT NULL COMMENT '机构id',
                                `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                PRIMARY KEY (`id`) USING BTREE,
                                UNIQUE KEY `campus_schoolId_code_idx` (`school_id`,`campus_code`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='基础校区表';

-- ----------------------------
-- Table structure for basic_course
-- ----------------------------
DROP TABLE IF EXISTS `basic_course`;
CREATE TABLE `basic_course` (
                                `id` bigint NOT NULL COMMENT '课程主键',
                                `school_id` bigint NOT NULL COMMENT '学校id',
                                `code` varchar(100) DEFAULT NULL COMMENT '课程编号',
                                `name` varchar(100) DEFAULT NULL COMMENT '课程名称',
                                `create_id` bigint DEFAULT NULL,
                                `create_time` bigint DEFAULT NULL,
                                `update_id` bigint DEFAULT NULL,
                                `update_time` bigint DEFAULT NULL,
                                `historic_name` varchar(100) DEFAULT NULL COMMENT '历史课程名称',
                                `org_id` bigint DEFAULT NULL COMMENT '机构id',
                                `teaching_room_id` bigint DEFAULT NULL COMMENT '教研室id',
                                `enable` tinyint DEFAULT NULL,
                                `short_code` varchar(10) DEFAULT NULL COMMENT '简码',
                                PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='课程基础表';

-- ----------------------------
-- Table structure for basic_school
-- ----------------------------
DROP TABLE IF EXISTS `basic_school`;
CREATE TABLE `basic_school` (
                                `id` bigint NOT NULL,
                                `code` varchar(100) DEFAULT NULL COMMENT '学校编号',
                                `domain_name` varchar(20) DEFAULT NULL COMMENT '域名',
                                `name` varchar(100) DEFAULT NULL COMMENT '学校名称',
                                `enable` tinyint DEFAULT '1' COMMENT '是否启用，false：停用，true：启用',
                                `access_key` varchar(64) DEFAULT NULL COMMENT '访问key',
                                `access_secret` varchar(64) DEFAULT NULL COMMENT '访问secret',
                                `remark` text COMMENT '备注',
                                `create_id` bigint DEFAULT NULL COMMENT '创建人',
                                `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                `update_id` bigint DEFAULT NULL COMMENT '更新人',
                                `logo` mediumtext DEFAULT NULL COMMENT 'logo',
                                `init_password` varchar(100) DEFAULT NULL COMMENT '该学校用户的初始密码，默认为12345678',
                                `default_school` TINYINT(1) NULL DEFAULT 0 COMMENT '默认学校',
                                PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='学校基础表';

-- ----------------------------
-- Table structure for basic_semester
-- ----------------------------
DROP TABLE IF EXISTS `basic_semester`;
CREATE TABLE `basic_semester` (
                                  `id` bigint NOT NULL COMMENT '主键',
                                  `school_id` bigint NOT NULL COMMENT '学校id',
                                  `name` varchar(100) NOT NULL COMMENT '学期名称',
                                  `code` varchar(100) DEFAULT NULL COMMENT '学期编码',
                                  `start_time` bigint DEFAULT NULL COMMENT '开始日期',
                                  `end_time` bigint DEFAULT NULL COMMENT '结束日期',
                                  `org_id` bigint DEFAULT NULL COMMENT '机构id',
                                  `enable` tinyint DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                  `in_used` tinyint DEFAULT NULL COMMENT '是否当前使用学期',
                                  PRIMARY KEY (`id`) USING BTREE,
                                  UNIQUE KEY `semester_schoolId_name_idx` (`school_id`,`name`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='学期字典表';

-- ----------------------------
-- Table structure for basic_user_course
-- ----------------------------
DROP TABLE IF EXISTS `basic_user_course`;
CREATE TABLE `basic_user_course` (
                                     `id` bigint NOT NULL,
                                     `user_id` bigint DEFAULT NULL COMMENT '用户ID',
                                     `course_id` bigint DEFAULT NULL COMMENT '课程ID',
                                     `create_id` bigint DEFAULT NULL,
                                     `create_time` bigint DEFAULT NULL,
                                     `update_id` bigint DEFAULT NULL,
                                     `update_time` bigint DEFAULT NULL,
                                     PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='用户和课程关联表';

-- ----------------------------
-- Table structure for basic_verify_code
-- ----------------------------
DROP TABLE IF EXISTS `basic_verify_code`;
CREATE TABLE `basic_verify_code` (
                                     `id` bigint NOT NULL,
                                     `user_id` bigint NOT NULL COMMENT '用户ID',
                                     `mobile_number` varchar(11) NOT NULL COMMENT '手机号',
                                     `expire_time` bigint NOT NULL COMMENT '到期时间',
                                     `valid_period` int NOT NULL COMMENT '单位（分）',
                                     `verify_code` varchar(4) NOT NULL COMMENT '4位数字',
                                     `create_id` bigint DEFAULT NULL,
                                     `create_time` bigint DEFAULT NULL,
                                     `update_id` bigint DEFAULT NULL,
                                     `update_time` bigint DEFAULT NULL,
                                     `school_id` bigint DEFAULT NULL COMMENT '学校id',
                                     `org_id` bigint DEFAULT NULL COMMENT '机构id',
                                     PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='短信验证码记录表';

-- ----------------------------
-- Table structure for grade_batch_student
-- ----------------------------
DROP TABLE IF EXISTS `grade_batch_student`;
CREATE TABLE `grade_batch_student` (
                                       `id` bigint NOT NULL,
                                       `batch_id` bigint DEFAULT NULL COMMENT '分析-批次表ID',
                                       `course_code` varchar(50) DEFAULT NULL COMMENT '课程编码',
                                       `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
                                       `paper_number` varchar(50) DEFAULT NULL COMMENT '试卷编号',
                                       `paper_type` varchar(16) DEFAULT NULL COMMENT '试卷类型',
                                       `student_code` varchar(45) DEFAULT NULL COMMENT '学号',
                                       `student_name` varchar(100) DEFAULT NULL COMMENT '学生姓名',
                                       `ticket_number` varchar(45) DEFAULT NULL COMMENT '准考证号',
                                       `teacher_name` varchar(100) DEFAULT NULL COMMENT '任课老师',
                                       `teacher_number` varchar(45) DEFAULT NULL COMMENT '任课老师工号',
                                       `create_id` bigint DEFAULT NULL,
                                       `create_time` bigint DEFAULT NULL,
                                       `update_id` bigint DEFAULT NULL,
                                       `update_time` bigint DEFAULT NULL,
                                       PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考务数据（包含任课老师）';

-- ----------------------------
-- Table structure for sys_config
-- ----------------------------
DROP TABLE IF EXISTS `sys_config`;
CREATE TABLE `sys_config` (
                              `id` bigint NOT NULL,
                              `school_id` bigint DEFAULT NULL COMMENT '学校id',
                              `org_id` bigint DEFAULT NULL COMMENT '机构id',
                              `config_key` varchar(100) NOT NULL COMMENT '参数键名',
                              `config_name` varchar(100) NOT NULL COMMENT '参数名称',
                              `config_value` varchar(100) NOT NULL COMMENT '参数键值',
                              `remark` varchar(100) DEFAULT NULL,
                              `enable` tinyint DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                              `sort` int DEFAULT '1' COMMENT '排序',
                              `create_id` bigint DEFAULT NULL,
                              `create_time` bigint DEFAULT NULL,
                              `update_id` bigint DEFAULT NULL,
                              `update_time` bigint DEFAULT NULL,
                              PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='参数表';

-- ----------------------------
-- Table structure for sys_org
-- ----------------------------
DROP TABLE IF EXISTS `sys_org`;
CREATE TABLE `sys_org` (
                           `id` bigint NOT NULL,
                           `school_id` bigint NOT NULL COMMENT '学校ID',
                           `type` varchar(50) NOT NULL DEFAULT 'COLLEGE' COMMENT '类型，SCHOOL：学校，COLLEGE：学院，FACULTY：院系，TEACHING_ROOM：教研室，PRINTING_HOUSE：印刷厂',
                           `code` varchar(100) DEFAULT NULL COMMENT '机构代码',
                           `name` varchar(100) NOT NULL COMMENT '机构名称',
                           `parent_id` bigint DEFAULT NULL COMMENT '上级机构ID',
                           `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，false：停用，true：启用',
                           `create_id` bigint DEFAULT NULL COMMENT '创建人',
                           `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                           `update_id` bigint DEFAULT NULL COMMENT '修改人',
                           `update_time` bigint DEFAULT NULL COMMENT '修改时间',
                           `historic_name` varchar(100) DEFAULT NULL COMMENT '历史机构名称',
                           `third_relate_id` int DEFAULT NULL COMMENT '云阅卷机构ID',
                           PRIMARY KEY (`id`) USING BTREE,
                           UNIQUE KEY `org_schoolId_code_idx` (`school_id`,`code`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='学校组织架构';

-- ----------------------------
-- Table structure for sys_privilege
-- ----------------------------
DROP TABLE IF EXISTS `sys_privilege`;
CREATE TABLE `sys_privilege` (
                                 `id` bigint NOT NULL,
                                 `name` varchar(100) NOT NULL COMMENT '菜单名称',
                                 `url` varchar(100) NOT NULL COMMENT 'URL地址',
                                 `type` varchar(50) NOT NULL COMMENT '类型，MENU：菜单，BUTTON：按钮，LINK：链接，URL：接口地址，LIST：列表，CONDITION：查询条件',
                                 `parent_id` bigint DEFAULT NULL COMMENT '上级菜单',
                                 `sequence` int NOT NULL COMMENT '序号',
                                 `property` varchar(30) DEFAULT NULL COMMENT '属性，NO_AUTH：无需鉴权，AUTH：鉴权，SYS：系统公用',
                                 `related` text COMMENT '关联属性，type为URL且property为AUTH才有',
                                 `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                 `default_auth` tinyint DEFAULT '1' COMMENT '默认权限，0：否，1：是',
                                 `front_display` tinyint DEFAULT '1' COMMENT '是否前端展示',
                                 PRIMARY KEY (`id`) USING BTREE,
                                 UNIQUE KEY `privilege_index` (`name`,`url`,`type`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='菜单权限表';

-- ----------------------------
-- Table structure for sys_role
-- ----------------------------
DROP TABLE IF EXISTS `sys_role`;
CREATE TABLE `sys_role` (
                            `id` bigint NOT NULL,
                            `school_id` bigint DEFAULT NULL COMMENT '学校ID',
                            `name` varchar(50) NOT NULL COMMENT '角色名称',
                            `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，false：停用，true：启用',
                            `type` varchar(20) DEFAULT NULL COMMENT '角色类别，ADMIN：超级管理员，SCHOOL_ADMIN：管理员：EXAM_TEACHER：考务老师，QUESTION_TEACHER：命题老师，CUSTOMER：客服人员，PRINTER：印刷人员，CUSTOM：自定义',
                            `create_id` bigint DEFAULT NULL COMMENT '创建人',
                            `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                            `update_id` bigint DEFAULT NULL COMMENT '修改人',
                            `update_time` bigint DEFAULT NULL COMMENT '修改时间',
                            `default_role` tinyint NOT NULL DEFAULT '0' COMMENT '是否系统内置角色，1：是，0：不是',
                            `org_id` bigint DEFAULT NULL COMMENT '机构id',
                            `source` varchar(100) DEFAULT NULL COMMENT '角色来源，ANALYSIS：教研分析',
                            `interpret` varchar(1000) DEFAULT NULL COMMENT '角色说明',
                            PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='角色表';

-- ----------------------------
-- Table structure for sys_role_privilege
-- ----------------------------
DROP TABLE IF EXISTS `sys_role_privilege`;
CREATE TABLE `sys_role_privilege` (
                                      `id` bigint NOT NULL AUTO_INCREMENT,
                                      `role_id` bigint NOT NULL COMMENT '角色ID',
                                      `privilege_id` bigint NOT NULL COMMENT '菜单ID',
                                      `enable` tinyint DEFAULT '1' COMMENT '是否禁用，false：禁用，true：启用',
                                      `school_id` bigint DEFAULT NULL COMMENT '内置角色，按学校自定义的权限',
                                      PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='角色和菜单关联表';

-- ----------------------------
-- Table structure for sys_user
-- ----------------------------
DROP TABLE IF EXISTS `sys_user`;
CREATE TABLE `sys_user` (
                            `id` bigint NOT NULL,
                            `school_id` bigint DEFAULT NULL COMMENT '学校ID',
                            `login_name` varchar(100) NOT NULL COMMENT '用户名',
                            `real_name` varchar(100) NOT NULL COMMENT '姓名',
                            `password` varchar(50) NOT NULL COMMENT '密码',
                            `mobile_number` varchar(30) DEFAULT NULL COMMENT '手机号',
                            `org_id` bigint DEFAULT NULL,
                            `enable` tinyint NOT NULL DEFAULT '1' COMMENT 'false-禁用，true-启用',
                            `pwd_update_time` bigint DEFAULT NULL COMMENT '密码修改时间',
                            `remark` varchar(200) DEFAULT NULL COMMENT '备注',
                            `create_id` bigint DEFAULT NULL COMMENT '创建人',
                            `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                            `update_id` bigint DEFAULT NULL,
                            `update_time` bigint DEFAULT NULL,
                            `code` varchar(100) DEFAULT NULL COMMENT '工号',
                            `pwd_count` tinyint DEFAULT '0' COMMENT '密码修改次数，默认为0',
                            `sync_status` varchar(100) DEFAULT NULL COMMENT '同步状态，NEED_PUSH_AGAIN("需要重新推送"),',
                            `historic_name` varchar(100) DEFAULT NULL COMMENT '历史用户名称',
                            `ui_setting` text COMMENT '阅卷参数',
							`allow_manual_card` tinyint(1) DEFAULT '0' COMMENT '是否允许新增卡格式',
                            PRIMARY KEY (`id`) USING BTREE,
                            UNIQUE KEY `user_schoolId_loginName_idx` (`school_id`,`login_name`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='用户表';

-- ----------------------------
-- Table structure for sys_user_role
-- ----------------------------
DROP TABLE IF EXISTS `sys_user_role`;
CREATE TABLE `sys_user_role` (
                                 `id` bigint NOT NULL AUTO_INCREMENT,
                                 `user_id` bigint NOT NULL COMMENT '用户ID',
                                 `role_id` bigint NOT NULL COMMENT '角色ID',
                                 `privilege_id` bigint DEFAULT NULL COMMENT '权限id',
                                 `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否禁用，false：禁用，true：启用',
                                 PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='用户和角色关联表';

-- ----------------------------
-- Table structure for t_a_exam_course
-- ----------------------------
DROP TABLE IF EXISTS `t_a_exam_course`;
CREATE TABLE `t_a_exam_course` (
                                   `id` bigint NOT NULL COMMENT '考试科目分析表id',
                                   `exam_id` bigint NOT NULL COMMENT '考试id',
                                   `exam_name` varchar(100) NOT NULL COMMENT '考试名称（冗余）',
                                   `course_code` varchar(100) NOT NULL COMMENT '科目编码',
                                   `course_name` varchar(100) NOT NULL COMMENT '科目名称',
                                   `school_id` bigint NOT NULL COMMENT '学校id（冗余）',
                                   `teach_college_id` bigint DEFAULT NULL COMMENT '开课学院id',
                                   `teach_college_name` varchar(64) DEFAULT NULL COMMENT '开课学院名称',
                                   `inspect_college_count` int DEFAULT NULL COMMENT '考察学院数量',
                                   `inspect_college_names` text COMMENT '考察学院名称',
                                   `avg_score` decimal(12,4) DEFAULT NULL COMMENT '总体平均分',
                                   `pass_count` int DEFAULT NULL COMMENT '通过人数',
                                   `pass_rate` decimal(12,4) DEFAULT NULL COMMENT '通过率',
                                   `current_pass_count` int DEFAULT NULL COMMENT '应届通过人数',
                                   `current_pass_rate` decimal(12,4) DEFAULT NULL COMMENT '应届通过率',
                                   `total_score` decimal(12,4) DEFAULT NULL COMMENT '试卷满分',
                                   `reality_count` int DEFAULT NULL COMMENT '实际参考人数',
                                   `absent_count` int DEFAULT NULL COMMENT '缺考人数',
                                   `total_count` int DEFAULT NULL COMMENT '总应参考人数',
                                   `current_min_score` decimal(12,4) DEFAULT NULL COMMENT '最低分',
                                   `current_max_score` decimal(12,4) DEFAULT NULL COMMENT '最高分',
                                   `current_avg_score` decimal(12,4) DEFAULT NULL COMMENT '全体参考平均分',
                                   `current_reality_count` int DEFAULT NULL COMMENT '应届实际考试人数',
                                   `current_absent_count` int DEFAULT NULL COMMENT '应届缺考人数',
                                   `current_total_count` int DEFAULT NULL COMMENT '应届总考试人数',
                                   `current_reality_rate` decimal(12,4) DEFAULT NULL COMMENT '应届实际考试人数占比',
                                   `past_reality_count` int DEFAULT NULL COMMENT '往届实际考试人数',
                                   `past_absent_count` int DEFAULT NULL COMMENT '往届缺考人数',
                                   `past_total_count` int DEFAULT NULL COMMENT '往届总人数',
                                   `past_reality_rate` decimal(12,4) DEFAULT NULL COMMENT '往届实际考试人数占比',
                                   `current_upper_quartile` decimal(12,4) DEFAULT NULL COMMENT '上四分位数',
                                   `current_median` decimal(12,4) DEFAULT NULL COMMENT '中位数',
                                   `current_lower_quartile` decimal(12,4) DEFAULT NULL COMMENT '下四分位数',
                                   `current_mode` varchar(64) DEFAULT NULL COMMENT '众数',
                                   `current_standard_deviation` decimal(12,4) DEFAULT NULL COMMENT '标准差',
                                   `current_score_rate` decimal(12,4) DEFAULT NULL COMMENT '难度系数（得分率）',
                                   `difficulty` varchar(16) DEFAULT NULL COMMENT '本次科目难度（应届）',
                                   `paper_avg_score` decimal(12,4) DEFAULT NULL COMMENT '总体卷面平均分',
                                   `paper_current_avg_score` decimal(12,4) DEFAULT NULL COMMENT '应届卷面平均分',
                                   `paper_pass_rate` decimal(12,4) DEFAULT NULL COMMENT '总体卷面成绩通过率',
                                   `paper_current_pass_rate` decimal(12,4) DEFAULT NULL COMMENT '应届卷面成绩通过率',
                                   `paper_pass_count` int DEFAULT NULL COMMENT '卷面总体通过人数',
                                   `paper_current_pass_count` int DEFAULT NULL COMMENT '卷面应届通过人数',
                                   `coefficient` varchar(100) DEFAULT NULL COMMENT '赋分系数',
                                   PRIMARY KEY (`id`) USING BTREE,
                                   KEY `t_a_exam_course_idx` (`exam_id`,`course_code`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考试课程维度分析表';

-- ----------------------------
-- Table structure for t_a_exam_course_clazz
-- ----------------------------
DROP TABLE IF EXISTS `t_a_exam_course_clazz`;
CREATE TABLE `t_a_exam_course_clazz` (
                                         `id` bigint NOT NULL COMMENT 'id',
                                         `exam_id` bigint NOT NULL COMMENT '考试id',
                                         `school_id` bigint DEFAULT NULL,
                                         `course_code` varchar(100) NOT NULL COMMENT '科目编码',
                                         `course_name` varchar(100) NOT NULL COMMENT '科目名称',
                                         `clazz_id` bigint NOT NULL COMMENT '班级表id',
                                         `max_score` decimal(12,4) DEFAULT NULL COMMENT '最高分（赋分）',
                                         `min_score` decimal(12,4) DEFAULT NULL COMMENT '最低分（赋分）',
                                         `avg_score` decimal(12,4) DEFAULT NULL COMMENT '平均分（赋分）',
                                         `reality_count` int DEFAULT NULL COMMENT '实际参考人数',
                                         `absent_count` int DEFAULT NULL COMMENT '缺考人数',
                                         `total_count` int DEFAULT NULL COMMENT '总人数',
                                         `college_name` varchar(100) NOT NULL COMMENT '所属考察学院名称',
                                         PRIMARY KEY (`id`) USING BTREE,
                                         KEY `t_a_exam_course_clazz_idx` (`exam_id`,`course_code`,`clazz_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='参考课程班级维度分析表';

-- ----------------------------
-- Table structure for t_a_exam_course_college_inspect
-- ----------------------------
DROP TABLE IF EXISTS `t_a_exam_course_college_inspect`;
CREATE TABLE `t_a_exam_course_college_inspect` (
                                                   `id` bigint NOT NULL,
                                                   `exam_id` bigint NOT NULL COMMENT '考试id',
                                                   `school_id` bigint DEFAULT NULL,
                                                   `course_code` varchar(100) NOT NULL COMMENT '科目编码',
                                                   `course_name` varchar(100) NOT NULL COMMENT '科目名称',
                                                   `college_id` bigint NOT NULL COMMENT '学院id',
                                                   `max_score` decimal(12,4) DEFAULT NULL COMMENT '最高分',
                                                   `min_score` decimal(12,4) DEFAULT NULL COMMENT '最低分',
                                                   `avg_score` decimal(12,4) DEFAULT NULL COMMENT '平均分',
                                                   `reality_count` int DEFAULT NULL COMMENT '实际参考人数',
                                                   `absent_count` int DEFAULT NULL COMMENT '缺考人数',
                                                   `total_count` int DEFAULT NULL COMMENT '总人数',
                                                   `upper_quartile` decimal(12,4) DEFAULT NULL COMMENT '上四分位数',
                                                   `median` decimal(12,4) DEFAULT NULL COMMENT '中位数',
                                                   `lower_quartile` decimal(12,4) DEFAULT NULL COMMENT '下四分位数',
                                                   `mode` varchar(255) DEFAULT NULL COMMENT '众数',
                                                   `standard_deviation` decimal(12,4) DEFAULT NULL COMMENT '标准差',
                                                   `relative_position` decimal(12,4) DEFAULT NULL COMMENT '相对位置',
                                                   PRIMARY KEY (`id`) USING BTREE,
                                                   KEY `t_a_exam_course_college_idx` (`exam_id`,`course_code`,`college_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='参考课程考察学院维度分析表';

-- ----------------------------
-- Table structure for t_a_exam_course_college_inspect_dio
-- ----------------------------
DROP TABLE IF EXISTS `t_a_exam_course_college_inspect_dio`;
CREATE TABLE `t_a_exam_course_college_inspect_dio` (
                                                       `id` bigint NOT NULL COMMENT 'id',
                                                       `exam_id` bigint NOT NULL COMMENT '考试id',
                                                       `school_id` bigint DEFAULT NULL,
                                                       `course_code` varchar(100) NOT NULL COMMENT '科目编码',
                                                       `course_name` varchar(100) DEFAULT NULL COMMENT '科目名称',
                                                       `inspect_college_id` bigint NOT NULL COMMENT '考察学院id',
                                                       `inspect_college_name` varchar(32) DEFAULT NULL COMMENT '考察学院名称',
                                                       `dimension_type` varchar(32) DEFAULT NULL COMMENT '模块类型',
                                                       `dimension_code` varchar(32) DEFAULT NULL COMMENT '考察点编号',
                                                       `avg_score` decimal(12,4) DEFAULT NULL COMMENT '考察点学院学生平均得分',
                                                       `score_rate` decimal(12,4) DEFAULT NULL COMMENT '该考察点学院平均得分率',
                                                       PRIMARY KEY (`id`) USING BTREE,
                                                       KEY `t_a_exam_course_college_inspect_dio_idx` (`school_id`,`exam_id`,`course_code`,`dimension_type`,`dimension_code`,`inspect_college_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='参考课程考察学院各考察点维度表';

-- ----------------------------
-- Table structure for t_a_exam_course_college_paper_struct
-- ----------------------------
DROP TABLE IF EXISTS `t_a_exam_course_college_paper_struct`;
CREATE TABLE `t_a_exam_course_college_paper_struct` (
                                                        `id` bigint NOT NULL COMMENT '主键',
                                                        `school_id` bigint NOT NULL COMMENT '学校id',
                                                        `paper_struct_id` bigint NOT NULL COMMENT '试卷结构基础表id',
                                                        `exam_id` bigint NOT NULL COMMENT '考试id',
                                                        `course_code` varchar(32) NOT NULL COMMENT '课程编号',
                                                        `course_name` varchar(32) DEFAULT NULL COMMENT '课程名称',
                                                        `paper_id` bigint NOT NULL COMMENT '试卷id',
                                                        `inspect_college_id` bigint NOT NULL COMMENT '考察学院id',
                                                        `inspect_college_name` varchar(64) DEFAULT NULL COMMENT '考察学院名称',
                                                        `paper_type` varchar(64) DEFAULT NULL COMMENT '试卷类型',
                                                        `question_name` varchar(64) DEFAULT NULL COMMENT '大题名称',
                                                        `number_type` varchar(64) DEFAULT NULL COMMENT '题号类型（客观题、主观题）',
                                                        `big_question_number` varchar(64) DEFAULT NULL COMMENT '大题号',
                                                        `small_question_number` varchar(64) DEFAULT NULL COMMENT '小题号(每一课程试卷的唯一标识)',
                                                        `question_type` varchar(64) DEFAULT NULL COMMENT '题目类型',
                                                        `full_score` decimal(12,4) DEFAULT NULL COMMENT '满分',
                                                        `score_rules` varchar(32) DEFAULT NULL COMMENT '计分规则',
                                                        `rules_desc` varchar(500) DEFAULT NULL COMMENT '规则说明',
                                                        `knowledge_dimension` varchar(255) DEFAULT NULL COMMENT '知识维度',
                                                        `ability_dimension` varchar(255) DEFAULT NULL COMMENT '能力维度',
                                                        `literacy_dimension` varchar(255) DEFAULT NULL COMMENT '素养维度',
                                                        `standard_score_rate` decimal(12,4) DEFAULT NULL COMMENT '标准难度系数（得分率）',
                                                        `score_rate` decimal(12,4) DEFAULT NULL COMMENT '得分率',
                                                        `difficult` varchar(32) DEFAULT NULL COMMENT '难度',
                                                        `paper_struct_judge` varchar(32) DEFAULT NULL COMMENT '试卷结构每道题目正确（全对），错误枚举\\nALL_CORRECT("全对"), NOT_QUITE_RIGHT("不完全正确的"),',
                                                        PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='教师考察学院下试卷结构维度分析表';

-- ----------------------------
-- Table structure for t_a_exam_course_college_teacher
-- ----------------------------
DROP TABLE IF EXISTS `t_a_exam_course_college_teacher`;
CREATE TABLE `t_a_exam_course_college_teacher` (
                                                   `id` bigint NOT NULL,
                                                   `exam_id` bigint NOT NULL COMMENT '考试id',
                                                   `school_id` bigint DEFAULT NULL,
                                                   `course_code` varchar(100) NOT NULL COMMENT '科目编码',
                                                   `course_name` varchar(100) DEFAULT NULL COMMENT '科目名称',
                                                   `teacher_id` bigint NOT NULL COMMENT '授课教师id',
                                                   `inspect_college_id` bigint NOT NULL COMMENT '考察学院id（此表以该字段为统计）',
                                                   `inspect_college_name` varchar(100) DEFAULT NULL COMMENT '考察学院名称（冗余）',
                                                   `teacher_name` varchar(100) DEFAULT NULL COMMENT '授课教师名称（冗余）',
                                                   `avg_score` decimal(12,4) DEFAULT NULL COMMENT '卷面平均分',
                                                   `min_score_assign` decimal(12,4) DEFAULT NULL COMMENT '赋分后授课学生最低分',
                                                   `max_score_assign` decimal(12,4) DEFAULT NULL COMMENT '赋分后授课学生最高分',
                                                   `avg_score_assign` decimal(12,4) DEFAULT NULL COMMENT '赋分后授课学生平均分',
                                                   `upper_quartile` decimal(12,4) DEFAULT NULL COMMENT '授课学生分数上四分位数',
                                                   `median` decimal(12,4) DEFAULT NULL COMMENT '授课学生中位数',
                                                   `lower_quartile` decimal(12,4) DEFAULT NULL COMMENT '授课学生分数下四分位数',
                                                   `mode` varchar(255) DEFAULT NULL COMMENT '授课学生分数众数',
                                                   `standard_deviation` decimal(12,4) DEFAULT NULL COMMENT '授课学生分数标准差',
                                                   `relative_position` decimal(12,4) DEFAULT NULL COMMENT '授课学生分数相对位置',
                                                   `reality_count` int DEFAULT NULL COMMENT '授课学生实际参考人数',
                                                   `absent_count` int DEFAULT NULL COMMENT '授课学生缺考人数',
                                                   `total_count` int DEFAULT NULL COMMENT '总人数',
                                                   `teach_college_id` bigint DEFAULT NULL COMMENT '授课学院id',
                                                   `teach_college_name` varchar(100) DEFAULT NULL COMMENT '授课学院名称',
                                                   `teacher_rank` int DEFAULT NULL COMMENT '该科目，学院下排名',
                                                   `teacher_count` int DEFAULT NULL COMMENT '教师人数',
                                                   `other_college_min_score_assign` decimal(12,4) DEFAULT NULL COMMENT '其他学院授课学生最低分（赋分）',
                                                   `other_college_max_score_assign` decimal(12,4) DEFAULT NULL COMMENT '其他学院授课学生最高分（赋分）',
                                                   `other_college_avg_score_assign` decimal(12,4) DEFAULT NULL COMMENT '其他学院授课学生平均分（赋分）',
                                                   `other_college_upper_quartile` decimal(12,4) DEFAULT NULL COMMENT '其他学院授课学生分数上四分位数',
                                                   `other_college_median` decimal(12,4) DEFAULT NULL COMMENT '其他学院授课学生中位数',
                                                   `other_college_lower_quartile` decimal(12,4) DEFAULT NULL COMMENT '其他学院授课学生分数下四分位数',
                                                   `other_college_mode` varchar(255) DEFAULT NULL COMMENT '其他学院授课学生分数众数',
                                                   `other_college_reality_count` int DEFAULT NULL COMMENT '其他学院授课学生实际参考人数',
                                                   `other_college_absent_count` int DEFAULT NULL COMMENT '其他学院授课学生缺考人数',
                                                   `other_college_total_count` int DEFAULT NULL COMMENT '其他学院总人数',
                                                   PRIMARY KEY (`id`) USING BTREE,
                                                   KEY `t_a_exam_course_college_teacher_idx` (`exam_id`,`course_code`,`teacher_name`,`inspect_college_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='参考课程学院下授课教师维度分析表（考试id、课程编号、考察学院id、教师id）';

-- ----------------------------
-- Table structure for t_a_exam_course_difficult
-- ----------------------------
DROP TABLE IF EXISTS `t_a_exam_course_difficult`;
CREATE TABLE `t_a_exam_course_difficult` (
                                             `id` bigint NOT NULL COMMENT 'id',
                                             `exam_id` bigint NOT NULL COMMENT '考试id',
                                             `school_id` bigint DEFAULT NULL,
                                             `course_code` varchar(100) NOT NULL COMMENT '科目编码',
                                             `course_name` varchar(100) DEFAULT NULL COMMENT '科目名称',
                                             `paper_id` bigint DEFAULT NULL COMMENT '试卷id',
                                             `paper_type` varchar(64) DEFAULT NULL COMMENT '试卷类型',
                                             `college_id` bigint NOT NULL COMMENT '学院id',
                                             `interpret` varchar(64) DEFAULT NULL COMMENT '难度类型',
                                             `count` int DEFAULT NULL COMMENT '该难度题目数量',
                                             `scope` varchar(64) DEFAULT NULL COMMENT '难度范围',
                                             `sch_avg_score_rate` decimal(12,4) DEFAULT NULL COMMENT '各难度题在全校的平均得分率',
                                             `col_avg_score_rate` decimal(12,4) DEFAULT NULL COMMENT '各难度题在学院的平均得分率',
                                             PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考试课程，各难度题目得分率表（难度题得分率、中等难度题得分率、简单题得分率）';

-- ----------------------------
-- Table structure for t_a_exam_course_dio
-- ----------------------------
DROP TABLE IF EXISTS `t_a_exam_course_dio`;
CREATE TABLE `t_a_exam_course_dio` (
                                       `id` bigint NOT NULL COMMENT 'id',
                                       `exam_id` bigint NOT NULL COMMENT '考试id',
                                       `school_Id` bigint DEFAULT NULL,
                                       `course_code` varchar(100) NOT NULL COMMENT '科目编码',
                                       `course_name` varchar(100) DEFAULT NULL COMMENT '科目名称',
                                       `paper_id` bigint NOT NULL COMMENT '试卷id',
                                       `dimension_type` varchar(32) DEFAULT NULL COMMENT '考察点类型',
                                       `dimension_code` varchar(32) DEFAULT NULL COMMENT '考察点编号',
                                       `score_rate` decimal(12,4) DEFAULT NULL COMMENT '该考察点平均得分率',
                                       `total_score` decimal(12,4) DEFAULT NULL COMMENT '该考察点在试卷中的总分',
                                       `total_count` decimal(12,4) DEFAULT NULL COMMENT '该考察点考察的题目数量',
                                       PRIMARY KEY (`id`) USING BTREE,
                                       KEY `t_a_exam_course_dio_idx` (`exam_id`,`course_code`,`dimension_type`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='参考课程试卷考察点（整体情况）维度表';

-- ----------------------------
-- Table structure for t_a_exam_course_record
-- ----------------------------
DROP TABLE IF EXISTS `t_a_exam_course_record`;
CREATE TABLE `t_a_exam_course_record` (
                                          `id` bigint NOT NULL,
                                          `school_id` bigint DEFAULT NULL,
                                          `exam_record_id` bigint NOT NULL COMMENT '基础考生成绩记录表id',
                                          `total_score` decimal(12,4) DEFAULT NULL COMMENT '卷面成绩（冗余）',
                                          `assigned_score` decimal(12,4) DEFAULT NULL COMMENT '赋分后成绩',
                                          `percent_grade` int DEFAULT NULL COMMENT '百分位等级',
                                          `score_level` varchar(16) DEFAULT NULL COMMENT '得分水平 ABCDE',
                                          `col_rank` int DEFAULT NULL COMMENT '该学生成绩在学院中的排名（赋分）',
                                          `rank_level` varchar(16) DEFAULT NULL COMMENT '该学生成绩排名所在的等级（col_rank）',
                                          `over_college_rate` decimal(12,4) DEFAULT NULL COMMENT '超过本院占比（赋分）',
                                          `standardized_coefficients` decimal(12,4) DEFAULT NULL COMMENT '标准回归系数',
                                          `exam_id` bigint DEFAULT NULL,
                                          `course_code` varchar(32) DEFAULT NULL,
                                          `paper_id` bigint DEFAULT NULL COMMENT '试卷id（冗余）',
                                          `paper_type` varchar(16) DEFAULT NULL COMMENT '试卷类型（冗余）',
                                          `student_id` bigint DEFAULT NULL,
                                          `student_code` varchar(64) DEFAULT NULL COMMENT '学生编号',
                                          `teacher_id` bigint DEFAULT NULL,
                                          `clazz_id` bigint DEFAULT NULL,
                                          `teach_college_id` bigint DEFAULT NULL,
                                          `inspect_college_id` bigint DEFAULT NULL,
                                          `major_id` bigint DEFAULT NULL,
                                          `absent` tinyint DEFAULT NULL COMMENT '是否缺考 1缺考 0正常（冗余）',
                                          `student_current` tinyint DEFAULT NULL COMMENT '是否应届 1应届 0非应届（冗余）',
                                          PRIMARY KEY (`id`) USING BTREE,
                                          KEY `t_a_exam_course_record_idx` (`exam_id`,`course_code`,`exam_record_id`) USING BTREE,
                                          KEY `t_a_exam_course_record_exam_record_id_idx` (`exam_record_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='参考课程考生成绩维度表';

-- ----------------------------
-- Table structure for t_a_exam_course_record_dio
-- ----------------------------
DROP TABLE IF EXISTS `t_a_exam_course_record_dio`;
CREATE TABLE `t_a_exam_course_record_dio` (
                                              `id` bigint NOT NULL COMMENT 'id',
                                              `school_id` bigint DEFAULT NULL,
                                              `exam_record_id` bigint NOT NULL COMMENT '基础考生成绩记录表id',
                                              `paper_id` bigint NOT NULL COMMENT '试卷id',
                                              `exam_id` bigint DEFAULT NULL,
                                              `course_code` varchar(32) DEFAULT NULL,
                                              `course_name` varchar(32) DEFAULT NULL,
                                              `student_id` bigint DEFAULT NULL,
                                              `student_code` varchar(32) DEFAULT NULL,
                                              `dimension_type` varchar(32) DEFAULT NULL COMMENT '模块类型',
                                              `dimension_code` varchar(32) DEFAULT NULL COMMENT '维度编号',
                                              `student_score` decimal(12,4) DEFAULT NULL COMMENT '该学生在该知识点的得分',
                                              `score_rate` decimal(12,4) DEFAULT NULL COMMENT '得分率',
                                              `proficiency` varchar(32) DEFAULT NULL COMMENT '熟练度等级',
                                              `inspect_college_id` bigint DEFAULT NULL COMMENT '考察学院id',
                                              `inspect_college_name` varchar(32) DEFAULT NULL COMMENT '考察学院名称',
                                              `teacher_id` bigint DEFAULT NULL,
                                              `teacher_name` varchar(32) DEFAULT NULL,
                                              PRIMARY KEY (`id`) USING BTREE,
                                              KEY `mark_idx` (`school_id`,`exam_id`,`course_code`,`student_code`,`dimension_type`,`dimension_code`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='参考课程考生各考察点维度分析表';

-- ----------------------------
-- Table structure for t_a_exam_course_record_mod
-- ----------------------------
DROP TABLE IF EXISTS `t_a_exam_course_record_mod`;
CREATE TABLE `t_a_exam_course_record_mod` (
                                              `id` bigint NOT NULL,
                                              `school_id` bigint DEFAULT NULL,
                                              `exam_record_id` bigint NOT NULL COMMENT '基础考生成绩表id',
                                              `paper_id` bigint NOT NULL COMMENT '试卷id',
                                              `exam_id` bigint DEFAULT NULL,
                                              `course_code` varchar(32) DEFAULT NULL,
                                              `course_name` varchar(32) DEFAULT NULL,
                                              `student_id` bigint DEFAULT NULL,
                                              `student_code` varchar(32) DEFAULT NULL,
                                              `module_type` varchar(32) DEFAULT NULL COMMENT '考察模块名称',
                                              `total_score` decimal(12,4) DEFAULT NULL COMMENT '该考生在该模块的总得分（实际分）',
                                              `score_rate` decimal(12,4) DEFAULT NULL COMMENT '该学生在该考察模块的整体掌握率（total_score/full_score）',
                                              `full_score` decimal(12,4) DEFAULT NULL COMMENT '考察模块在试卷中的总分(实际分)',
                                              `proficiency` varchar(32) DEFAULT NULL COMMENT '该学生在该模块掌握的熟练度（score_rate）',
                                              `col_rank` int DEFAULT NULL COMMENT '该学生在该模块的得分在学院排名(赋分)',
                                              `percent_grade` int DEFAULT NULL COMMENT '百分等级数',
                                              `level` varchar(32) DEFAULT NULL COMMENT '等级(不确定百分等级、排名等级、得分率等级)',
                                              `inspect_college_id` bigint DEFAULT NULL COMMENT '所属考察学院id',
                                              `inspect_college_name` varchar(32) DEFAULT NULL COMMENT '学生所属考察学院名称',
                                              PRIMARY KEY (`id`) USING BTREE,
                                              KEY `t_a_exam_course_record_mod_idx` (`school_id`,`exam_id`,`student_code`,`course_code`,`module_type`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='参考课程考生考察点模块维度表';

-- ----------------------------
-- Table structure for t_a_exam_course_teacher
-- ----------------------------
DROP TABLE IF EXISTS `t_a_exam_course_teacher`;
CREATE TABLE `t_a_exam_course_teacher` (
                                           `id` bigint NOT NULL COMMENT 'id',
                                           `exam_id` bigint NOT NULL COMMENT '考试id',
                                           `school_id` bigint DEFAULT NULL,
                                           `course_code` varchar(100) NOT NULL COMMENT '科目编码',
                                           `course_name` varchar(100) DEFAULT NULL COMMENT '科目名称',
                                           `teacher_id` bigint NOT NULL COMMENT '教师id',
                                           `teacher_name` varchar(100) DEFAULT NULL COMMENT '授课教师名称（冗余字段）',
                                           `min_score` decimal(12,4) DEFAULT NULL COMMENT '最低分',
                                           `max_score` decimal(12,4) DEFAULT NULL COMMENT '最高分',
                                           `avg_score` decimal(12,4) DEFAULT NULL COMMENT '平均分',
                                           `upper_quartile` decimal(12,4) DEFAULT NULL COMMENT '上四分位数',
                                           `median` decimal(12,4) DEFAULT NULL COMMENT '中位数',
                                           `lower_quartile` decimal(12,4) DEFAULT NULL COMMENT '下四分位数',
                                           `mode` varchar(255) DEFAULT NULL COMMENT '众数',
                                           `standard_deviation` decimal(12,4) DEFAULT NULL COMMENT '标准差',
                                           `relative_position` decimal(12,4) DEFAULT NULL COMMENT '相对位置',
                                           `reality_count` int DEFAULT NULL COMMENT '实际参考人数',
                                           `absent_count` int DEFAULT NULL COMMENT '缺考人数',
                                           `total_count` int DEFAULT NULL COMMENT '总人数',
                                           PRIMARY KEY (`id`) USING BTREE,
                                           KEY `t_a_exam_course_teacher_idx` (`exam_id`,`course_code`,`teacher_name`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='参考课程授课教师维度表';

-- ----------------------------
-- Table structure for t_a_exam_course_teacher_college_difficult
-- ----------------------------
DROP TABLE IF EXISTS `t_a_exam_course_teacher_college_difficult`;
CREATE TABLE `t_a_exam_course_teacher_college_difficult` (
                                                             `id` bigint NOT NULL COMMENT '主键',
                                                             `exam_id` bigint NOT NULL COMMENT '考试id',
                                                             `school_id` bigint NOT NULL COMMENT '学校id',
                                                             `course_code` varchar(100) NOT NULL COMMENT '课程编号',
                                                             `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
                                                             `paper_id` bigint NOT NULL COMMENT '试卷id',
                                                             `paper_type` varchar(64) DEFAULT NULL COMMENT '试卷类型',
                                                             `teacher_id` bigint NOT NULL COMMENT '教师id',
                                                             `teacher_name` varchar(64) DEFAULT NULL COMMENT '教师名称',
                                                             `inspect_college_id` bigint NOT NULL COMMENT '考察学院id',
                                                             `inspect_college_name` varchar(64) DEFAULT NULL COMMENT '考察学院名称',
                                                             `interpret` varchar(64) DEFAULT NULL COMMENT '难度类型',
                                                             `count` int DEFAULT NULL COMMENT '该难度题目数量',
                                                             `scope` varchar(64) DEFAULT NULL COMMENT '难度范围',
                                                             `col_avg_score_rate` decimal(12,4) DEFAULT NULL COMMENT '各难度题在考察学院集合的平均得分率',
                                                             `tea_col_avg_score_rate` decimal(12,4) DEFAULT NULL COMMENT '各难度题在教师考察学院下集合的平均得分率',
                                                             PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='授课教师-本院难度分布和该教师在本院难度分布对比';

-- ----------------------------
-- Table structure for t_a_exam_course_teacher_college_dio
-- ----------------------------
DROP TABLE IF EXISTS `t_a_exam_course_teacher_college_dio`;
CREATE TABLE `t_a_exam_course_teacher_college_dio` (
                                                       `id` bigint NOT NULL,
                                                       `exam_id` bigint NOT NULL COMMENT '考试id',
                                                       `school_id` bigint NOT NULL COMMENT '学校id',
                                                       `course_code` varchar(100) NOT NULL COMMENT '课程编号',
                                                       `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
                                                       `teacher_id` bigint NOT NULL COMMENT '教师id',
                                                       `teacher_name` varchar(64) DEFAULT NULL COMMENT '教师名称',
                                                       `inspect_college_id` bigint NOT NULL COMMENT '考察学院id',
                                                       `inspect_college_name` varchar(64) DEFAULT NULL COMMENT '考察学院名称',
                                                       `dimension_type` varchar(64) DEFAULT NULL COMMENT '考察点类型',
                                                       `dimension_code` varchar(64) DEFAULT NULL COMMENT '考察点编号',
                                                       `avg_score` decimal(12,4) DEFAULT NULL COMMENT '考察点平均分',
                                                       `score_rate` decimal(12,4) DEFAULT NULL COMMENT '考察点得分率',
                                                       PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='各个教师在考察学院下的各考察点得分情况表';

-- ----------------------------
-- Table structure for t_a_exam_course_teacher_college_paper_struct
-- ----------------------------
DROP TABLE IF EXISTS `t_a_exam_course_teacher_college_paper_struct`;
CREATE TABLE `t_a_exam_course_teacher_college_paper_struct` (
                                                                `id` bigint NOT NULL COMMENT '主键',
                                                                `school_id` bigint NOT NULL COMMENT '学校id',
                                                                `paper_struct_id` bigint NOT NULL COMMENT '试卷结构基础表id',
                                                                `exam_id` bigint NOT NULL COMMENT '考试id',
                                                                `course_code` varchar(32) NOT NULL COMMENT '课程编号',
                                                                `course_name` varchar(32) DEFAULT NULL COMMENT '课程名称',
                                                                `paper_id` bigint NOT NULL COMMENT '试卷id',
                                                                `teacher_id` bigint NOT NULL COMMENT '教师id',
                                                                `teacher_name` varchar(64) DEFAULT NULL COMMENT '教师名称',
                                                                `inspect_college_id` bigint NOT NULL COMMENT '考察学院id',
                                                                `inspect_college_name` varchar(64) DEFAULT NULL COMMENT '考察学院名称',
                                                                `paper_type` varchar(64) DEFAULT NULL COMMENT '试卷类型',
                                                                `question_name` varchar(64) DEFAULT NULL COMMENT '大题名称',
                                                                `number_type` varchar(64) DEFAULT NULL COMMENT '题号类型（客观题、主观题）',
                                                                `big_question_number` varchar(64) DEFAULT NULL COMMENT '大题号',
                                                                `small_question_number` varchar(64) DEFAULT NULL COMMENT '小题号(每一课程试卷的唯一标识)',
                                                                `question_type` varchar(64) DEFAULT NULL COMMENT '题目类型',
                                                                `full_score` decimal(12,4) DEFAULT NULL COMMENT '满分',
                                                                `score_rules` varchar(32) DEFAULT NULL COMMENT '计分规则',
                                                                `rules_desc` varchar(500) DEFAULT NULL COMMENT '规则说明',
                                                                `knowledge_dimension` varchar(255) DEFAULT NULL COMMENT '知识维度',
                                                                `ability_dimension` varchar(255) DEFAULT NULL COMMENT '能力维度',
                                                                `literacy_dimension` varchar(255) DEFAULT NULL COMMENT '素养维度',
                                                                `standard_score_rate` decimal(12,4) DEFAULT NULL COMMENT '标准难度系数（得分率）',
                                                                `score_rate` decimal(12,4) DEFAULT NULL COMMENT '得分率',
                                                                `difficult` varchar(32) DEFAULT NULL COMMENT '难度',
                                                                `paper_struct_judge` varchar(32) DEFAULT NULL COMMENT '试卷结构每道题目正确（全对），错误枚举\\nALL_CORRECT("全对"), NOT_QUITE_RIGHT("不完全正确的"),',
                                                                PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='教师考察学院下试卷结构维度分析表';

-- ----------------------------
-- Table structure for t_a_exam_course_teacher_difficult
-- ----------------------------
DROP TABLE IF EXISTS `t_a_exam_course_teacher_difficult`;
CREATE TABLE `t_a_exam_course_teacher_difficult` (
                                                     `id` bigint NOT NULL,
                                                     `exam_id` bigint NOT NULL COMMENT '考试id',
                                                     `school_id` bigint DEFAULT NULL,
                                                     `course_code` varchar(100) NOT NULL,
                                                     `course_name` varchar(100) DEFAULT NULL,
                                                     `paper_id` bigint DEFAULT NULL,
                                                     `paper_type` varchar(64) DEFAULT NULL,
                                                     `teacher_id` bigint NOT NULL,
                                                     `teacher_name` varchar(64) DEFAULT NULL,
                                                     `interpret` varchar(64) DEFAULT NULL COMMENT '难度类型',
                                                     `count` int DEFAULT NULL COMMENT '该难度题目数量',
                                                     `scope` varchar(64) DEFAULT NULL COMMENT '难度范围',
                                                     `sch_avg_score_rate` decimal(12,4) DEFAULT NULL COMMENT '各难度题在全校的平均得分率',
                                                     `tea_avg_score_rate` decimal(12,4) DEFAULT NULL COMMENT '各难度题在教师的平均得分率',
                                                     PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='授课教师-全校各难度得分率对比';

-- ----------------------------
-- Table structure for t_a_exam_course_teacher_dio
-- ----------------------------
DROP TABLE IF EXISTS `t_a_exam_course_teacher_dio`;
CREATE TABLE `t_a_exam_course_teacher_dio` (
                                               `id` bigint NOT NULL,
                                               `exam_id` bigint NOT NULL COMMENT '考试id',
                                               `school_id` bigint DEFAULT NULL COMMENT '学校id',
                                               `course_code` varchar(100) NOT NULL COMMENT '课程编号',
                                               `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
                                               `teacher_id` bigint DEFAULT NULL,
                                               `teacher_name` varchar(45) DEFAULT NULL COMMENT '教师id',
                                               `dimension_type` varchar(32) DEFAULT NULL COMMENT '模块类型',
                                               `dimension_code` varchar(32) DEFAULT NULL COMMENT '考察点编号',
                                               `avg_score` decimal(12,4) DEFAULT NULL COMMENT '考察点教师授课学生平均得分',
                                               `score_rate` decimal(12,4) DEFAULT NULL COMMENT '该考察点教师平均得分率',
                                               PRIMARY KEY (`id`) USING BTREE,
                                               KEY `t_a_exam_course_teacher_dio_idx` (`exam_id`,`course_code`,`dimension_code`,`dimension_type`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='教师维度分析表';

-- ----------------------------
-- Table structure for t_a_exam_course_teacher_paper_struct
-- ----------------------------
DROP TABLE IF EXISTS `t_a_exam_course_teacher_paper_struct`;
CREATE TABLE `t_a_exam_course_teacher_paper_struct` (
                                                        `id` bigint NOT NULL COMMENT '主键',
                                                        `school_id` bigint NOT NULL COMMENT '学校id',
                                                        `paper_struct_id` bigint NOT NULL COMMENT '试卷结构基础表id',
                                                        `exam_id` bigint NOT NULL COMMENT '考试id',
                                                        `course_code` varchar(32) NOT NULL COMMENT '课程编号',
                                                        `course_name` varchar(32) DEFAULT NULL COMMENT '课程名称',
                                                        `paper_id` bigint NOT NULL COMMENT '试卷id',
                                                        `teacher_id` bigint NOT NULL COMMENT '教师id',
                                                        `teacher_name` varchar(64) DEFAULT NULL COMMENT '教师名称',
                                                        `paper_type` varchar(64) DEFAULT NULL COMMENT '试卷类型',
                                                        `question_name` varchar(64) DEFAULT NULL COMMENT '大题名称',
                                                        `number_type` varchar(64) DEFAULT NULL COMMENT '题号类型（客观题、主观题）',
                                                        `big_question_number` varchar(64) DEFAULT NULL COMMENT '大题号',
                                                        `small_question_number` varchar(64) DEFAULT NULL COMMENT '小题号(每一课程试卷的唯一标识)',
                                                        `question_type` varchar(64) DEFAULT NULL COMMENT '题目类型',
                                                        `full_score` decimal(12,4) DEFAULT NULL COMMENT '满分',
                                                        `score_rules` varchar(32) DEFAULT NULL COMMENT '计分规则',
                                                        `rules_desc` varchar(500) DEFAULT NULL COMMENT '规则说明',
                                                        `knowledge_dimension` varchar(255) DEFAULT NULL COMMENT '知识维度',
                                                        `ability_dimension` varchar(255) DEFAULT NULL COMMENT '能力维度',
                                                        `literacy_dimension` varchar(255) DEFAULT NULL COMMENT '素养维度',
                                                        `standard_score_rate` decimal(12,4) DEFAULT NULL COMMENT '标准难度系数（得分率）',
                                                        `score_rate` decimal(12,4) DEFAULT NULL COMMENT '得分率',
                                                        `difficult` varchar(32) DEFAULT NULL COMMENT '难度',
                                                        `paper_struct_judge` varchar(32) DEFAULT NULL COMMENT '试卷结构每道题目正确（全对），错误枚举\\nALL_CORRECT("全对"), NOT_QUITE_RIGHT("不完全正确的"),',
                                                        PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='教师下试卷结构维度分析表';

-- ----------------------------
-- Table structure for t_a_exam_total
-- ----------------------------
DROP TABLE IF EXISTS `t_a_exam_total`;
CREATE TABLE `t_a_exam_total` (
                                  `id` bigint NOT NULL,
                                  `exam_id` bigint NOT NULL COMMENT '基础考试表id',
                                  `school_id` bigint DEFAULT NULL,
                                  `college_count` int DEFAULT NULL COMMENT '考察学院总数',
                                  `college_names` text COMMENT '考察学院名称（、隔开）',
                                  `course_count` int DEFAULT NULL COMMENT '课程总数',
                                  `public_course_count` int DEFAULT NULL COMMENT '公共课总数',
                                  `major_course_count` int DEFAULT NULL COMMENT '专业课总数',
                                  `total_count` int DEFAULT NULL COMMENT '总人数',
                                  `reality_count` int DEFAULT NULL COMMENT '实际参考人数',
                                  `absent_count` int DEFAULT NULL COMMENT '缺考人数',
                                  `total_course_times` int DEFAULT NULL COMMENT '总考试科次',
                                  `reality_course_times` int DEFAULT NULL COMMENT '实际考科次',
                                  `absent_course_times` int DEFAULT NULL COMMENT '缺考科次',
                                  `exam_time` bigint DEFAULT NULL COMMENT '考试时间',
                                  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考试维度总计';

-- ----------------------------
-- Table structure for t_a_paper_struct
-- ----------------------------
DROP TABLE IF EXISTS `t_a_paper_struct`;
CREATE TABLE `t_a_paper_struct` (
                                    `id` bigint NOT NULL,
                                    `school_id` bigint DEFAULT NULL,
                                    `paper_struct_id` bigint NOT NULL COMMENT '基础试卷结构表id',
                                    `exam_id` bigint DEFAULT NULL COMMENT '考试id',
                                    `course_code` varchar(32) DEFAULT NULL COMMENT '课程编号',
                                    `course_name` varchar(32) DEFAULT NULL COMMENT '课程名称',
                                    `paper_id` bigint DEFAULT NULL COMMENT '基础试卷表id（冗余）',
                                    `paper_type` varchar(32) DEFAULT NULL COMMENT '试卷类型',
                                    `question_name` varchar(200) DEFAULT NULL COMMENT '大题名称',
                                    `number_type` varchar(32) DEFAULT NULL COMMENT '题号类型（客观题、主观题）',
                                    `big_question_number` varchar(64) DEFAULT NULL COMMENT '大题号',
                                    `small_question_number` varchar(64) DEFAULT NULL COMMENT '小题号(每一课程试卷的唯一标识)',
                                    `question_type` varchar(32) DEFAULT NULL COMMENT '题目类型',
                                    `full_score` decimal(12,4) DEFAULT NULL COMMENT '满分',
                                    `score_rules` varchar(32) DEFAULT NULL COMMENT '计分规则',
                                    `rules_desc` varchar(500) DEFAULT NULL COMMENT '规则说明',
                                    `knowledge_dimension` varchar(255) DEFAULT NULL COMMENT '知识维度',
                                    `ability_dimension` varchar(255) DEFAULT NULL COMMENT '能力维度',
                                    `literacy_dimension` varchar(255) DEFAULT NULL COMMENT '素养维度',
                                    `standard_score_rate` decimal(12,4) DEFAULT NULL COMMENT '标准难度系数（得分率）',
                                    `score_rate` decimal(12,4) DEFAULT NULL COMMENT '得分率',
                                    `difficult` varchar(32) DEFAULT NULL COMMENT '难度',
                                    `validity` decimal(12,4) DEFAULT NULL COMMENT '效度',
                                    `paper_struct_judge` varchar(32) DEFAULT NULL COMMENT '试卷结构每道题目正确（全对），错误枚举\nALL_CORRECT("全对"), NOT_QUITE_RIGHT("不完全正确的"),',
                                    PRIMARY KEY (`id`) USING BTREE,
                                    KEY `t_a_paper_struct_idx` (`exam_id`,`course_code`,`paper_struct_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='试卷结构维度分析表';

-- ----------------------------
-- Table structure for t_b_answer
-- ----------------------------
DROP TABLE IF EXISTS `t_b_answer`;
CREATE TABLE `t_b_answer` (
                              `id` bigint NOT NULL COMMENT '考生作答记录详情id',
                              `exam_record_id` bigint NOT NULL COMMENT '考试记录id',
                              `number_type` varchar(32) NOT NULL COMMENT '题号类型（客观题、主观题）',
                              `main_number` varchar(32) NOT NULL COMMENT '大题号',
                              `sub_number` varchar(32) NOT NULL COMMENT '小题号',
                              `answer` varchar(255) DEFAULT NULL COMMENT '考生作答答案',
                              `score` decimal(12,4) DEFAULT NULL COMMENT '分数',
                              `path` varchar(255) DEFAULT NULL COMMENT '作答轨迹',
                              `time` bigint DEFAULT NULL COMMENT '作答时间',
                              PRIMARY KEY (`id`) USING BTREE,
                              KEY `t_b_answer_idx` (`exam_record_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='基础考生作答记录详情表';

-- ----------------------------
-- Table structure for t_b_cloud_exam
-- ----------------------------
DROP TABLE IF EXISTS `t_b_cloud_exam`;
CREATE TABLE `t_b_cloud_exam` (
                                  `id` bigint NOT NULL COMMENT '对应云阅卷主键',
                                  `exam_id` bigint NOT NULL COMMENT '教研分析考试id',
                                  `school_id` bigint NOT NULL COMMENT '学校id',
                                  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，false：停用，true：启用',
                                  `remark` varchar(64) DEFAULT NULL COMMENT '备注（记录云阅卷该考试下包含的学科名称）',
                                  `access_key` varchar(64) NOT NULL COMMENT '密钥key',
                                  `access_secret` varchar(64) NOT NULL COMMENT '密钥secret',
                                  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                  `create_time` bigint DEFAULT NULL COMMENT '创建人名称',
                                  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                  `update_time` bigint DEFAULT NULL COMMENT '更新人名称',
                                  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='云阅卷考试信息表（同步考生成绩使用）';

-- ----------------------------
-- Table structure for t_b_common_level_config
-- ----------------------------
DROP TABLE IF EXISTS `t_b_common_level_config`;
CREATE TABLE `t_b_common_level_config` (
                                           `id` bigint NOT NULL COMMENT '常规等级配置表主键',
                                           `exam_id` bigint NOT NULL,
                                           `course_code` varchar(100) NOT NULL COMMENT '科目编码',
                                           `course_name` varchar(100) DEFAULT NULL COMMENT '科目名称',
                                           `level_type` varchar(32) DEFAULT NULL COMMENT '等级类型（熟练度、百分等级、题目难度）',
                                           `level` varchar(32) DEFAULT NULL COMMENT '等级',
                                           `scope` varchar(32) DEFAULT NULL COMMENT '取值范围（计算用）',
                                           `interpret` varchar(32) DEFAULT NULL COMMENT '描述',
                                           `min` decimal(12,4) DEFAULT NULL COMMENT '最小边界（展示用）',
                                           `max` decimal(12,4) DEFAULT NULL COMMENT '最大边界（展示用）',
                                           PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='等级配置表（试卷难易度、成绩在学院排名百分比）';

-- ----------------------------
-- Table structure for t_b_common_rank_level_config
-- ----------------------------
DROP TABLE IF EXISTS `t_b_common_rank_level_config`;
CREATE TABLE `t_b_common_rank_level_config` (
                                                `id` bigint NOT NULL,
                                                `exam_id` bigint NOT NULL,
                                                `course_code` varchar(100) NOT NULL COMMENT '科目编码',
                                                `course_name` varchar(100) DEFAULT NULL COMMENT '科目名称',
                                                `level` varchar(32) DEFAULT NULL COMMENT '等级',
                                                `min` decimal(12,4) DEFAULT NULL COMMENT '展示用最小值',
                                                `max` decimal(12,4) DEFAULT NULL COMMENT '展示用最大值',
                                                `scope` varchar(32) DEFAULT NULL COMMENT '取值范围（计算用）',
                                                `interpret` varchar(255) DEFAULT NULL COMMENT '解释',
                                                PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='学生成绩在学院排名的配置表（赋分成绩在学院排名档次）';

-- ----------------------------
-- Table structure for t_b_dimension
-- ----------------------------
DROP TABLE IF EXISTS `t_b_dimension`;
CREATE TABLE `t_b_dimension` (
                                 `id` bigint NOT NULL,
                                 `exam_id` bigint DEFAULT NULL COMMENT '考试id',
                                 `course_code` varchar(100) DEFAULT NULL COMMENT '科目编码',
                                 `course_name` varchar(100) DEFAULT NULL COMMENT '科目名称',
                                 `dimension_type` varchar(32) DEFAULT NULL COMMENT '维度类型 - 知识，能力，素养',
                                 `code_primary` varchar(32) DEFAULT NULL COMMENT '一级维度编号',
                                 `name_primary` varchar(255) DEFAULT NULL COMMENT '一级维度名称',
                                 `code_second` varchar(32) DEFAULT NULL COMMENT '二级维度编号',
                                 `name_second` varchar(255) DEFAULT NULL COMMENT '二级维度名称',
                                 `interpretation` varchar(255) DEFAULT NULL COMMENT '一级维度术语解释',
                                 PRIMARY KEY (`id`) USING BTREE,
                                 KEY `t_b_dimension_idx` (`exam_id`,`course_code`,`dimension_type`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考察点维度基础表';

-- ----------------------------
-- Table structure for t_b_exam
-- ----------------------------
DROP TABLE IF EXISTS `t_b_exam`;
CREATE TABLE `t_b_exam` (
                            `id` bigint NOT NULL COMMENT '教研分析考试主键',
                            `school_id` bigint NOT NULL COMMENT '学校id',
                            `exam_code` varchar(32) NOT NULL COMMENT '考试编号（在学校下唯一）',
                            `exam_name` varchar(32) NOT NULL COMMENT '考试名称（2020~2021学年上学期期末考试）',
                            `semester_id` bigint NOT NULL COMMENT '学期id',
                            `exam_time` bigint DEFAULT NULL COMMENT '考试时间',
                            `create_id` bigint DEFAULT NULL,
                            `create_time` bigint DEFAULT NULL,
                            `update_id` bigint DEFAULT NULL,
                            `update_time` bigint DEFAULT NULL,
                            PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='教研分析基础考试信息表';

-- ----------------------------
-- Table structure for t_b_exam_course
-- ----------------------------
DROP TABLE IF EXISTS `t_b_exam_course`;
CREATE TABLE `t_b_exam_course` (
                                   `id` bigint NOT NULL COMMENT '主键',
                                   `school_id` bigint DEFAULT NULL COMMENT '学校id',
                                   `exam_id` bigint NOT NULL COMMENT '考试id',
                                   `course_code` varchar(100) NOT NULL COMMENT '科目编码',
                                   `course_name` varchar(100) NOT NULL COMMENT '科目名称',
                                   `teach_college_id` bigint DEFAULT NULL COMMENT '开课学院id',
                                   `teach_college_name` varchar(100) DEFAULT NULL COMMENT '开课学院名称',
                                   `test_status` varchar(30) DEFAULT NULL COMMENT '测试状态（UN_TEST：未测试、TEST：已测试、CHECKED：已抽检）',
                                   `publish_status` varchar(30) DEFAULT NULL COMMENT '发布状态（UN_COMPUTE：未计算、COMPUTING：计算中、 UN_PUBLISH：未发布、PUBLISH：已发布）',
                                   `open_data_md5` mediumtext COMMENT '开放接口获取的数据md5记录',
                                   `create_id` bigint DEFAULT NULL COMMENT '创建人',
                                   `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                   `update_id` bigint DEFAULT NULL COMMENT '更新人',
                                   `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                   `paper_type` varchar(100) NOT NULL COMMENT '试卷编号',
                                   PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考试科目表';

-- ----------------------------
-- Table structure for t_b_exam_record
-- ----------------------------
DROP TABLE IF EXISTS `t_b_exam_record`;
CREATE TABLE `t_b_exam_record` (
                                   `id` bigint NOT NULL,
                                   `exam_id` bigint NOT NULL COMMENT '考试id',
                                   `paper_id` bigint NOT NULL COMMENT '试卷id',
                                   `paper_type` varchar(32) NOT NULL COMMENT '试卷类型',
                                   `exam_student_id` bigint NOT NULL COMMENT '考生id',
                                   `subjective_score` decimal(12,4) DEFAULT NULL COMMENT '主观题总分',
                                   `objective_score` decimal(12,4) DEFAULT NULL COMMENT '客观题总分',
                                   `total_score` decimal(12,4) DEFAULT NULL COMMENT '总分',
                                   PRIMARY KEY (`id`) USING BTREE,
                                   KEY `t_b_exam_record_idx` (`exam_student_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考生考试记录';

-- ----------------------------
-- Table structure for t_b_exam_student
-- ----------------------------
DROP TABLE IF EXISTS `t_b_exam_student`;
CREATE TABLE `t_b_exam_student` (
                                    `id` bigint NOT NULL COMMENT '主键',
                                    `school_id` bigint DEFAULT NULL COMMENT '学校id',
                                    `exam_id` bigint NOT NULL COMMENT '考试id',
                                    `name` varchar(100) NOT NULL COMMENT '姓名',
                                    `student_id` bigint NOT NULL COMMENT '学生id',
                                    `course_code` varchar(100) NOT NULL COMMENT '科目编码',
                                    `course_name` varchar(100) DEFAULT NULL COMMENT '科目名称',
                                    `teach_college_id` bigint NOT NULL COMMENT '开课学院id',
                                    `inspect_college_id` bigint NOT NULL COMMENT '考查学院id',
                                    `clazz_id` bigint NOT NULL COMMENT '考试时候的班级id',
                                    `major_id` bigint DEFAULT NULL COMMENT '考试时候所在的专业id',
                                    `teacher_id` bigint NOT NULL COMMENT '授课老师id',
                                    `ticket_number` varchar(100) NOT NULL COMMENT '准考证号',
                                    `student_code` varchar(100) NOT NULL COMMENT '学号',
                                    `absent` tinyint NOT NULL DEFAULT '0' COMMENT '是否缺考，false：没缺考，true：缺考',
                                    `student_current` tinyint NOT NULL DEFAULT '1' COMMENT '是否本届，false：非本届，true：本届',
                                    PRIMARY KEY (`id`) USING BTREE,
                                    KEY `t_b_exam_student_idx` (`exam_id`,`student_code`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考生表';

-- ----------------------------
-- Table structure for t_b_examination
-- ----------------------------
DROP TABLE IF EXISTS `t_b_examination`;
CREATE TABLE `t_b_examination` (
                                   `id` bigint NOT NULL,
                                   `school_id` bigint NOT NULL,
                                   `exam_id` bigint NOT NULL,
                                   `course_header_code` varchar(64) DEFAULT NULL COMMENT '教务课头号',
                                   `examination_course_code` varchar(64) DEFAULT NULL COMMENT '教务课程号',
                                   `course_name` varchar(64) DEFAULT NULL COMMENT '课程名称',
                                   `teacher_code` varchar(45) DEFAULT NULL COMMENT '教师编号',
                                   `teacher_name` varchar(64) DEFAULT NULL COMMENT '教师名称',
                                   `teach_college_name` varchar(64) DEFAULT NULL COMMENT '开课学院名称',
                                   `credit` varchar(64) DEFAULT NULL COMMENT '学分',
                                   `student_code` varchar(64) DEFAULT NULL COMMENT '学号',
                                   `student_name` varchar(64) DEFAULT NULL COMMENT '学生名称',
                                   `inspect_college_name` varchar(64) DEFAULT NULL COMMENT '考察学院名称',
                                   `major` varchar(64) DEFAULT NULL COMMENT '专业',
                                   `study_type` varchar(64) DEFAULT NULL COMMENT '学习类型（应届、本届）',
                                   `grade` varchar(64) DEFAULT NULL COMMENT '年级',
                                   PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考务数据基础表';

-- ----------------------------
-- Table structure for t_b_examination_relation
-- ----------------------------
DROP TABLE IF EXISTS `t_b_examination_relation`;
CREATE TABLE `t_b_examination_relation` (
                                            `id` bigint NOT NULL,
                                            `school_id` bigint NOT NULL COMMENT '学校id',
                                            `exam_id` bigint NOT NULL COMMENT '考试id',
                                            `examination_course_code` varchar(64) NOT NULL COMMENT '考务数据课程编号',
                                            `course_name` varchar(64) NOT NULL COMMENT '课程名称',
                                            `teach_college_name` varchar(64) NOT NULL COMMENT '开课学院名称',
                                            `teacher_code` varchar(64) DEFAULT NULL COMMENT '教师编号',
                                            `teacher_name` varchar(64) DEFAULT NULL COMMENT '教师名称',
                                            PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考务关系表（学校-考试-课程-开课学院-教师）';

-- ----------------------------
-- Table structure for t_b_module_config
-- ----------------------------
DROP TABLE IF EXISTS `t_b_module_config`;
CREATE TABLE `t_b_module_config` (
                                     `id` bigint NOT NULL,
                                     `exam_id` bigint NOT NULL COMMENT '考试id',
                                     `course_code` varchar(100) NOT NULL COMMENT '科目编码',
                                     `course_name` varchar(100) DEFAULT NULL COMMENT '科目名称',
                                     `module_type` varchar(64) DEFAULT NULL COMMENT '模块类型（知识、维度、素养）',
                                     `formula` varchar(64) DEFAULT NULL COMMENT '计算公式规则',
                                     `scope` varchar(64) DEFAULT NULL COMMENT '取值范围（计算用）',
                                     `level_code` varchar(32) DEFAULT NULL COMMENT '等级',
                                     `level_name` varchar(32) DEFAULT NULL COMMENT '水平等级层次',
                                     `result` text COMMENT '诊断结果',
                                     `advice` text COMMENT '学习建议',
                                     `attribute` varchar(32) DEFAULT NULL COMMENT '试卷结构表（t_b_paper_struct）中的模块属性列的列名称',
                                     PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考察点模块配置表';

-- ----------------------------
-- Table structure for t_b_module_proficiency
-- ----------------------------
DROP TABLE IF EXISTS `t_b_module_proficiency`;
CREATE TABLE `t_b_module_proficiency` (
                                          `id` bigint NOT NULL COMMENT '模块熟练度主键',
                                          `exam_id` bigint NOT NULL COMMENT '考试id',
                                          `course_code` varchar(100) NOT NULL COMMENT '科目编码',
                                          `course_name` varchar(100) DEFAULT NULL COMMENT '科目名称',
                                          `module_type` varchar(32) DEFAULT NULL COMMENT '模块类型',
                                          `interpret` varchar(255) DEFAULT NULL COMMENT '解释',
                                          `remark` varchar(255) DEFAULT NULL COMMENT '备注',
                                          `define` varchar(255) DEFAULT NULL COMMENT '二级熟练度定义',
                                          `level` varchar(32) DEFAULT NULL COMMENT '等级',
                                          `min` varchar(32) DEFAULT NULL COMMENT '展示用最小边界',
                                          `max` varchar(32) DEFAULT NULL COMMENT '展示用最大边界',
                                          `scope` varchar(32) DEFAULT NULL COMMENT '取值范围（计算用）',
                                          PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='模块二级考察点维度熟练度配置，及模块简要描述表';

-- ----------------------------
-- Table structure for t_b_paper
-- ----------------------------
DROP TABLE IF EXISTS `t_b_paper`;
CREATE TABLE `t_b_paper` (
                             `id` bigint NOT NULL COMMENT '试卷id',
                             `exam_id` bigint NOT NULL COMMENT '考试id',
                             `course_code` varchar(100) NOT NULL COMMENT '科目编码',
                             `course_name` varchar(100) DEFAULT NULL COMMENT '科目名称',
                             `paper_type` varchar(32) NOT NULL COMMENT '试卷类型',
                             `start_time` bigint DEFAULT NULL COMMENT '该试卷考试开始时间',
                             `end_time` bigint DEFAULT NULL COMMENT '该试卷考试结束时间',
                             `coefficient` decimal(12,4) DEFAULT NULL COMMENT '赋分系数（赋值分=卷面成绩 +（卷面满分-卷面成绩）/ 赋分系数）',
                             `total_score` decimal(12,4) DEFAULT NULL COMMENT '试卷满分',
                             `pass_score` decimal(12,4) DEFAULT NULL COMMENT '试卷及格分',
                             `score_type` varchar(64) DEFAULT NULL COMMENT '赋分类型\\n1.NEED_ASSIGN_SCORE("是","需要且按照公式赋分")\\n2.NO_NEED_ASSIGN_SCORE("否","不需要赋分，标准分即为赋分")\\n3.SPECIAL_ASSIGN_SCORE("特殊赋分","特殊赋分，按照excel表进行赋分")',
                             PRIMARY KEY (`id`) USING BTREE,
                             KEY `t_b_paper_idx` (`exam_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='试卷基础信息表（赋分系数、试卷类型、满分、及格分等）';

-- ----------------------------
-- Table structure for t_b_paper_struct
-- ----------------------------
DROP TABLE IF EXISTS `t_b_paper_struct`;
CREATE TABLE `t_b_paper_struct` (
                                    `id` bigint NOT NULL,
                                    `paper_id` bigint DEFAULT NULL COMMENT '试卷基础表id',
                                    `question_name` varchar(200) DEFAULT NULL COMMENT '大题名称',
                                    `number_type` varchar(32) DEFAULT NULL COMMENT '题号类型（客观题、主观题）',
                                    `big_question_number` varchar(64) DEFAULT NULL COMMENT '大题号',
                                    `small_question_number` varchar(64) DEFAULT NULL COMMENT '小题号(每一课程试卷的唯一标识)',
                                    `question_type` varchar(32) DEFAULT NULL COMMENT '题目类型',
                                    `full_score` decimal(12,4) DEFAULT NULL COMMENT '满分',
                                    `score_rules` varchar(32) DEFAULT NULL COMMENT '计分规则',
                                    `rules_desc` varchar(500) DEFAULT NULL COMMENT '规则说明',
                                    `knowledge_dimension` varchar(255) DEFAULT NULL COMMENT '知识维度',
                                    `ability_dimension` varchar(255) DEFAULT NULL COMMENT '能力维度',
                                    `literacy_dimension` varchar(255) DEFAULT NULL COMMENT '素养维度',
                                    PRIMARY KEY (`id`) USING BTREE,
                                    KEY `t_b_paper_struct_idx` (`paper_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='试卷结构表';

-- ----------------------------
-- Table structure for t_b_school_clazz
-- ----------------------------
DROP TABLE IF EXISTS `t_b_school_clazz`;
CREATE TABLE `t_b_school_clazz` (
                                    `id` bigint NOT NULL COMMENT '班级id',
                                    `school_id` bigint NOT NULL COMMENT '学校id',
                                    `clazz_code` varchar(100) DEFAULT NULL COMMENT '班级编号',
                                    `clazz_name` varchar(100) DEFAULT NULL COMMENT '班级名称',
                                    `enable` tinyint DEFAULT '1' COMMENT '是否启用，false：停用，true：启用',
                                    `create_id` bigint DEFAULT NULL,
                                    `create_time` bigint DEFAULT NULL,
                                    `update_id` bigint DEFAULT NULL,
                                    `update_time` bigint DEFAULT NULL,
                                    PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='学校班级表';

-- ----------------------------
-- Table structure for t_b_school_major
-- ----------------------------
DROP TABLE IF EXISTS `t_b_school_major`;
CREATE TABLE `t_b_school_major` (
                                    `id` bigint NOT NULL,
                                    `school_id` bigint DEFAULT NULL COMMENT '学校id',
                                    `major_code` varchar(100) DEFAULT NULL COMMENT '专业编号',
                                    `major_name` varchar(100) DEFAULT NULL COMMENT '专业名称',
                                    `enable` tinyint DEFAULT '1' COMMENT '是否启用，false：停用，true：启用',
                                    `create_id` bigint DEFAULT NULL,
                                    `create_time` bigint DEFAULT NULL,
                                    `update_id` bigint DEFAULT NULL,
                                    `update_time` bigint DEFAULT NULL,
                                    PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='学校专业表';

-- ----------------------------
-- Table structure for t_b_session
-- ----------------------------
DROP TABLE IF EXISTS `t_b_session`;
CREATE TABLE `t_b_session` (
                               `id` varchar(100) NOT NULL COMMENT '主键',
                               `identity` varchar(100) NOT NULL COMMENT '用户标识',
                               `type` varchar(50) NOT NULL COMMENT '用户类型',
                               `source` varchar(50) NOT NULL COMMENT '访问来源',
                               `platform` varchar(50) NOT NULL COMMENT '设备分类',
                               `device_id` varchar(100) NOT NULL COMMENT '设备标识',
                               `address` varchar(100) NOT NULL COMMENT '登录IP地址',
                               `access_token` varchar(50) NOT NULL COMMENT '访问令牌',
                               `last_access_time` bigint DEFAULT NULL COMMENT '最近访问时间',
                               `last_access_ip` varchar(100) DEFAULT NULL COMMENT '最近访问IP地址',
                               `update_time` bigint DEFAULT NULL COMMENT '令牌更新时间',
                               `expire_time` bigint DEFAULT NULL COMMENT '令牌强制失效时间',
                               `app_source` varchar(50) DEFAULT NULL COMMENT '应用来源',
                               `school_id` bigint DEFAULT NULL COMMENT '学校id',
                               `org_id` bigint DEFAULT NULL COMMENT '机构id',
                               PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='会话表';

-- ----------------------------
-- Table structure for t_b_student
-- ----------------------------
DROP TABLE IF EXISTS `t_b_student`;
CREATE TABLE `t_b_student` (
                               `id` bigint NOT NULL,
                               `school_id` bigint NOT NULL COMMENT '学校id',
                               `name` varchar(100) DEFAULT NULL COMMENT '姓名',
                               `student_code` varchar(100) DEFAULT NULL COMMENT '学号(某学校学生的唯一标识)',
                               `gender` varchar(30) DEFAULT NULL COMMENT '性别，MAN：男，WOMAN：女',
                               `person_img_url` varchar(255) DEFAULT NULL COMMENT '照片地址',
                               PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='基础学生表';

-- ----------------------------
-- Table structure for t_b_sync_task
-- ----------------------------
DROP TABLE IF EXISTS `t_b_sync_task`;
CREATE TABLE `t_b_sync_task` (
                                 `id` bigint NOT NULL COMMENT '主键',
                                 `school_id` bigint DEFAULT NULL COMMENT '学校id',
                                 `org_id` bigint DEFAULT NULL COMMENT '机构id',
                                 `type` varchar(50) NOT NULL COMMENT '任务类型，CALCULATE：分析计算',
                                 `third_relate_id` bigint DEFAULT NULL COMMENT '第三方关联ID',
                                 `obj` varchar(500) DEFAULT NULL COMMENT '实体信息',
                                 `progress` decimal(8,2) DEFAULT NULL COMMENT '进度',
                                 `status` varchar(30) NOT NULL COMMENT '任务状态，INIT：未开始，DATA_VALID：数据校验中，RUNNING：进行中，FINISH：已完成',
                                 `summary` text COMMENT '实时摘要信息',
                                 `result` varchar(30) DEFAULT NULL COMMENT '数据结果，SUCCESS：成功，ERROR：失败',
                                 `create_id` bigint DEFAULT NULL COMMENT '创建人',
                                 `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                 `update_id` bigint DEFAULT NULL COMMENT '修改人',
                                 `update_time` bigint DEFAULT NULL COMMENT '修改时间',
                                 `remark` varchar(500) DEFAULT NULL COMMENT '备注',
                                 `reset_count` int DEFAULT '0' COMMENT '重试次数',
                                 `error_message` varchar(500) DEFAULT NULL COMMENT '人工错误原因',
                                 `report_file_path` varchar(500) DEFAULT NULL COMMENT '报告路径',
                                 `source` varchar(1000) DEFAULT NULL COMMENT '原始数据',
                                 PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='同步任务表';

-- ----------------------------
-- Table structure for t_b_task
-- ----------------------------
DROP TABLE IF EXISTS `t_b_task`;
CREATE TABLE `t_b_task` (
                            `id` bigint NOT NULL COMMENT '主键',
                            `type` varchar(50) NOT NULL COMMENT '任务类型，QUESTION_MISSION_BATCH_CREATE：批量新建命题任务，\nSAMPLE_EXPORT：导出审核样本，\nEXAMINATION_IMPORT：考务数据导入，\nEXAMINATION_EXPORT：考务数据导出，\nPRINT_PDF_DOWNLOAD：批量下载pdf，\nCREATE_PDF：生成pdf',
                            `entity_id` bigint DEFAULT NULL COMMENT '关联业务对象id',
                            `status` varchar(30) NOT NULL COMMENT '任务状态，INIT：未开始，RUNNING：进行中，FINISH：已完成',
                            `summary` varchar(2000) DEFAULT NULL COMMENT '实时摘要信息',
                            `progress` double DEFAULT NULL COMMENT '执行进度',
                            `result` varchar(30) DEFAULT NULL COMMENT '数据结果，SUCCESS：成功，ERROR：失败',
                            `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，false：停用，true：启用',
                            `import_file_name` varchar(100) DEFAULT NULL COMMENT '导入文件名',
                            `import_file_path` varchar(500) DEFAULT NULL COMMENT '导入文件路径',
                            `result_file_path` varchar(500) DEFAULT NULL COMMENT '导出文件路径',
                            `report_file_path` varchar(500) DEFAULT NULL COMMENT '报告路径',
                            `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                            `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                            `school_id` bigint DEFAULT NULL COMMENT '学校id',
                            `org_id` bigint DEFAULT NULL COMMENT '机构id',
                            `print_plan_id` bigint DEFAULT NULL COMMENT '印刷计划id',
                            `remark` mediumtext COMMENT '备注',
                            `obj_name` varchar(500) DEFAULT NULL COMMENT '实体名称',
                            `reset_count` int DEFAULT '0' COMMENT '重试次数',
                            `version` int DEFAULT '0' COMMENT '更新版本号',
                            `error_message` varchar(500) DEFAULT NULL COMMENT '人工错误原因',
                            PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='导入导出任务表';

-- ----------------------------
-- Table structure for t_b_teacher
-- ----------------------------
DROP TABLE IF EXISTS `t_b_teacher`;
CREATE TABLE `t_b_teacher` (
                               `id` bigint NOT NULL,
                               `school_id` bigint NOT NULL COMMENT '学校id',
                               `org_id` bigint DEFAULT NULL COMMENT '机构id',
                               `teacher_name` varchar(45) DEFAULT NULL COMMENT '教师名称',
                               `teacher_code` varchar(45) DEFAULT NULL COMMENT '教师编号（学校内唯一）',
                               `role_type` varchar(45) DEFAULT NULL COMMENT '角色类型',
                               PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='教师表';

-- ----------------------------
-- Table structure for t_g_error
-- ----------------------------
DROP TABLE IF EXISTS `t_g_error`;
CREATE TABLE `t_g_error` (
                             `id` bigint NOT NULL COMMENT '主键',
                             `obj_id` bigint DEFAULT NULL COMMENT '实体id',
                             `summary` mediumtext COMMENT '实时摘要信息',
                             `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                             PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='全局异常错误信息表';

-- ----------------------------
-- Table structure for t_s_auth
-- ----------------------------
DROP TABLE IF EXISTS `t_s_auth`;
CREATE TABLE `t_s_auth` (
                            `id` bigint NOT NULL COMMENT '主键',
                            `school_id` bigint NOT NULL COMMENT '学校id',
                            `access_key` varchar(255) DEFAULT NULL COMMENT 'accessKey',
                            `access_secret` varchar(255) DEFAULT NULL COMMENT 'accessSecret',
                            `description` text COMMENT '离线授权证书',
                            `type` varchar(16) NOT NULL COMMENT '授权类型',
                            `create_id` bigint DEFAULT NULL COMMENT '创建人',
                            `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                            `update_id` bigint DEFAULT NULL COMMENT '更新人',
                            `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                            `expire_time` bigint DEFAULT NULL COMMENT '过期时间',
                            `file` mediumblob COMMENT '文件数据',
                            PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='激活授权配置表';

-- ----------------------------
-- Table structure for t_school_role
-- ----------------------------
DROP TABLE IF EXISTS `t_school_role`;
CREATE TABLE `t_school_role` (
                                 `id` bigint NOT NULL COMMENT '主键',
                                 `school_id` bigint NOT NULL COMMENT '学校id',
                                 `role_id` bigint NOT NULL COMMENT '菜单权限id',
                                 PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='学校角色权限表';

-- ----------------------------
-- Function structure for getRootChildId
-- ----------------------------
delimiter //
DROP FUNCTION IF EXISTS `getRootChildId`;
CREATE FUNCTION `getRootChildId`(rootId bigint) RETURNS varchar(1000) CHARSET utf8mb4
    DETERMINISTIC
BEGIN
    DECLARE ptemp VARCHAR(1000);
    DECLARE ctemp VARCHAR(1000);
    SET ptemp = '#';
    SET ctemp = cast(rootId as CHAR);
    WHILE ctemp IS NOT NULL DO
      SET ptemp = concat(ptemp, ',', ctemp);
SELECT group_concat(id) INTO ctemp FROM sys_org WHERE FIND_IN_SET(parent_id, ctemp) > 0;
END WHILE;
RETURN ptemp;
END;
//
delimiter ;

INSERT INTO `sys_privilege` VALUES (1, '基础配置', 'base', 'MENU', NULL, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (2, '用户管理', 'user', 'MENU', 1, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (4, '组织架构', 'OrganizationManage', 'MENU', 2, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (5, '用户管理', 'UserManage', 'MENU', 2, 2, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (6, '角色管理', 'RoleManage', 'MENU', 2, 3, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (7, '权限管理', 'MenuManage', 'MENU', 129, 6, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (77, '用户管理-查询', '/api/admin/sys/user/list', 'URL', 5, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (78, '用户管理-新增/修改', '/api/admin/sys/user/save', 'URL', 5, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (79, '用户管理-启用/禁用', '/api/admin/sys/user/enable', 'URL', 5, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (80, '用户管理-重置密码', '/api/admin/sys/user/reset_password', 'URL', 5, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (81, '用户管理-导入', '/api/admin/sys/user/import', 'URL', 5, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (82, '用户管理-绑定角色', '/api/admin/sys/user/bind_roles', 'URL', 5, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (83, '角色管理-查询', '/api/admin/sys/role/list', 'URL', 6, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (84, '角色管理-删除', '/api/admin/sys/role/remove', 'URL', 6, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (85, '角色管理-新增/修改', '/api/admin/sys/role/save', 'URL', 6, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (86, '角色管理-启用/禁用', '/api/admin/sys/role/enable', 'URL', 6, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (87, '用户管理-用户已绑定角色列表', '/api/admin/sys/role/get_user_roles', 'URL', 5, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (88, '权限管理-查询', '/api/admin/sys/privilege/list', 'URL', 7, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (89, '权限管理-新增/修改', '/api/admin/sys/privilege/save', 'URL', 7, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (90, '权限管理-删除', '/api/admin/sys/privilege/remove', 'URL', 7, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (91, '角色管理-角色已绑定权限列表', '/api/admin/sys/privilege/get_role_privileges', 'URL', 6, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (92, '组织机构-查询', '/api/admin/sys/org/list', 'URL', 4, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (93, '组织机构-新增/修改', '/api/admin/sys/org/save', 'URL', 4, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (94, '组织机构-删除', '/api/admin/sys/org/remove', 'URL', 4, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (95, '组织机构-启用/禁用', '/api/admin/sys/org/enable', 'URL', 4, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (129, '公共接口', 'common', 'MENU', NULL, 31, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (130, '用户登录相关', 'login', 'MENU', 129, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (131, '用户登录', '/api/admin/common/login', 'URL', 130, 1, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (132, '查询用户权限', '/api/admin/common/get_menu', 'URL', 130, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (134, '修改密码', '/api/admin/sys/user/update_password', 'URL', 130, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (135, '用户登出', '/api/admin/common/logout', 'URL', 130, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (144, '文件相关', 'file', 'MENU', 129, 3, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (145, '文件上传', '/api/admin/common/file/upload', 'URL', 144, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (146, '文件查询', '/api/admin/common/file/get_one', 'URL', 144, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (147, '文件下载', '/api/admin/common/file/download', 'URL', 144, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (148, '文件预览', '/api/admin/common/file/preview', 'URL', 144, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (149, '系统相关', 'sys', 'MENU', 129, 4, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (151, '查询学校列表', '/api/admin/common/school/list', 'URL', 149, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (152, '查询系统参数', '/api/admin/common/sys_config/get_one', 'URL', 149, 3, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (153, '根据学校代码查询学校信息', '/api/admin/common/school/query_by_school_code', 'URL', 149, 4, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (180, '角色管理-查询可设置权限列表', '/api/admin/sys/privilege/list_auth', 'URL', 6, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (184, '用户管理-查询用户可绑定角色', '/api/admin/sys/role/list_to_user', 'URL', 5, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (250, '教研分析', 'report', 'MENU', NULL, 4, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (251, '学期列表', '/api/report/school/common/list_semester', 'URL', 250, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (252, '考试列表', '/api/report/school/common/list_exam', 'URL', 250, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (253, '课程列表', '/api/report/school/common/list_course', 'URL', 250, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (254, '开课学院列表', '/api/report/school/common/list_teach_college', 'URL', 250, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (255, '考查学院列表', '/api/report/school/common/list_inspect_college', 'URL', 250, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (256, '教师列表', '/api/report/school/common/list_teacher', 'URL', 250, 6, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (257, '班级列表', '/api/report/school/common/list_class', 'URL', 250, 7, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (258, '数据驾驶舱', 'cockpit', 'MENU', 250, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (259, '全校考试总览', 'SchoolExam', 'MENU', 258, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (260, '全校考试概况', '/api/report/school/exam/get_overview', 'URL', 259, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (261, '全校课程考试分析汇总', '/api/report/school/exam/get_course_summary', 'URL', 259, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (262, '全校课程考试分析汇总-导出课程分析Excel报表', '/api/report/school/exam/export_course_summary', 'URL', 259, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (263, '全校课程考试分析', 'SchoolCourse', 'MENU', 258, 2, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (264, '考试概况', '/api/report/school/course/get_overview', 'URL', 263, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (265, '分数段分布情况', '/api/report/school/course/get_grade_distribute', 'URL', 263, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (266, '教师各课堂成绩排名', '/api/report/school/course/list_teacher_class_rank', 'URL', 263, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (267, '导出课程分析报表', '/api/report/school/course/export_data_analysis', 'URL', 263, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (268, '全校教师各课堂成绩排名', 'SchoolTeacherClassScore', 'MENU', 258, 3, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (269, '查询', '/api/report/school/teacher/list_all_teacher_class_rank', 'URL', 268, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (270, '学生', 'student', 'MENU', 250, 2, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (271, '开课成绩查询', 'TeachCollegeScore', 'MENU', 270, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (272, '查询', '/api/report/school/student/list_teach_college_result', 'URL', 271, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (273, '导入异常考试类型', '/api/report/school/student/import_abnormal_data', 'URL', 271, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (274, '导出查询结果', '/api/report/school/student/export_teach_college_result', 'URL', 271, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (275, '考查成绩查询', 'InspectCollegeScore', 'MENU', 270, 2, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (276, '查询', '/api/report/school/student/list_inspect_college_result', 'URL', 275, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (277, '导出查询结果', '/api/report/school/student/export_inspect_college_result', 'URL', 275, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (278, '教师', 'teacher', 'MENU', 250, 3, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (279, '教师各课堂成绩排名', 'TeacherClassScore', 'MENU', 278, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (280, '查询', '/api/report/school/teacher/list_class_rank', 'URL', 279, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (281, '我的课堂', 'MyClassScore', 'MENU', 278, 2, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (282, '查询', '/api/report/school/teacher/list_myclass_rank', 'URL', 281, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (283, '课程', 'course', 'MENU', 250, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (284, '考查课程考试总览', 'InspectCourseExamOverview', 'MENU', 283, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (285, '考查课程考试分析', 'InspectCourseExamAnalysis', 'MENU', 283, 2, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (286, '开课课程考试总览', 'OwnerCourseExamOverview', 'MENU', 283, 3, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (287, '开课课程考试分析', 'OwnerCourseExamAnalysis', 'MENU', 283, 4, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (288, '赋分管理', 'AssignScoreAction', 'MENU', 283, 5, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (289, '考查课程考试总览-公共课概况接口', '/api/report/course/survey_inspect/view', 'URL', 284, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (290, '考查课程考试分析-图表接口', '/api/report/course/survey_inspect_analyse/view', 'URL', 285, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (291, '开课课程考试总览-列表接口', '/api/report/course/survey_teacher/list', 'URL', 286, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (292, '开课课程考试总览-图表接口', '/api/report/course/survey_teacher/view', 'URL', 287, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (293, '赋分管理-列表接口', '/api/report/course/survey_aspoints/list', 'URL', 288, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (294, '赋分管理-开始&重新试算接口', '/api/report/course/survey_aspoints/calculation', 'URL', 288, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (295, '赋分管理-预览&查看接口', '/api/report/course/survey_aspoints/view', 'URL', 288, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (296, '赋分管理-发布&撤回接口', '/api/report/course/survey_aspoints/publish', 'URL', 288, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (297, '赋分管理-保存&保存并发布接口', '/api/report/course/survey_aspoints/save', 'URL', 288, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (298, '学生报告预览', 'StudentReportManage', 'MENU', 270, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (299, '学生报告预览-列表接口', '/api/report/exam_student/report/list', 'URL', 298, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (300, '学生报告-个人成绩总览接口', '/api/report/exam_student/report/result', 'URL', 298, 2, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (301, '学生报告接口', '/api/report/exam_student/report/mark', 'URL', 298, 3, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (302, '获取服务器时间', '/api/admin/common/get_system_time', 'URL', 129, 2, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (303, '开课课程考试总览-教师各课堂成绩排名接口', '/api/report/course/survey_teacher/teacher_view', 'URL', 287, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (304, '开课课程考试总览-试题难度得分情况分析接口', '/api/report/course/survey_teacher/question_view', 'URL', 287, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (305, '开课课程考试总览-各难度水平题目上的作答分析接口', '/api/report/course/survey_teacher/question_answer_view', 'URL', 287, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (306, '常规配置信息导入：试卷难易度和学院排名百分等级档位', '/api/report/datasource/config/import', 'URL', 250, 100, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (307, '知识点导入模板', '/api/report/datasource/dimension/import', 'URL', 250, 101, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (308, '考试作答信息导入模板', '/api/report/datasource/examAnswer/import', 'URL', 250, 102, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (309, '模块和等级导入接口', '/api/report/datasource/moduleLevel/import', 'URL', 250, 103, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (310, '公式赋分操作', '/api/report/datasource/normal/assignScore', 'URL', 250, 104, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (311, '试卷数据导入', '/api/report/datasource/paper/import', 'URL', 250, 105, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (312, '特殊科目赋分操作', '/api/report/datasource/special/assignScore', 'URL', 250, 106, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (313, '数据计算', '/api/report/analyze/calculate', 'URL', 250, 107, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (314, '任课老师-课程列表', '/api/report/school/common/list_course_teach', 'URL', 250, 8, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (315, '开课课程考试总览-列表导出', '/api/report/course/survey_teacher/export', 'URL', 286, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (316, '考务数据同步', '/api/report/datasource/examination/sync', 'URL', 250, 109, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (317, '创建课程', '/api/report/datasource/course/create', 'URL', 250, 108, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (318, '公共实际（应届不缺考）学院查询', '/api/report/school/common/list_inspect_college_not_absent', 'URL', 250, 9, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (319, '公共实际（应届不缺考）教师查询', '/api/report/school/common/list_teacher_not_absent', 'URL', 250, 10, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (320, '导出课程分析接口', '/api/report/course/survey_teacher/course_export', 'URL', 250, 9, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (321, '武大学生开放鉴权接口', '/api/report/wuda/open/authentication', 'URL', 250, 11, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (322, '异常考生-批量处理', '/api/report/datasource/exception_student/batch_dispose', 'URL', 250, 110, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (323, '特殊科目合并题目处理', '/api/report/datasource/special/combine', 'URL', 250, 111, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (324, '异常答案/分数处理（先处理异常后赋分）', '/api/report/datasource/exception_answer/score_dispose', 'URL', 250, 112, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (325, '武大学生开放鉴权用户临时授权登录接口', '/api/report/wuda/open/user/authentication/login', 'URL', 250, 14, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (326, '武大学生开放鉴权测试接口', '/api/report/wuda/open/user/authentication/test', 'URL', 250, 15, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (327, '武大用户鉴权退出接口', '/api/report/wuda/open/user/authentication/logout', 'URL', 250, 16, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (328, '留学生异常处理', '/api/report/datasource/exception_student/dispose', 'URL', 250, 113, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (329, '异常考生-按学号处理', '/api/report/datasource/exception_student/dispose_by_student_code', 'URL', 250, 17, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (330, '我的课堂考试分析', 'MyClassExamView', 'MENU', 278, 2, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (331, '我的课堂考试分析-图表接口', '/api/report/school/teacher/view', 'URL', 330, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (332, '我的课堂考试分析-试题难度得分情况分析接口', '/api/report/school/teacher/question_view', 'URL', 330, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (333, '我的课堂考试分析-各难度水平题目上的作答分析接口', '/api/report/school/teacher/question_answer_view', 'URL', 330, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (334, '授权配置', 'AuthManage', 'MENU', 1, 6, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` VALUES (335, '授权配置-导出硬件信息按钮', 'Export', 'BUTTON', 334, 1, 'AUTH', '336,337,338', 1, 0, 0);
INSERT INTO `sys_privilege` VALUES (336, '导出硬件信息', '/api/admin/auth/export/device/info', 'URL', 335, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (337, '离线激活', '/api/admin/auth/offline/activation', 'URL', 335, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (338, '查询激活信息', '/api/admin/auth/select', 'URL', 335, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (339, '学校管理', 'SchoolManage', 'MENU', 1, 7, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` VALUES (340, '查询', 'Select', 'BUTTON', 339, 1, 'AUTH', '342', 1, 0, 0);
INSERT INTO `sys_privilege` VALUES (341, '查询条件', 'Condition', 'CONDITION', 339, 2, 'AUTH', '342', 1, 0, 0);
INSERT INTO `sys_privilege` VALUES (342, '查询', '/api/admin/commom/school/list', 'URL', 339, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (343, '机构同步', 'Sync', 'BUTTON', 339, 3, 'AUTH', '344', 1, 0, 0);
INSERT INTO `sys_privilege` VALUES (344, '机构同步', '/api/admin/auth/sync/org', 'URL', 339, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (345, '编辑', 'Edit', 'LINK', 339, 1, 'AUTH', '346', 1, 0, 0);
INSERT INTO `sys_privilege` VALUES (346, '编辑', '/api/admin/commom/school/update', 'URL', 339, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (347, '同步配置保存', '/api/report/set/sync/save', 'URL', 339, 4, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (348, '同步配置查询', '/api/report/set/sync/select', 'URL', 339, 5, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (349, '设置', 'Set', 'LINK', 339, 2, 'AUTH', '347,348', 1, 0, 0);
INSERT INTO `sys_privilege` VALUES (138647380756004864, '赋分即时预览', 'AssignScoreActionTry', 'MENU', 283, 6, NULL, NULL, 1, 1, 1);

INSERT INTO `sys_role` VALUES (1, NULL, '系统管理员', 1, 'ADMIN', NULL, NULL, NULL, NULL, 1, NULL, NULL, NULL);
INSERT INTO `sys_role` VALUES (2, NULL, '教务处老师', 1, 'OFFICE_TEACHER', NULL, NULL, NULL, NULL, 1, NULL, NULL, NULL);
INSERT INTO `sys_role` VALUES (3, NULL, '学院院长', 1, 'PRESIDENT', NULL, NULL, 1, 1623997638797, 1, NULL, NULL, NULL);
INSERT INTO `sys_role` VALUES (4, NULL, '任课老师', 1, 'TEACHER', NULL, NULL, NULL, NULL, 1, NULL, NULL, NULL);
INSERT INTO `sys_role` VALUES (5, NULL, '学校管理员', 1, 'SCHOOL_ADMIN', NULL, NULL, NULL, NULL, 1, NULL, NULL, NULL);

INSERT INTO `sys_role_privilege` VALUES (1, 1, 1, 1, null);
INSERT INTO `sys_role_privilege` VALUES (2, 1, 2, 1, null);
INSERT INTO `sys_role_privilege` VALUES (3, 1, 4, 1, null);
INSERT INTO `sys_role_privilege` VALUES (4, 1, 5, 1, null);
INSERT INTO `sys_role_privilege` VALUES (5, 1, 6, 1, null);
INSERT INTO `sys_role_privilege` VALUES (6, 1, 7, 1, null);
INSERT INTO `sys_role_privilege` VALUES (8, 1, 130, 1, null);
INSERT INTO `sys_role_privilege` VALUES (9, 1, 144, 1, null);
INSERT INTO `sys_role_privilege` VALUES (10, 1, 149, 1, null);
INSERT INTO `sys_role_privilege` VALUES (11, 1, 250, 1, null);
INSERT INTO `sys_role_privilege` VALUES (12, 1, 258, 1, null);
INSERT INTO `sys_role_privilege` VALUES (13, 1, 259, 1, null);
INSERT INTO `sys_role_privilege` VALUES (14, 1, 263, 1, null);
INSERT INTO `sys_role_privilege` VALUES (15, 1, 268, 1, null);
INSERT INTO `sys_role_privilege` VALUES (16, 1, 270, 1, null);
INSERT INTO `sys_role_privilege` VALUES (17, 1, 271, 1, null);
INSERT INTO `sys_role_privilege` VALUES (18, 1, 275, 1, null);
INSERT INTO `sys_role_privilege` VALUES (19, 1, 278, 1, null);
INSERT INTO `sys_role_privilege` VALUES (20, 1, 279, 1, null);
INSERT INTO `sys_role_privilege` VALUES (21, 1, 281, 1, null);
INSERT INTO `sys_role_privilege` VALUES (22, 1, 283, 1, null);
INSERT INTO `sys_role_privilege` VALUES (23, 1, 284, 1, null);
INSERT INTO `sys_role_privilege` VALUES (24, 1, 285, 1, null);
INSERT INTO `sys_role_privilege` VALUES (25, 1, 286, 1, null);
INSERT INTO `sys_role_privilege` VALUES (26, 1, 287, 1, null);
INSERT INTO `sys_role_privilege` VALUES (27, 1, 288, 1, null);
INSERT INTO `sys_role_privilege` VALUES (28, 1, 298, 1, null);
INSERT INTO `sys_role_privilege` VALUES (29, 1, 330, 1, null);
INSERT INTO `sys_role_privilege` VALUES (31, 1, 77, 1, null);
INSERT INTO `sys_role_privilege` VALUES (32, 1, 78, 1, null);
INSERT INTO `sys_role_privilege` VALUES (33, 1, 79, 1, null);
INSERT INTO `sys_role_privilege` VALUES (34, 1, 80, 1, null);
INSERT INTO `sys_role_privilege` VALUES (35, 1, 81, 1, null);
INSERT INTO `sys_role_privilege` VALUES (36, 1, 82, 1, null);
INSERT INTO `sys_role_privilege` VALUES (37, 1, 83, 1, null);
INSERT INTO `sys_role_privilege` VALUES (38, 1, 84, 1, null);
INSERT INTO `sys_role_privilege` VALUES (39, 1, 85, 1, null);
INSERT INTO `sys_role_privilege` VALUES (40, 1, 86, 1, null);
INSERT INTO `sys_role_privilege` VALUES (41, 1, 87, 1, null);
INSERT INTO `sys_role_privilege` VALUES (42, 1, 88, 1, null);
INSERT INTO `sys_role_privilege` VALUES (43, 1, 89, 1, null);
INSERT INTO `sys_role_privilege` VALUES (44, 1, 90, 1, null);
INSERT INTO `sys_role_privilege` VALUES (45, 1, 91, 1, null);
INSERT INTO `sys_role_privilege` VALUES (46, 1, 92, 1, null);
INSERT INTO `sys_role_privilege` VALUES (47, 1, 93, 1, null);
INSERT INTO `sys_role_privilege` VALUES (48, 1, 94, 1, null);
INSERT INTO `sys_role_privilege` VALUES (49, 1, 95, 1, null);
INSERT INTO `sys_role_privilege` VALUES (50, 1, 131, 1, null);
INSERT INTO `sys_role_privilege` VALUES (51, 1, 132, 1, null);
INSERT INTO `sys_role_privilege` VALUES (52, 1, 134, 1, null);
INSERT INTO `sys_role_privilege` VALUES (53, 1, 135, 1, null);
INSERT INTO `sys_role_privilege` VALUES (54, 1, 145, 1, null);
INSERT INTO `sys_role_privilege` VALUES (55, 1, 146, 1, null);
INSERT INTO `sys_role_privilege` VALUES (56, 1, 147, 1, null);
INSERT INTO `sys_role_privilege` VALUES (57, 1, 148, 1, null);
INSERT INTO `sys_role_privilege` VALUES (58, 1, 151, 1, null);
INSERT INTO `sys_role_privilege` VALUES (59, 1, 152, 1, null);
INSERT INTO `sys_role_privilege` VALUES (60, 1, 153, 1, null);
INSERT INTO `sys_role_privilege` VALUES (61, 1, 180, 1, null);
INSERT INTO `sys_role_privilege` VALUES (62, 1, 184, 1, null);
INSERT INTO `sys_role_privilege` VALUES (63, 1, 251, 1, null);
INSERT INTO `sys_role_privilege` VALUES (64, 1, 252, 1, null);
INSERT INTO `sys_role_privilege` VALUES (65, 1, 253, 1, null);
INSERT INTO `sys_role_privilege` VALUES (66, 1, 254, 1, null);
INSERT INTO `sys_role_privilege` VALUES (67, 1, 255, 1, null);
INSERT INTO `sys_role_privilege` VALUES (68, 1, 256, 1, null);
INSERT INTO `sys_role_privilege` VALUES (69, 1, 257, 1, null);
INSERT INTO `sys_role_privilege` VALUES (70, 1, 260, 1, null);
INSERT INTO `sys_role_privilege` VALUES (71, 1, 261, 1, null);
INSERT INTO `sys_role_privilege` VALUES (72, 1, 262, 1, null);
INSERT INTO `sys_role_privilege` VALUES (73, 1, 264, 1, null);
INSERT INTO `sys_role_privilege` VALUES (74, 1, 265, 1, null);
INSERT INTO `sys_role_privilege` VALUES (75, 1, 266, 1, null);
INSERT INTO `sys_role_privilege` VALUES (76, 1, 267, 1, null);
INSERT INTO `sys_role_privilege` VALUES (77, 1, 269, 1, null);
INSERT INTO `sys_role_privilege` VALUES (78, 1, 272, 1, null);
INSERT INTO `sys_role_privilege` VALUES (79, 1, 273, 1, null);
INSERT INTO `sys_role_privilege` VALUES (80, 1, 274, 1, null);
INSERT INTO `sys_role_privilege` VALUES (81, 1, 276, 1, null);
INSERT INTO `sys_role_privilege` VALUES (82, 1, 277, 1, null);
INSERT INTO `sys_role_privilege` VALUES (83, 1, 280, 1, null);
INSERT INTO `sys_role_privilege` VALUES (84, 1, 282, 1, null);
INSERT INTO `sys_role_privilege` VALUES (85, 1, 289, 1, null);
INSERT INTO `sys_role_privilege` VALUES (86, 1, 290, 1, null);
INSERT INTO `sys_role_privilege` VALUES (87, 1, 291, 1, null);
INSERT INTO `sys_role_privilege` VALUES (88, 1, 292, 1, null);
INSERT INTO `sys_role_privilege` VALUES (89, 1, 293, 1, null);
INSERT INTO `sys_role_privilege` VALUES (90, 1, 294, 1, null);
INSERT INTO `sys_role_privilege` VALUES (91, 1, 295, 1, null);
INSERT INTO `sys_role_privilege` VALUES (92, 1, 296, 1, null);
INSERT INTO `sys_role_privilege` VALUES (93, 1, 297, 1, null);
INSERT INTO `sys_role_privilege` VALUES (94, 1, 299, 1, null);
INSERT INTO `sys_role_privilege` VALUES (95, 1, 300, 1, null);
INSERT INTO `sys_role_privilege` VALUES (96, 1, 301, 1, null);
INSERT INTO `sys_role_privilege` VALUES (97, 1, 302, 1, null);
INSERT INTO `sys_role_privilege` VALUES (98, 1, 303, 1, null);
INSERT INTO `sys_role_privilege` VALUES (99, 1, 304, 1, null);
INSERT INTO `sys_role_privilege` VALUES (100, 1, 305, 1, null);
INSERT INTO `sys_role_privilege` VALUES (101, 1, 306, 1, null);
INSERT INTO `sys_role_privilege` VALUES (102, 1, 307, 1, null);
INSERT INTO `sys_role_privilege` VALUES (103, 1, 308, 1, null);
INSERT INTO `sys_role_privilege` VALUES (104, 1, 309, 1, null);
INSERT INTO `sys_role_privilege` VALUES (105, 1, 310, 1, null);
INSERT INTO `sys_role_privilege` VALUES (106, 1, 311, 1, null);
INSERT INTO `sys_role_privilege` VALUES (107, 1, 312, 1, null);
INSERT INTO `sys_role_privilege` VALUES (108, 1, 313, 1, null);
INSERT INTO `sys_role_privilege` VALUES (109, 1, 314, 1, null);
INSERT INTO `sys_role_privilege` VALUES (110, 1, 315, 1, null);
INSERT INTO `sys_role_privilege` VALUES (111, 1, 316, 1, null);
INSERT INTO `sys_role_privilege` VALUES (112, 1, 317, 1, null);
INSERT INTO `sys_role_privilege` VALUES (113, 1, 318, 1, null);
INSERT INTO `sys_role_privilege` VALUES (114, 1, 319, 1, null);
INSERT INTO `sys_role_privilege` VALUES (115, 1, 320, 1, null);
INSERT INTO `sys_role_privilege` VALUES (116, 1, 321, 1, null);
INSERT INTO `sys_role_privilege` VALUES (117, 1, 322, 1, null);
INSERT INTO `sys_role_privilege` VALUES (118, 1, 323, 1, null);
INSERT INTO `sys_role_privilege` VALUES (119, 1, 324, 1, null);
INSERT INTO `sys_role_privilege` VALUES (120, 1, 325, 1, null);
INSERT INTO `sys_role_privilege` VALUES (121, 1, 326, 1, null);
INSERT INTO `sys_role_privilege` VALUES (122, 1, 327, 1, null);
INSERT INTO `sys_role_privilege` VALUES (123, 1, 328, 1, null);
INSERT INTO `sys_role_privilege` VALUES (124, 1, 329, 1, null);
INSERT INTO `sys_role_privilege` VALUES (125, 1, 331, 1, null);
INSERT INTO `sys_role_privilege` VALUES (126, 1, 332, 1, null);
INSERT INTO `sys_role_privilege` VALUES (127, 1, 333, 1, null);
INSERT INTO `sys_role_privilege` VALUES (128, 1, 334, 1, null);
INSERT INTO `sys_role_privilege` VALUES (129, 1, 335, 1, null);
INSERT INTO `sys_role_privilege` VALUES (130, 1, 336, 1, null);
INSERT INTO `sys_role_privilege` VALUES (131, 1, 337, 1, null);
INSERT INTO `sys_role_privilege` VALUES (132, 1, 338, 1, null);
INSERT INTO `sys_role_privilege` VALUES (133, 2, 258, 1, null);
INSERT INTO `sys_role_privilege` VALUES (134, 2, 259, 1, null);
INSERT INTO `sys_role_privilege` VALUES (135, 2, 260, 1, null);
INSERT INTO `sys_role_privilege` VALUES (136, 2, 261, 1, null);
INSERT INTO `sys_role_privilege` VALUES (137, 2, 262, 1, null);
INSERT INTO `sys_role_privilege` VALUES (138, 2, 263, 1, null);
INSERT INTO `sys_role_privilege` VALUES (139, 2, 264, 1, null);
INSERT INTO `sys_role_privilege` VALUES (140, 2, 265, 1, null);
INSERT INTO `sys_role_privilege` VALUES (141, 2, 266, 1, null);
INSERT INTO `sys_role_privilege` VALUES (142, 2, 267, 1, null);
INSERT INTO `sys_role_privilege` VALUES (143, 2, 268, 1, null);
INSERT INTO `sys_role_privilege` VALUES (144, 2, 269, 1, null);
INSERT INTO `sys_role_privilege` VALUES (145, 2, 250, 1, null);
INSERT INTO `sys_role_privilege` VALUES (146, 5, 1, 1, null);
INSERT INTO `sys_role_privilege` VALUES (147, 5, 2, 1, null);
INSERT INTO `sys_role_privilege` VALUES (148, 5, 4, 1, null);
INSERT INTO `sys_role_privilege` VALUES (149, 5, 93, 1, null);
INSERT INTO `sys_role_privilege` VALUES (150, 5, 94, 1, null);
INSERT INTO `sys_role_privilege` VALUES (151, 5, 95, 1, null);
INSERT INTO `sys_role_privilege` VALUES (152, 5, 5, 1, null);
INSERT INTO `sys_role_privilege` VALUES (153, 5, 77, 1, null);
INSERT INTO `sys_role_privilege` VALUES (154, 5, 78, 1, null);
INSERT INTO `sys_role_privilege` VALUES (155, 5, 79, 1, null);
INSERT INTO `sys_role_privilege` VALUES (156, 5, 80, 1, null);
INSERT INTO `sys_role_privilege` VALUES (157, 5, 81, 1, null);
INSERT INTO `sys_role_privilege` VALUES (158, 5, 87, 1, null);
INSERT INTO `sys_role_privilege` VALUES (159, 5, 82, 1, null);
INSERT INTO `sys_role_privilege` VALUES (160, 5, 184, 1, null);
INSERT INTO `sys_role_privilege` VALUES (161, 5, 6, 1, null);
INSERT INTO `sys_role_privilege` VALUES (162, 5, 83, 1, null);
INSERT INTO `sys_role_privilege` VALUES (163, 5, 84, 1, null);
INSERT INTO `sys_role_privilege` VALUES (164, 5, 85, 1, null);
INSERT INTO `sys_role_privilege` VALUES (165, 5, 91, 1, null);
INSERT INTO `sys_role_privilege` VALUES (166, 5, 86, 1, null);
INSERT INTO `sys_role_privilege` VALUES (167, 5, 180, 1, null);
INSERT INTO `sys_role_privilege` VALUES (168, 5, 298, 1, null);
INSERT INTO `sys_role_privilege` VALUES (169, 5, 299, 1, null);
INSERT INTO `sys_role_privilege` VALUES (170, 5, 300, 1, null);
INSERT INTO `sys_role_privilege` VALUES (171, 5, 301, 1, null);
INSERT INTO `sys_role_privilege` VALUES (172, 5, 250, 1, null);
INSERT INTO `sys_role_privilege` VALUES (173, 5, 270, 1, null);
INSERT INTO `sys_role_privilege` VALUES (174, 3, 283, 1, null);
INSERT INTO `sys_role_privilege` VALUES (175, 3, 284, 1, null);
INSERT INTO `sys_role_privilege` VALUES (176, 3, 289, 1, null);
INSERT INTO `sys_role_privilege` VALUES (177, 3, 285, 1, null);
INSERT INTO `sys_role_privilege` VALUES (178, 3, 290, 1, null);
INSERT INTO `sys_role_privilege` VALUES (179, 3, 286, 1, null);
INSERT INTO `sys_role_privilege` VALUES (180, 3, 291, 1, null);
INSERT INTO `sys_role_privilege` VALUES (181, 3, 287, 1, null);
INSERT INTO `sys_role_privilege` VALUES (182, 3, 292, 1, null);
INSERT INTO `sys_role_privilege` VALUES (183, 3, 303, 1, null);
INSERT INTO `sys_role_privilege` VALUES (184, 3, 304, 1, null);
INSERT INTO `sys_role_privilege` VALUES (185, 3, 305, 1, null);
INSERT INTO `sys_role_privilege` VALUES (186, 3, 288, 1, null);
INSERT INTO `sys_role_privilege` VALUES (187, 3, 293, 1, null);
INSERT INTO `sys_role_privilege` VALUES (188, 3, 294, 1, null);
INSERT INTO `sys_role_privilege` VALUES (189, 3, 295, 1, null);
INSERT INTO `sys_role_privilege` VALUES (190, 3, 296, 1, null);
INSERT INTO `sys_role_privilege` VALUES (191, 3, 297, 1, null);
INSERT INTO `sys_role_privilege` VALUES (192, 3, 271, 1, null);
INSERT INTO `sys_role_privilege` VALUES (193, 3, 272, 1, null);
INSERT INTO `sys_role_privilege` VALUES (194, 3, 273, 1, null);
INSERT INTO `sys_role_privilege` VALUES (195, 3, 274, 1, null);
INSERT INTO `sys_role_privilege` VALUES (196, 3, 275, 1, null);
INSERT INTO `sys_role_privilege` VALUES (197, 3, 276, 1, null);
INSERT INTO `sys_role_privilege` VALUES (198, 3, 277, 1, null);
INSERT INTO `sys_role_privilege` VALUES (199, 3, 279, 1, null);
INSERT INTO `sys_role_privilege` VALUES (200, 3, 280, 1, null);
INSERT INTO `sys_role_privilege` VALUES (201, 3, 250, 1, null);
INSERT INTO `sys_role_privilege` VALUES (202, 3, 270, 1, null);
INSERT INTO `sys_role_privilege` VALUES (203, 3, 278, 1, null);
INSERT INTO `sys_role_privilege` VALUES (204, 3, 315, 1, null);
INSERT INTO `sys_role_privilege` VALUES (205, 4, 281, 1, null);
INSERT INTO `sys_role_privilege` VALUES (206, 4, 282, 1, null);
INSERT INTO `sys_role_privilege` VALUES (207, 4, 330, 1, null);
INSERT INTO `sys_role_privilege` VALUES (208, 4, 331, 1, null);
INSERT INTO `sys_role_privilege` VALUES (209, 4, 332, 1, null);
INSERT INTO `sys_role_privilege` VALUES (210, 4, 333, 1, null);
INSERT INTO `sys_role_privilege` VALUES (211, 4, 250, 1, null);
INSERT INTO `sys_role_privilege` VALUES (212, 4, 278, 1, null);

INSERT INTO `sys_user` VALUES (1, NULL, 'sysadmin', 'sysadmin', 'MTIzNDU2', NULL, NULL, 1, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, NULL, NULL, NULL);

INSERT INTO `sys_user_role` VALUES (1, 1, 1, NULL, 1);

INSERT INTO `sys_config` VALUES (1, NULL, NULL, 'sys.user.initPassword', '初始密码', '123456', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (2, NULL, NULL, 'sys.warning.days', '预警天数', '3', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (3, NULL, NULL, 'sys.code.enable', '是否启用短信验证码', 'true', 'true-启用，false-禁用', 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (4, NULL, NULL, 'sys.message.enable', '是否启用短信消息提示', 'true', 'true-启用，false-禁用', 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (5, NULL, NULL, 'sys.message.resendCount', '失败短信重试次数', '3', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (9, NULL, NULL, 'admin.logo.url', '超级管理员logo', 'http://qmth-test.oss-cn-shenzhen.aliyuncs.com/frontend/wx_logo.png', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (10, NULL, NULL, 'attachment.type', '附件类型', '[.xlsx,.xls,.doc,.docx,.pdf,.jpg,.jpeg,.png,.html,.zip,.mp3,.wav,.dll,.exe,.ftl,.bpmn,.xml]', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (11, NULL, NULL, 'attachment.length', '附件名称长度', '100', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (12, NULL, NULL, 'attachment.size', '附件大小 ', '200', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (13, NULL, NULL, 'auto.create.pdf.reset.max.count', '自动重试生成pdf失败最大次数', '5', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (14, NULL, NULL, 'thread.pool.core.size', '自定义线程池大小', '1', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (15, NULL, NULL, 'custom.thread.pool.core.size', '是否自定义线程池大小', 'true', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (16, NULL, NULL, 'session.active', 'session会话时长', '2h', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (17, NULL, NULL, 'cloudmark.host.url', '云阅卷地址', 'http://192.168.10.224:80', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (18, NULL, NULL, 'sms.normal.code', '万能短信验证码', 'qmth', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (19, NULL, NULL, 'code.expired.time', '短信验证码有效时间', '2', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (20, NULL, NULL, 'code.send.interval', '短信验证码发送间隔', '60', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (21, NULL, NULL, 'teachcloud.report.student.url', '学生报告地址', 'http://127.0.0.1:7500/#/student-report/', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (22, NULL, NULL, 'teachcloud.report.login.url', '单点登录地址', 'http://127.0.0.1:7500/#/login-authen/', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (23, NULL, NULL, 'teachcloud.print.open.callback.pwd', '分布式印刷回调密码', '123456', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (24, NULL, NULL, 'sms.sign.name', '短信签名', '逸教云', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (25, NULL, NULL, 'sms.tpl.code', '登录短信验证码', 'SMS_147416565', NULL, 1, 2, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (26, NULL, NULL, 'sys.txt.charset', 'txt文件编码', 'utf8', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (27, NULL, NULL, 'teachcloud.print.host.default.url', '分布式印刷地址', 'http://127.0.0.1:7400', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (28, NULL, NULL, 'teachcloud.report.location.url', '报告和登录地址', 'http://127.0.0.1:7500', NULL, 1, 2, 1, NULL, NULL, NULL);
