USE teachcloud_db;

-- ----------------------------
-- Table structure for basic_attachment
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_attachment` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint DEFAULT NULL COMMENT '学校id',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `name` varchar(100) NOT NULL COMMENT '文件名',
        `type` varchar(10) NOT NULL COMMENT '文件类型',
        `size` int NOT NULL COMMENT '单位（KB）',
        `md5` varchar(32) NOT NULL COMMENT '文件md5值',
        `path` varchar(2000) NOT NULL COMMENT '存储路径',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        `obj_id` bigint DEFAULT NULL COMMENT '关联业务id',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='所有附件记录表';


-- ----------------------------
-- Table structure for basic_campus
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_campus` (
        `id` bigint NOT NULL COMMENT '校区id',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `campus_name` varchar(64) DEFAULT NULL COMMENT '校区名称',
        `campus_code` varchar(64) DEFAULT NULL COMMENT '校区编码',
        `print_house_id` bigint DEFAULT NULL COMMENT '印刷室id',
        `enable` tinyint DEFAULT '1' COMMENT '是否可用',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `create_id` bigint DEFAULT NULL COMMENT '创建人id',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人id',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE,
        UNIQUE KEY `campus_schoolId_code_idx` (`school_id`,`campus_code`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='基础校区表';


-- ----------------------------
-- Table structure for basic_card_rule
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_card_rule` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `name` varchar(100) NOT NULL COMMENT '题卡规则名称',
        `exam_number_style` varchar(5) NOT NULL COMMENT 'PRINT-印刷条码，PASTE-粘贴条码，FILL-卡号填涂',
        `exam_number_digit` int DEFAULT NULL COMMENT '考号位数',
        `paper_type` varchar(5) NOT NULL COMMENT 'PRINT-印刷条码，FILL-卡号填涂',
        `exam_absent` tinyint NOT NULL COMMENT '0-禁用，1-启用',
        `write_sign` tinyint NOT NULL COMMENT '0-禁用，1-启用',
        `discipline` tinyint(1) DEFAULT NULL COMMENT '违纪填涂（1：是，0：否）',
        `undertaking_enable` TINYINT(1) NULL COMMENT '启用考生承诺书',
        `undertaking_body` VARCHAR(300) NULL COMMENT '考生承诺书内容',
        `required_fields` text NOT NULL COMMENT '必选字段',
        `extend_fields` text NOT NULL COMMENT '扩展字段',
        `title_rule` varchar(200) NOT NULL COMMENT '题卡标题规则',
        `attention` text NOT NULL COMMENT '注意事项',
        `objective_attention` text COMMENT '客观题注意事项',
        `subjective_attention` text COMMENT '主观题注意事项',
        `enable` tinyint NOT NULL COMMENT '0-禁用，1-启用',
        `remark` varchar(200) DEFAULT NULL COMMENT '备注',
        `create_id` bigint DEFAULT NULL COMMENT '创建人id',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        `fill_number` tinyint DEFAULT NULL COMMENT '卡号填涂位数',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='题卡规则';


-- ----------------------------
-- Table structure for basic_clazz
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_clazz` (
        `id` bigint NOT NULL,
        `school_id` bigint NOT NULL COMMENT '学校id',
        `major_id` bigint NOT NULL COMMENT '专业id',
        `clazz_code` varchar(50) DEFAULT NULL COMMENT '班级编号',
        `short_code` varchar(10) DEFAULT NULL COMMENT '简码',
        `clazz_name` varchar(50) DEFAULT NULL COMMENT '班级名称',
        `enable` tinyint(1) DEFAULT NULL COMMENT '是否可用',
        `org_id` bigint DEFAULT NULL COMMENT '权限机构id',
        `create_id` bigint DEFAULT NULL COMMENT '创建人id',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人id',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE,
        UNIQUE KEY `clazz_schoolId_name_idx` (`school_id`,`major_id`,`clazz_name`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='基础班级表';


-- ----------------------------
-- Table structure for basic_college
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_college` (
        `id` bigint NOT NULL,
        `school_id` bigint NOT NULL,
        `college_name` varchar(50) NOT NULL COMMENT '学院名称',
        `org_id` bigint NOT NULL COMMENT '数据权限机构id',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE,
        UNIQUE KEY `college_schoolId_name_idx` (`school_id`,`college_name`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='学生学院表';


-- ----------------------------
-- Table structure for basic_course
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_course` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `org_id` bigint NOT NULL COMMENT '机构id',
        `code` varchar(100) NOT NULL COMMENT '课程编码',
        `short_code` varchar(10) DEFAULT NULL COMMENT '简码',
        `name` varchar(50) NOT NULL COMMENT '课程名称',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '修改人',
        `update_time` bigint DEFAULT NULL COMMENT '修改时间',
        `teaching_room_id` bigint DEFAULT NULL COMMENT '教研室id',
        `enable` tinyint DEFAULT NULL,
        `historic_name` varchar(100) DEFAULT NULL COMMENT '历史课程名称',
        PRIMARY KEY (`id`) USING BTREE,
        UNIQUE KEY `course_schoolId_code_idx` (`school_id`,`code`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='课程表';

-- ----------------------------
-- Table structure for basic_exam
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_exam` (
        `id` bigint NOT NULL,
        `school_id` bigint NOT NULL,
        `org_id` bigint NOT NULL,
        `semester_id` bigint DEFAULT NULL COMMENT '学期ID',
        `short_code` varchar(10) DEFAULT NULL COMMENT '简码',
        `name` varchar(100) DEFAULT NULL COMMENT '考试名称',
        `category` varchar(45) DEFAULT NULL COMMENT '考试类型（ONLINE：线上，OFFLINE：线下）',
        `exam_model` varchar(45) DEFAULT NULL COMMENT '业务模式（MODEL1：电子交卷环节需要提交考务数据，MODEL2：电子交卷环节不需要提交考务数据，只输入印刷份数即可，MODEL3：电子交卷环节不需要提交考务数据）',
        `review` tinyint DEFAULT NULL COMMENT '是否启用入库审核：0-禁用，1-启用',
        `print_method` varchar(6) DEFAULT NULL COMMENT 'AUTO-自动，MANUAL-手动',
        `enable` tinyint(1) NOT NULL DEFAULT '1' COMMENT '1启用/0禁用',
        `create_id` bigint DEFAULT NULL,
        `create_time` bigint DEFAULT NULL,
        `update_id` bigint DEFAULT NULL,
        `update_time` bigint DEFAULT NULL,
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考试表';

-- ----------------------------
-- Table structure for basic_exam_rule
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_exam_rule` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `required_fields` text COMMENT '必选字段',
        `extend_fields` text NOT NULL COMMENT '扩展字段',
        `exam_number_digit` int DEFAULT NULL COMMENT '考号位数',
        `sign_scope` text COMMENT '签到表字段可选范围',
        `package_scope` text COMMENT '卷袋贴字段可选范围',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考务规则';

-- ----------------------------
-- Table structure for basic_major
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_major` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `org_id` bigint NOT NULL COMMENT '机构id',
        `belong_org_id` bigint NOT NULL COMMENT '所属机构id',
        `name` varchar(100) NOT NULL COMMENT '专业名称',
        `code` varchar(100) DEFAULT NULL COMMENT '专业编码',
        `enable` tinyint DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
        `create_id` bigint DEFAULT NULL COMMENT '创建人id',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人id',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE,
        UNIQUE KEY `major_schoolId_name_idx` (`school_id`,`name`,`belong_org_id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='专业字典表';

-- ----------------------------
-- Table structure for basic_message
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_message` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint DEFAULT NULL COMMENT '学校id',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `user_id` bigint NOT NULL COMMENT '消息接收人用户',
        `user_name` varchar(100) DEFAULT NULL COMMENT '用户名称',
        `mobile_number` varchar(11) DEFAULT NULL COMMENT '电话号码',
        `paper_number` varchar(50) DEFAULT NULL COMMENT '试卷编号',
        `course_code` varchar(50) DEFAULT NULL COMMENT '课程代码',
        `message_type` varchar(45) DEFAULT NULL COMMENT '消息类型',
        `business_operate` varchar(45) NOT NULL COMMENT '业务操作',
        `business_id` bigint DEFAULT NULL COMMENT '业务id',
        `template_code` varchar(45) NOT NULL COMMENT '消息模板代码',
        `variable_params` text COMMENT '变量参数内容',
        `template_content` mediumtext COMMENT '模板内容',
        `send_status` varchar(45) DEFAULT NULL COMMENT '消息发送状态',
        `send_result` varchar(200) DEFAULT NULL COMMENT '消息发送结果',
        `remark` text COMMENT '备注',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        `resend_count` int DEFAULT NULL COMMENT '重试次数',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for basic_operation_log
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_operation_log` (
        `id` bigint NOT NULL COMMENT 'id',
        `school_id` bigint DEFAULT NULL COMMENT '学校id',
        `org_id` bigint DEFAULT NULL COMMENT '数据权限id',
        `detail` mediumtext NOT NULL COMMENT '日志详情',
        `level` varchar(32) DEFAULT NULL COMMENT '日志等级（PRIMARY("一级日志：敏感信息"),SECONDARY("二级日志：重要信息"),TERTIARY("三级日志：次要信息"),QUATERNARY("四级日志：普通信息")）',
        `customized_operation_type` varchar(32) NOT NULL COMMENT '自定义记录日志操作类型（LOGIN("用户登录"),LOGOUT("用户登出"),ADD("新增"),DELETE("删除"),UPDATE("修改"),IMPORT("导入"),EXPORT("导出"),UN_KNOW("未知")）',
        `operation_type` varchar(32) DEFAULT NULL COMMENT '操作类型（UN_KNOW("未知"),SELECT("查询"),EDIT("新增或更新"),DELETE("删除")）',
        `operation_unit` varchar(128) DEFAULT NULL COMMENT '操作对象（略）',
        `system_type` varchar(32) DEFAULT NULL COMMENT '日志系统类型',
        `url` varchar(64) DEFAULT NULL COMMENT '请求地址url',
        `privilege_id` bigint DEFAULT NULL COMMENT '权限菜单id',
        `clazz_name` varchar(64) DEFAULT NULL COMMENT '类名称',
        `method` varchar(64) DEFAULT NULL COMMENT '方法名称',
        `args` mediumtext COMMENT '参数',
        `run_time` bigint DEFAULT NULL COMMENT '方法运行时间',
        `run_status` varchar(32) DEFAULT NULL COMMENT '方法运行状态',
        `return_value` mediumtext COMMENT '方法返回值',
        `user_id` bigint NOT NULL COMMENT '用户对象id',
        `user_name` varchar(32) DEFAULT NULL COMMENT '用户名称',
        `ip` varchar(32) NOT NULL COMMENT 'ip地址',
        `create_time` bigint DEFAULT NULL COMMENT '日志记录时间',
        PRIMARY KEY (`id`) USING BTREE,
        KEY `operation_log_idx` (`school_id`,`customized_operation_type`,`org_id`,`user_id`,`create_time`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='基础日志记录表';

-- ----------------------------
-- Table structure for basic_print_config
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_print_config` (
        `id` bigint NOT NULL,
        `school_id` bigint NOT NULL COMMENT '学校ID',
        `org_id` bigint NOT NULL COMMENT '机构id',
        `exam_id` bigint DEFAULT NULL COMMENT '考试ID',
        `enable` tinyint(1) NOT NULL DEFAULT '1' COMMENT '1启用/0禁用',
        `card_rule_id` bigint DEFAULT NULL COMMENT '题卡规则（-1代表通卡）',
        `print_content` varchar(255) DEFAULT NULL COMMENT '试卷、题卡',
        `backup_method` varchar(10) DEFAULT NULL COMMENT '试卷题卡备份方式（ROOM按考场）',
        `backup_count` double DEFAULT NULL COMMENT '试卷、题卡备份数量（0-1为按比例，大于等于1为按数量）',
        `draw_rule` varchar(10) DEFAULT NULL COMMENT '抽卷规则（ONE只抽一次，CIRCLE循环抽取）',
        `variable_content` mediumtext COMMENT '变量印品参数json',
        `ordinary_content` mediumtext COMMENT '普通印品参数json',
        `create_id` bigint DEFAULT NULL,
        `create_time` bigint DEFAULT NULL,
        `update_id` bigint DEFAULT NULL,
        `update_time` bigint DEFAULT NULL,
        `review` tinyint DEFAULT NULL COMMENT '是否启用入库审核：0-禁用，1-启用',
        `audit_content` varchar(45) DEFAULT NULL COMMENT '[PAPER,CARD]',
        `print_method` varchar(6) DEFAULT NULL COMMENT 'AUTO-自动，MANUAL-手动',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='印品配置表';

-- ----------------------------
-- Table structure for basic_role_data_permission
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_role_data_permission` (
        `id` bigint NOT NULL COMMENT 'id',
        `school_id` bigint DEFAULT NULL COMMENT '学校id',
        `role_id` bigint NOT NULL COMMENT '角色id',
        `privilege_id` bigint NOT NULL COMMENT '权限id',
        `data_permission_type` varchar(45) NOT NULL COMMENT '数据权限类型',
        `org_scope` varchar(45) DEFAULT NULL COMMENT '数据权限机构权限组',
        `org_id` bigint DEFAULT NULL COMMENT '创建人机构id',
        `create_id` bigint DEFAULT NULL COMMENT '创建人id',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人id',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE,
        KEY `role_data_permission_idx` (`role_id`,`privilege_id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='角色数据权限表';

INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176721063936, NULL, 2, 4, 'ALL', NULL, NULL, 1, 1700550684436, NULL, 1700550684436);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176721063937, NULL, 2, 5, 'ALL', NULL, NULL, 1, 1700550684440, NULL, 1700550684440);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176721063938, NULL, 2, 6, 'ALL', NULL, NULL, 1, 1700550684440, NULL, 1700550684440);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176721063939, NULL, 2, 8, 'ALL', NULL, NULL, 1, 1700550684440, NULL, 1700550684440);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176721063940, NULL, 2, 9, 'ALL', NULL, NULL, 1, 1700550684441, NULL, 1700550684441);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258240, NULL, 2, 624, 'ALL', NULL, NULL, 1, 1700550684441, NULL, 1700550684441);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258241, NULL, 2, 11, 'ALL', NULL, NULL, 1, 1700550684442, NULL, 1700550684442);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258242, NULL, 2, 402, 'ALL', NULL, NULL, 1, 1700550684442, NULL, 1700550684442);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258243, NULL, 2, 403, 'ALL', NULL, NULL, 1, 1700550684443, NULL, 1700550684443);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258244, NULL, 2, 510, 'ALL', NULL, NULL, 1, 1700550684443, NULL, 1700550684443);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258245, NULL, 2, 609, 'ALL', NULL, NULL, 1, 1700550684444, NULL, 1700550684444);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258246, NULL, 2, 515, 'ALL', NULL, NULL, 1, 1700550684444, NULL, 1700550684444);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258247, NULL, 2, 450, 'ALL', NULL, NULL, 1, 1700550684444, NULL, 1700550684444);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258248, NULL, 2, 358, 'ALL', NULL, NULL, 1, 1700550684445, NULL, 1700550684445);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258249, NULL, 2, 13, 'ALL', NULL, NULL, 1, 1700550684445, NULL, 1700550684445);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258250, NULL, 2, 113, 'ALL', NULL, NULL, 1, 1700550684445, NULL, 1700550684445);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258251, NULL, 2, 521, 'ALL', NULL, NULL, 1, 1700550684446, NULL, 1700550684446);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258252, NULL, 2, 549, 'ALL', NULL, NULL, 1, 1700550684446, NULL, 1700550684446);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258253, NULL, 2, 653, 'ALL', NULL, NULL, 1, 1700550684446, NULL, 1700550684446);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258254, NULL, 2, 782, 'ALL', NULL, NULL, 1, 1700550684447, NULL, 1700550684447);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258255, NULL, 2, 576, 'ALL', NULL, NULL, 1, 1700550684447, NULL, 1700550684447);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258256, NULL, 2, 39, 'ALL', NULL, NULL, 1, 1700550684447, NULL, 1700550684447);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258257, NULL, 2, 42, 'ALL', NULL, NULL, 1, 1700550684448, NULL, 1700550684448);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258258, NULL, 2, 43, 'ALL', NULL, NULL, 1, 1700550684448, NULL, 1700550684448);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258259, NULL, 2, 44, 'ALL', NULL, NULL, 1, 1700550684448, NULL, 1700550684448);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258260, NULL, 2, 45, 'ALL', NULL, NULL, 1, 1700550684449, NULL, 1700550684449);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258261, NULL, 2, 47, 'ALL', NULL, NULL, 1, 1700550684449, NULL, 1700550684449);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258262, NULL, 2, 491, 'ALL', NULL, NULL, 1, 1700550684449, NULL, 1700550684449);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258263, NULL, 2, 897, 'ALL', NULL, NULL, 1, 1700550684450, NULL, 1700550684450);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258264, NULL, 2, 917, 'ALL', NULL, NULL, 1, 1700550684450, NULL, 1700550684450);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258265, NULL, 2, 946, 'ALL', NULL, NULL, 1, 1700550684450, NULL, 1700550684450);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737176725258266, NULL, 2, 487, 'ALL', NULL, NULL, 1, 1700550684451, NULL, 1700550684451);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737638522322944, NULL, 3, 576, 'SELF', NULL, NULL, 1, 1700550794536, NULL, 1700550794537);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737638522322945, NULL, 3, 491, 'SELF', NULL, NULL, 1, 1700550794538, NULL, 1700550794538);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737638522322946, NULL, 3, 897, 'SELF', NULL, NULL, 1, 1700550794539, NULL, 1700550794539);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737638522322947, NULL, 3, 917, 'SELF', NULL, NULL, 1, 1700550794539, NULL, 1700550794539);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737638522322948, NULL, 3, 946, 'SELF', NULL, NULL, 1, 1700550794539, NULL, 1700550794539);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737638522322949, NULL, 3, 487, 'SELF', NULL, NULL, 1, 1700550794540, NULL, 1700550794540);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737876502937600, NULL, 4, 576, 'SELF_ORG_BELOW', NULL, NULL, 1, 1700550851276, NULL, 1700550851276);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737876502937601, NULL, 4, 39, 'SELF_ORG_BELOW', NULL, NULL, 1, 1700550851279, NULL, 1700550851279);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459737876502937602, NULL, 4, 42, 'SELF_ORG_BELOW', NULL, NULL, 1, 1700550851279, NULL, 1700550851279);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459738193709760512, NULL, 6, 970, 'SELF_ORG_BELOW', NULL, NULL, 1, 1700550926903, NULL, 1700550926903);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (459738339466018816, NULL, 7, 491, 'SELF', NULL, NULL, 1, 1700550961655, NULL, 1700550961655);

-- ----------------------------
-- Table structure for basic_school
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_school` (
        `id` bigint NOT NULL COMMENT '主键',
        `code` varchar(20) NOT NULL COMMENT '学校编码',
        `domain_name` varchar(20) DEFAULT NULL COMMENT '域名',
        `name` varchar(100) DEFAULT NULL COMMENT '学校名称',
        `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，false：停用，true：启用',
        `access_key` varchar(50) DEFAULT NULL COMMENT '访问key',
        `access_secret` varchar(50) DEFAULT NULL COMMENT '访问秘钥',
        `remark` text COMMENT '备注',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        `logo` mediumtext COMMENT '学校logo',
        `init_password` varchar(100) DEFAULT NULL COMMENT '该学校用户的初始密码，默认为12345678',
        `default_school` TINYINT(1) NULL DEFAULT 0 COMMENT '默认学校',
        PRIMARY KEY (`id`) USING BTREE,
        UNIQUE KEY `code` (`code`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='学校表';

-- ----------------------------
-- Table structure for basic_semester
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_semester` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `name` varchar(100) NOT NULL COMMENT '学期名称',
        `code` varchar(100) DEFAULT NULL COMMENT '学期编码',
        `start_time` bigint DEFAULT NULL COMMENT '开始日期',
        `end_time` bigint DEFAULT NULL COMMENT '结束日期',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `enable` tinyint DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
        `create_id` bigint DEFAULT NULL COMMENT '创建人id',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人id',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE,
        UNIQUE KEY `semester_schoolId_name_idx` (`school_id`,`name`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='学期字典表';

-- ----------------------------
-- Table structure for basic_student
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_student` (
        `id` bigint NOT NULL COMMENT '学生id',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `student_name` varchar(100) NOT NULL COMMENT '学生姓名',
        `student_code` varchar(100) NOT NULL COMMENT '学生编号',
        `phone_number` varchar(11) DEFAULT NULL COMMENT '电话号码',
        `belong_org_id` bigint DEFAULT NULL COMMENT '所属院系',
        `major_id` bigint DEFAULT NULL COMMENT '专业id',
        `clazz_id` bigint DEFAULT NULL COMMENT '班级主键',
        `enable` tinyint DEFAULT '1' COMMENT '可用状态（1：可用，0：不可用）',
        `org_id` bigint DEFAULT NULL COMMENT '可操作人机构id',
        `create_id` bigint DEFAULT NULL COMMENT '创建人id',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人id',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE,
        UNIQUE KEY `student_schoolId_code_orgId_idx` (`school_id`,`student_code`,`belong_org_id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='学校id';

-- ----------------------------
-- Table structure for basic_teach_course
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_teach_course` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint DEFAULT NULL COMMENT '学校id',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `user_id` bigint DEFAULT NULL COMMENT '用户ID',
        `course_id` bigint DEFAULT NULL COMMENT '课程ID',
        `enable` tinyint DEFAULT '1' COMMENT '是否启用？1-是 ；0-否',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`)
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='老师和课程关联表';

-- ----------------------------
-- Table structure for basic_template
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_template` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `name` varchar(30) NOT NULL COMMENT '模板名称',
        `classify` varchar(10) DEFAULT NULL COMMENT 'template_type值为VARIABLE、ORDINARY时，不为空。\n  SIGN-签到表，PACKAGE-卷袋贴，CHECK_IN-登记表',
        `display_range` mediumtext COMMENT '卷袋贴、签到表数据范围',
        `preview_path` varchar(500) DEFAULT NULL COMMENT '预览地址',
        `attachment_id` bigint DEFAULT NULL COMMENT '模板附件ID（保存到附件表ID）',
        `enable` tinyint NOT NULL COMMENT '0-禁用，1-启用',
        `diallel` tinyint(1) DEFAULT '1' COMMENT '考生信息是否双列显示',
        `text_desc` varchar(500) DEFAULT NULL COMMENT '文字说明',
        `remark` varchar(200) DEFAULT NULL COMMENT '备注',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        `add_blank_page` TINYINT(1) NULL DEFAULT 1 COMMENT '是否增加空白页',
        `line_height_times` DOUBLE NULL DEFAULT 1 COMMENT '行间距在初始值扩大倍数（默认1）',
        `font_size` VARCHAR(10) NULL COMMENT '字体大小（small,medium,large）',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='通用模板';

-- ----------------------------
-- Table structure for basic_template_org
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_template_org` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `type` varchar(20) NOT NULL COMMENT '模板类型：GENERIC-通卡模板，VARIABLE-变量印品模板，ORDINARY-普通印品模板，CARD_RULE-题卡规则，PRINT_CONFIG-印品配置',
        `rule_id` bigint NOT NULL COMMENT '通卡模板ID',
        `org_id` bigint NOT NULL COMMENT '组织机构ID',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='适用学院（题卡规则、模板适用学院）';

-- ----------------------------
-- Table structure for basic_user_course
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_user_course` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint DEFAULT NULL COMMENT '学校id',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `user_id` bigint DEFAULT NULL COMMENT '用户ID',
        `course_id` bigint DEFAULT NULL COMMENT '课程ID',
        `clazz_id` bigint DEFAULT NULL COMMENT '授课班级',
        `enable` tinyint(1) DEFAULT NULL COMMENT '是否启用？1-是 ；0-否',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='用户和课程关联表';

-- ----------------------------
-- Table structure for basic_verify_code
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_verify_code` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint DEFAULT NULL COMMENT '学校id',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `user_id` bigint NOT NULL COMMENT '用户ID',
        `mobile_number` varchar(11) NOT NULL COMMENT '手机号',
        `expire_time` bigint NOT NULL COMMENT '到期时间',
        `valid_period` int NOT NULL COMMENT '单位（分）',
        `verify_code` varchar(4) NOT NULL COMMENT '4位数字',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='短信验证码记录表';

-- ----------------------------
-- Table structure for client_print_data
-- ----------------------------
CREATE TABLE IF NOT EXISTS `client_print_data` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint DEFAULT NULL COMMENT '学校id',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `exam_detail_id` bigint DEFAULT NULL COMMENT '考场ID',
        `machine_code` varchar(45) DEFAULT NULL COMMENT '机器唯一码',
        `pages_a4` int DEFAULT NULL COMMENT 'A4页数量',
        `paper_pages` int DEFAULT NULL COMMENT '试卷数量',
        `card_pages` int DEFAULT NULL COMMENT '题卡数量',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='各机器打印数量';

-- ----------------------------
-- Table structure for client_status
-- ----------------------------
CREATE TABLE IF NOT EXISTS `client_status` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint DEFAULT NULL COMMENT '学校id',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `machine_code` varchar(100) NOT NULL COMMENT '机器唯一码',
        `exam_detail_id` bigint DEFAULT NULL COMMENT '考场ID',
        `print_plan_id` bigint DEFAULT NULL COMMENT '印刷计划ID',
        `course_code` varchar(100) NOT NULL COMMENT '课程代码',
        `course_name` varchar(200) NOT NULL COMMENT '课程名称',
        `paper_number` varchar(50) NOT NULL COMMENT '试卷编号',
        `is_try` tinyint DEFAULT NULL COMMENT '是否打样：true-是，false-否',
        `try_time` bigint DEFAULT NULL COMMENT '打样时间',
        `is_pass` tinyint DEFAULT NULL COMMENT '是否合格：true-合格，false-不合格',
        `is_download` tinyint DEFAULT NULL COMMENT '是否缓存：true-是，false-否',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE,
        KEY `idx_1` (`school_id`,`exam_detail_id`,`course_code`,`paper_number`,`machine_code`)
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='机器打样记录';

-- ----------------------------
-- Table structure for cloud_user_push_status
-- ----------------------------
CREATE TABLE IF NOT EXISTS `cloud_user_push_status` (
        `id` bigint NOT NULL,
        `school_id` bigint NOT NULL COMMENT '学校id',
        `user_id` bigint NOT NULL,
        `marker_status` varchar(100) NOT NULL COMMENT '评卷员推送状态(UserPushResultEnum):BIND_SUCCESS("绑定成功"),BIND_FAILED("绑定失败"),UNBIND_SUCCESS("解绑成功"),UNBIND_FAILED("解绑失败");',
        `marker_leader_status` varchar(100) NOT NULL COMMENT '科组长推送状态(UserPushResultEnum):BIND_SUCCESS("绑定成功"),BIND_FAILED("绑定失败"),UNBIND_SUCCESS("解绑成功"),UNBIND_FAILED("解绑失败");',
        `sync_status` varchar(100) NOT NULL COMMENT '同步状态(SyncStatusEnum)，ALREADY_PUSH("已经完成推送"),NEED_PUSH_AGAIN("需要重新推送");',
        `create_id` bigint DEFAULT NULL COMMENT '创建人\n',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE,
        KEY `idx_school_id_user_id` (`school_id`,`user_id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='云阅卷用户推送状态表';

-- ----------------------------
-- Table structure for exam_card
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_card` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `course_code` varchar(30) DEFAULT NULL COMMENT '课程编码',
        `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
        `title` varchar(200) NOT NULL COMMENT '标题',
        `make_method` varchar(10) DEFAULT NULL COMMENT '题卡制作方式：SELECT-选择已有题卡，SELF-自助创建，CUST-客户制卡',
        `status` varchar(45) DEFAULT NULL COMMENT 'STAGE-暂存，SUBMIT-提交',
        `content` mediumtext COMMENT '题卡工具制作题卡内容',
        `stage_content` mediumtext COMMENT '暂存内容（提交后置空）',
        `html_content` mediumtext COMMENT 'html格式内容',
        `type` varchar(45) NOT NULL COMMENT '题卡类型：GENERIC-通卡，CUSTOM-自定义',
        `create_method` varchar(45) DEFAULT NULL COMMENT '通卡创建方式（UPLOAD-上传，CUSTOMIZE-自定义）',
        `attachment_id` bigint DEFAULT NULL COMMENT 'type=GENERIC时必传，上传附件ID',
        `enable` tinyint(1) DEFAULT '1' COMMENT '1正常，0禁用',
        `used` tinyint(1) DEFAULT '0' COMMENT '是否绑定命题任务，默认0',
        `remark` mediumtext COMMENT '备注',
        `card_rule_id` bigint DEFAULT NULL COMMENT '题卡规则Id',
        `backup_attachment` mediumtext COMMENT '备用题卡json',
        `jpg_attachment` text COMMENT '题卡转换成的jpg文件信息',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='题卡';

-- ----------------------------
-- Table structure for exam_card_detail
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_card_detail` (
        `id` bigint NOT NULL COMMENT '主键',
        `card_id` bigint NOT NULL COMMENT '题卡ID',
        `content` mediumtext COMMENT '题卡工具制作题卡内容',
        `html_content` mediumtext COMMENT 'html格式内容',
        `attachment_id` mediumtext COMMENT '备用题卡json',
        `cust_attachment_id` bigint DEFAULT NULL COMMENT '客服制卡上传附件ID',
        `jpg_attachment_info` text COMMENT '题卡转换成的jpg文件信息',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='题卡详情';

-- ----------------------------
-- Table structure for exam_detail
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_detail` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `org_id` bigint NOT NULL COMMENT '机构id',
        `print_plan_id` bigint NOT NULL COMMENT '印刷计划ID',
        `exam_id` bigint DEFAULT NULL COMMENT '考试ID',
        `print_plan_name` varchar(100) DEFAULT NULL COMMENT '印刷计划名称',
        `exam_start_time` bigint NOT NULL COMMENT '考试开始日期',
        `exam_end_time` bigint NOT NULL COMMENT '考试结束日期',
        `exam_place` varchar(30) DEFAULT NULL COMMENT '考场',
        `total_subjects` int DEFAULT NULL COMMENT '总科次：考场总科次',
        `exam_room` varchar(30) DEFAULT NULL COMMENT '教室',
        `print_progress` double DEFAULT NULL COMMENT '打印进度',
        `print_user` varchar(30) DEFAULT NULL COMMENT '打印员：一个考场只能被一个打印员认领',
        `package_code` varchar(30) DEFAULT NULL COMMENT '考场代码：卷袋贴、签到表上条码',
        `paper_pages` int DEFAULT NULL COMMENT '试卷总计（张）',
        `card_pages` int DEFAULT NULL COMMENT '题卡总计（张）',
        `pages_a4` int DEFAULT NULL COMMENT 'A4印量小计（页）',
        `status` varchar(30) DEFAULT NULL COMMENT '状态：NEW: "新建",READY: "就绪",WAITING: "待印刷",PRINTING: "印刷中",FINISH: "已完成"',
        `validate` tinyint(1) DEFAULT '0' COMMENT '是否校验：true-是，false-否',
        `print_start_time` bigint DEFAULT NULL COMMENT '打印开始时间',
        `print_end_time` bigint DEFAULT NULL COMMENT '打印完成时间',
        `attachment_id` bigint DEFAULT NULL COMMENT '附件id',
        `card_attachment_id` bigint DEFAULT NULL COMMENT '题卡合并A3文件附件ID',
        `attachment_path` mediumtext COMMENT '印品附件路径',
        `exam_data_source` varchar(20) DEFAULT NULL COMMENT '数据来源（EXAM_TASK：卷库查询生成，FILE_IMPORT：考务文件导入）',
        `normal` tinyint DEFAULT '1' COMMENT '是否正常，true正常，false做废',
        `print_house_id` bigint DEFAULT NULL COMMENT '印刷室ID',
        `backup_count` double DEFAULT NULL COMMENT '试卷、题卡备份数量（0-1为按比例，大于等于1为按数量）',
        `task_id` bigint DEFAULT NULL COMMENT 'pdf生成异步任务ID（t_b_task）',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE,
        KEY `idx_print_plan_id` (`print_plan_id`) USING BTREE,
        KEY `exam_detail_school_id_IDX` (`school_id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考务数据';

-- ----------------------------
-- Table structure for exam_detail_course
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_detail_course` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `exam_detail_id` bigint NOT NULL COMMENT '考试明细id',
        `course_code` varchar(20) NOT NULL COMMENT '课程编码',
        `course_name` varchar(50) NOT NULL COMMENT '课程名称',
        `paper_number` varchar(30) NOT NULL COMMENT '页数',
        `course_paper_id` varchar(45) DEFAULT NULL COMMENT '试卷编号（交互）',
        `total_subjects` int DEFAULT NULL COMMENT '总科次：考场总科次',
        `paper_pages_a3` int DEFAULT NULL COMMENT '试卷单科次准印量A3（页）',
        `card_pages_a3` int DEFAULT NULL COMMENT '题卡单科次准印量A3（页）',
        `clazz_id` text COMMENT '班级id',
        `clazz_name` mediumtext COMMENT '班级（取学生表中的班级集合）',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        `paper_type` varchar(30) DEFAULT NULL COMMENT '当前试卷类型',
        `attachment_id` mediumtext COMMENT '备用题卡json',
        `common_attachment_id` mediumtext COMMENT '模式2下考生公共题卡json',
        PRIMARY KEY (`id`) USING BTREE,
        KEY `detail_course_schoolId_idx` (`school_id`,`paper_number`,`paper_type`) USING BTREE,
        KEY `idx_exam_detail_id` (`exam_detail_id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考务数据（考场关联科目）';

-- ----------------------------
-- Table structure for exam_detail_course_paper_type
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_detail_course_paper_type` (
        `id` bigint NOT NULL,
        `exam_detail_course_id` bigint NOT NULL,
        `paper_type` varchar(5) NOT NULL COMMENT '卷型',
        PRIMARY KEY (`id`),
        KEY `idx_exam_detail_course_id` (`exam_detail_course_id`)
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考务数据（考场关联科目对应的卷型）';

-- ----------------------------
-- Table structure for exam_paper_class_marker
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_paper_class_marker` (
        `id` bigint NOT NULL,
        `exam_paper_structure_id` bigint DEFAULT NULL COMMENT '评卷参数id',
        `marker_id` bigint DEFAULT NULL COMMENT '评卷员id',
        `login_name` varchar(100) DEFAULT NULL COMMENT '评卷员登录名',
        `class_name` mediumtext COMMENT '班级',
        `create_id` bigint DEFAULT NULL,
        `create_time` bigint DEFAULT NULL,
        `update_id` bigint DEFAULT NULL,
        `update_time` bigint DEFAULT NULL,
        PRIMARY KEY (`id`)
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='分班阅（评卷员班级对应关系表）';

-- ----------------------------
-- Table structure for exam_paper_group
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_paper_group` (
        `id` bigint NOT NULL COMMENT '分组id',
        `exam_paper_structure_id` bigint NOT NULL COMMENT '试卷结构id',
        `group_number` int NOT NULL COMMENT '分组序号',
        `double_rate` tinyint DEFAULT NULL COMMENT '双评比例（0关闭，1开启）',
        `arbitrate_threshold` int DEFAULT NULL COMMENT '仲裁阈值（开启双评必填）',
        `score_policy` varchar(45) DEFAULT NULL COMMENT '合分策略 开启双评是必填 AVG("平均分", 1), MAX("最高分", 2), MIN("最低分", 3)',
        `trial_count` int DEFAULT NULL COMMENT '试评数量',
        `mark_mode` varchar(45) DEFAULT NULL COMMENT '评卷模式 COMMON("普通"), TRACK("轨迹")',
        `question_info` text COMMENT '题目信息',
        `picture_config` text COMMENT '图片配置',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE,
        UNIQUE KEY `exam_paper_group_unique` (`exam_paper_structure_id`,`group_number`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='试卷分组表';

-- ----------------------------
-- Table structure for exam_paper_group_marker
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_paper_group_marker` (
        `id` bigint NOT NULL COMMENT '评卷员-分组关系id',
        `group_id` bigint NOT NULL COMMENT '分组id',
        `marker_id` bigint NOT NULL COMMENT '评卷员id',
        `login_name` varchar(100) DEFAULT NULL COMMENT '评卷员登录名',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='评卷员-分组关系表';

-- ----------------------------
-- Table structure for exam_paper_structure
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_paper_structure` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint DEFAULT NULL COMMENT '学校ID',
        `semester_id` bigint DEFAULT NULL COMMENT '学期ID',
        `semester_name` varchar(100) DEFAULT NULL COMMENT '学期',
        `exam_id` bigint NOT NULL COMMENT '考试id',
        `third_relate_id` bigint DEFAULT NULL COMMENT '云阅卷考试ID',
        `third_relate_name` varchar(255) DEFAULT NULL COMMENT '云阅卷考试名称',
        `paper_number` varchar(50) NOT NULL COMMENT '试卷编号',
        `course_code` varchar(50) DEFAULT NULL COMMENT '课程代码',
        `course_name` varchar(255) DEFAULT NULL COMMENT '课程名称',
        `sequence` varchar(5) DEFAULT NULL COMMENT '课程创建的任务序号（和exam_task保持一致）',
        `paper_type` varchar(20) DEFAULT NULL COMMENT '试卷类型',
        `status` varchar(700) DEFAULT NULL COMMENT '{\n    "structure":\n    {\n        "save": false,\n        "sync": false\n    },\n    "group":\n    {\n        "save": false,\n        "sync": false\n    },\n    "class":\n    {\n        "save": false,\n        "sync": false\n    },\n    "objective":\n    {\n        "save": false,\n        "sync": false\n    },\n    "markerLeader":\n    {\n        "save": false,\n        "sync": false\n    },\n    "marker":\n    {\n        "save": false,\n        "sync": false\n    },\n    "answerFile":\n    {\n        "save": false,\n        "sync": false\n    }\n}',
        `is_finish` tinyint(1) DEFAULT '0' COMMENT '是否完成。根据status中组合状态计算。用于当查询条件',
        `paper_answer` text COMMENT '试卷原卷和标答附件ID',
        `objective_structure` mediumtext COMMENT '客观题试卷结构JSON',
        `subjective_structure` mediumtext COMMENT '主观题试卷结构JSON',
        `paper_info_json` mediumtext COMMENT '整个试卷信息json（前端提供的全部信息）',
        `proposition_teacher_id` bigint DEFAULT NULL COMMENT '命题老师ID',
        `enable` tinyint(1) DEFAULT NULL COMMENT '是否启用（0-禁用，1-启用）',
        `mark_leader` text COMMENT '科组长，样例：[{markerId:1, loginName:"mt01"}]',
        `cloud_info_json` text COMMENT '云阅卷试卷结构',
        `structure_change` tinyint(1) DEFAULT NULL COMMENT '云阅卷试卷结构和知学知考结构不一样（默认false）',
        `object_answer_change` tinyint(1) DEFAULT '0' COMMENT '客观题答案有提交，需要重新统分。（1：有变动，需要统分，0：不统分）',
        `open_class_reading` tinyint(1) DEFAULT '0' COMMENT '是否开启分班阅',
        `create_id` bigint DEFAULT NULL,
        `create_time` bigint DEFAULT NULL,
        `update_id` bigint DEFAULT NULL,
        `update_time` bigint DEFAULT NULL,
        PRIMARY KEY (`id`) USING BTREE,
        UNIQUE KEY `exam_paper_structure_unique` (`school_id`,`exam_id`,`paper_number`,`paper_type`) USING BTREE,
        KEY `exam_paper_structure_status_IDX` (`status`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for exam_print_plan
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_print_plan` (
        `id` bigint NOT NULL COMMENT '主键',
        `exam_id` bigint NOT NULL COMMENT '考试id',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `org_id` bigint NOT NULL COMMENT '机构id',
        `name` varchar(150) DEFAULT NULL COMMENT '名称',
        `exam_start_time` bigint NOT NULL COMMENT '考试开始时间',
        `exam_end_time` bigint NOT NULL COMMENT '考试结束时间',
        `print_content` varchar(30) NOT NULL COMMENT '{“PAPER”, "CARD"}',
        `backup_method` varchar(10) NOT NULL COMMENT 'PLACE-按考点，ROOM-按考场',
        `backup_count` double NOT NULL COMMENT '试卷、题卡备份数量（0-1为按比例，大于等于1为按数量）',
        `draw_rule` varchar(10) DEFAULT NULL COMMENT 'ONE-只抽一次，CIRCLE-可循环抽取',
        `variable_content` text NOT NULL COMMENT '[\n  	{\n  		"type": "SIGN",\n  		"usedId": 1,\n  		"backupMethod": "ROOM",\n  		"backupCount": 3\n  	},\n  	{\n  		"type": "PACKAG",\n  		"usedId": 3,\n  		"backupMethod": "STUDENT",\n  		"backupCount": 5\n  	}\n  ]',
        `ordinary_content` text NOT NULL COMMENT '[\n  	{\n  		"type": "CHECK_IN",\n  		"usedId": 1,\n  		"backupMethod": "ROOM",\n  		"backupCount": 3\n  	}\n  ]',
        `status` varchar(20) NOT NULL COMMENT '印刷计划状态',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        `third_relate_id` bigint DEFAULT NULL COMMENT '第三方系统关联ID（目前只有云阅卷）',
        `third_relate_name` varchar(100) DEFAULT NULL COMMENT '第三方系统关联考试名称（目前只有云阅卷）',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='印刷计划管理';

-- ----------------------------
-- Table structure for exam_student
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_student` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `exam_detail_course_id` bigint NOT NULL COMMENT '考试明细课程id',
        `student_name` varchar(100) NOT NULL COMMENT '考生姓名',
        `student_code` varchar(100) NOT NULL COMMENT '考生代码',
        `exam_id` bigint DEFAULT NULL COMMENT '考试ID',
        `paper_number` varchar(30) NOT NULL COMMENT '试卷编号',
        `course_paper_id` varchar(45) DEFAULT NULL COMMENT '试卷编号（交互）',
        `ticket_number` varchar(30) DEFAULT NULL COMMENT '准考证号',
        `site_number` varchar(30) DEFAULT NULL COMMENT '座位号',
        `print_paper` tinyint DEFAULT NULL COMMENT '打印试卷',
        `print_card` tinyint DEFAULT NULL COMMENT '打印题卡',
        `required_fields` mediumtext COMMENT '基础字段',
        `extend_fields` text COMMENT '扩展字段',
        `paper_type` varchar(2) DEFAULT NULL COMMENT '试卷类型',
        `college_name` varchar(100) DEFAULT NULL COMMENT '学生学院名称',
        `major_name` varchar(100) DEFAULT NULL COMMENT '专业名称',
        `clazz_id` varchar(100) DEFAULT NULL COMMENT '班级Id',
        `clazz_name` varchar(100) DEFAULT NULL COMMENT '班级名称',
        `teach_clazz_id` bigint DEFAULT NULL COMMENT '教学班id',
        `teach_clazz_name` varchar(100) DEFAULT NULL COMMENT '教学班名称',
        `student_clazz_type` varchar(100) DEFAULT NULL COMMENT '主体班级类型',
        `campus_name` varchar(200) DEFAULT NULL COMMENT '校区',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        `attachment_id` bigint DEFAULT NULL COMMENT '附件id',
        `can_delete` tinyint(1) DEFAULT '0' COMMENT '是否可以删除（手动新增可以删除）',
        PRIMARY KEY (`id`) USING BTREE,
        UNIQUE KEY `unique_ticket_number_idx` (`school_id`,`ticket_number`),
        KEY `exam_student_schoolId_idx` (`school_id`,`student_code`) USING BTREE,
        KEY `idx_exam_detail_course_id` (`exam_detail_course_id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考生';

-- ----------------------------
-- Table structure for exam_task
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_task` (
        `id` bigint NOT NULL COMMENT '主键',
        `exam_id` bigint NOT NULL COMMENT '考试id',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `course_code` varchar(20) NOT NULL COMMENT '课程代码',
        `course_name` varchar(50) NOT NULL COMMENT '课程名称',
        `specialty` varchar(100) DEFAULT NULL COMMENT '适用专业',
        `paper_number` varchar(30) NOT NULL COMMENT '试卷编号',
        `sequence` varchar(5) DEFAULT NULL COMMENT '课程创建的任务序号',
        `start_time` bigint DEFAULT NULL COMMENT '命题开始时间',
        `end_time` bigint DEFAULT NULL COMMENT '命题结束时间',
        `card_rule_id` bigint NOT NULL COMMENT '题卡规则',
        `user_id` bigint DEFAULT NULL COMMENT '命题老师',
        `transfer_id` bigint DEFAULT NULL COMMENT '转办人ID',
        `status` varchar(50) DEFAULT NULL COMMENT '任务状态，DRAFT：新建，SUBMIT：已提交，STAGE：待命题',
        `review` tinyint DEFAULT NULL COMMENT '是否审核，0-禁用，1-启用',
        `review_status` varchar(8) DEFAULT NULL COMMENT '审核状态：PASS-审核通过，NOT_PASS-审核不通过，默认为空-未审核',
        `enable` tinyint DEFAULT '1' COMMENT '是否启用，false：停用，true：启用',
        `batch_no` varchar(32) DEFAULT NULL COMMENT '批次id',
        `teaching_room_id` bigint DEFAULT NULL COMMENT '教学单位、教研室',
        `teacher_name` varchar(100) DEFAULT NULL COMMENT '拟卷教师',
        `lecturer_name` varchar(100) DEFAULT NULL COMMENT '主讲教师',
        `paper_name` varchar(100) DEFAULT NULL COMMENT '试卷名称',
        `cancel_remark` mediumtext COMMENT '作废原因',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '修改人',
        `update_time` bigint DEFAULT NULL COMMENT '修改时间',
        `flow_id` bigint DEFAULT NULL COMMENT '流程id',
        `approve_form_attachment_id` bigint DEFAULT NULL COMMENT '命题任务审批表附件id',
        PRIMARY KEY (`id`) USING BTREE,
        KEY `task_schoolId_flowId_idx` (`school_id`,`flow_id`,`card_rule_id`) USING BTREE,
        KEY `idx_school_id_paper_number` (`school_id`,`paper_number`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='命题任务';

-- ----------------------------
-- Table structure for exam_task_apply_temp
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_task_apply_temp` (
        `id` bigint unsigned NOT NULL,
        `school_id` bigint NOT NULL COMMENT '学校Id',
        `org_id` bigint NOT NULL COMMENT '机构id',
        `exam_task_content` mediumtext COMMENT '命题任务json',
        `create_id` bigint DEFAULT NULL,
        `create_time` bigint DEFAULT NULL,
        `update_id` bigint DEFAULT NULL,
        `update_time` bigint DEFAULT NULL,
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='卷库查询-命题任务申请临时表';

-- ----------------------------
-- Table structure for exam_task_detail
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_task_detail` (
        `id` bigint NOT NULL COMMENT '主键',
        `exam_task_id` bigint NOT NULL COMMENT '命题任务ID',
        `paper_type` varchar(20) DEFAULT NULL COMMENT '试卷类型，多个以”/“分隔，如A/B/C/D',
        `draw_count` int DEFAULT NULL COMMENT '单次抽卷数量',
        `paper_attachment_ids` text COMMENT '转换为A3格式的试卷附件\n[\\n  	{\\n  		"type": "A",\\n  		"attachmentId": 12,\\n  		"totalPages": 3\\n  	},\\n  	{\\n  		"type": "B",\\n  		"attachmentId": 13,\\n  		"totalPages": 2\\n  	}\\n  ]',
        `make_method` varchar(45) DEFAULT NULL COMMENT '题卡绑定方式',
        `paper_confirm_attachment_ids` text COMMENT '[\n  	{\n  		"attachmentId": 12\n  	},\n  	{\n  		"attachmentId": 13\n  	}\n  ]',
        `remark` varchar(200) DEFAULT NULL COMMENT '备注',
        `exposed_paper_type` varchar(20) DEFAULT NULL COMMENT '曝光卷型：已打印卷型，多个以”/“分隔，如A/B/C/D',
        `relate_paper_type` varchar(5) DEFAULT NULL COMMENT '已曝光卷型',
        `unexposed_paper_type` varchar(20) DEFAULT NULL COMMENT '未曝光卷型',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        `enable` tinyint DEFAULT '1' COMMENT '0-禁用，1-启用',
        PRIMARY KEY (`id`) USING BTREE,
        KEY `exam_task_detail_idx` (`exam_task_id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='命题任务明细表（入库申请）';

-- ----------------------------
-- Table structure for exam_task_paper_log
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_task_paper_log` (
        `id` bigint NOT NULL COMMENT '主键',
        `exam_task_id` bigint NOT NULL COMMENT '命题任务ID',
        `paper_type` varchar(45) DEFAULT NULL COMMENT '试卷类型',
        `paper_attachment_ids` mediumtext COMMENT '修改后试卷',
        `card_id` bigint DEFAULT NULL COMMENT '修改后题卡ID',
        `review` tinyint(1) DEFAULT NULL COMMENT '是否审核：true-审核，false-未审核',
        `review_status` varchar(45) DEFAULT NULL COMMENT '审核状态：PASS-通过，NOT_PASS-不通过',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE,
        KEY `exam_task_paper_idx` (`exam_task_id`,`review`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='卷库修改审核临时表';

-- ----------------------------
-- Table structure for exam_task_print
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_task_print` (
        `id` bigint NOT NULL,
        `school_id` bigint DEFAULT NULL,
        `org_id` bigint DEFAULT NULL,
        `print_plan_id` bigint DEFAULT NULL COMMENT '印刷计划ID',
        `course_code` varchar(45) DEFAULT NULL COMMENT '课程代码',
        `course_name` varchar(50) DEFAULT NULL COMMENT '课程名称',
        `paper_number` varchar(45) DEFAULT NULL COMMENT '试卷编号',
        `exam_start_time` bigint DEFAULT NULL COMMENT '考试开始时间',
        `exam_end_time` bigint DEFAULT NULL COMMENT '考试结束时间',
        `package_seq` varchar(30) DEFAULT NULL COMMENT '卷袋序号',
        `exam_place` varchar(100) DEFAULT NULL,
        `exam_room` varchar(100) DEFAULT NULL,
        `invigilator_teacher` varchar(45) DEFAULT NULL COMMENT '监考老师',
        `class_id` varchar(100) DEFAULT NULL COMMENT '班级ID集合',
        `class_name` varchar(500) DEFAULT NULL COMMENT '班级名称集合',
        `student_count` int DEFAULT NULL COMMENT '考生总人数',
        `campus_id` bigint DEFAULT NULL COMMENT '校区ID',
        `print_house_id` bigint DEFAULT NULL COMMENT '印刷室ID',
        `print_house_name` varchar(100) DEFAULT NULL COMMENT '印刷室名称',
        `exam_detail_id` bigint DEFAULT NULL COMMENT '考场ID',
        `extend_fields` text COMMENT '扩展字段',
        `create_id` bigint DEFAULT NULL,
        `create_time` bigint DEFAULT NULL,
        `update_id` bigint DEFAULT NULL,
        `update_time` bigint DEFAULT NULL,
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='以试卷作为起点生成考务数据';

-- ----------------------------
-- Table structure for exam_task_review_log
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_task_review_log` (
        `id` bigint NOT NULL COMMENT '主键',
        `exam_task_id` bigint NOT NULL COMMENT '命题任务ID',
        `review_status` varchar(8) NOT NULL COMMENT 'PASS-通过，NOT_PASS-不通过',
        `reason` mediumtext COMMENT '审核意见',
        `operate_id` bigint DEFAULT NULL COMMENT '操作人id',
        `operate_time` bigint DEFAULT NULL COMMENT '操作时间',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='命题任务审核日志表';

-- ----------------------------
-- Table structure for exam_task_sync
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_task_sync` (
        `id` bigint NOT NULL,
        `school_id` bigint NOT NULL,
        `exam_id` bigint NOT NULL COMMENT '考试ID',
        `course_code` varchar(50) NOT NULL COMMENT '课程代码',
        `paper_number` varchar(50) NOT NULL COMMENT '试着编号',
        `paper_type` varchar(50) DEFAULT NULL COMMENT '试卷类型',
        `total_paper_type` varchar(50) DEFAULT NULL COMMENT '所有卷型',
        `org_code` varchar(45) DEFAULT NULL COMMENT '机构代码',
        `third_relate_id` int DEFAULT NULL COMMENT '云阅卷考试ID',
        `sync_status` varchar(45) DEFAULT NULL COMMENT '同步状态',
        `error_msg` varchar(500) DEFAULT NULL COMMENT '同步失败原因',
        `sync_user_id` bigint DEFAULT NULL COMMENT '同步用户ID',
        `sync_start_time` bigint DEFAULT NULL COMMENT '同步开始时间 ',
        `sync_end_time` bigint DEFAULT NULL COMMENT '同步结束时间',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for exam_task_temp
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_task_temp` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `batch_no` varchar(32) NOT NULL COMMENT '本次导入数据临时ID',
        `course_code` varchar(20) NOT NULL COMMENT '课程代码',
        `course_name` varchar(50) NOT NULL COMMENT '课程名称',
        `paper_number` varchar(30) DEFAULT NULL COMMENT '试卷编号',
        `specialty` varchar(100) DEFAULT NULL COMMENT '适用专业',
        `user_account` varchar(45) DEFAULT NULL COMMENT '命题老师账号',
        `user_name` varchar(45) DEFAULT NULL COMMENT '命题老师姓名',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='命题任务临时表';

-- ----------------------------
-- Table structure for grade_batch
-- ----------------------------
CREATE TABLE IF NOT EXISTS `grade_batch` (
        `id` bigint NOT NULL COMMENT '分析批次主键',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `org_id` bigint NOT NULL COMMENT '机构id',
        `semester_id` bigint NOT NULL COMMENT '学期id',
        `exam_id` bigint NOT NULL COMMENT '知学知考考试id',
        `batch_code` varchar(32) NOT NULL COMMENT '批次编号（在学校下唯一）',
        `batch_name` varchar(32) NOT NULL COMMENT '批次名称（2020~2021学年上学期期末考试）',
        `batch_time` bigint NOT NULL COMMENT '分析批次时间',
        `status` varchar(32) NOT NULL COMMENT '批次状态\nSETTING_GRADE_PAPER("设置分析试卷"),PUSH_GRADE_BATCH("推送教研分析批次数据"),SETTING_GRADE_PAPER_PARAM("设置分析试卷参数"),READY_TO_CALCULATE("待计算"),FINISH_CALCULATE("完成计算")',
        `third_semester_id` bigint DEFAULT NULL COMMENT '教研分析学期id',
        `third_exam_id` bigint DEFAULT NULL COMMENT '第三方考试id',
        `create_id` bigint DEFAULT NULL,
        `create_time` bigint DEFAULT NULL,
        `update_id` bigint DEFAULT NULL,
        `update_time` bigint DEFAULT NULL,
        `source` varchar(1000) DEFAULT NULL COMMENT '原始数据',
        `progress` decimal(8,2) DEFAULT NULL COMMENT '进度',
        `report_file_path` varchar(500) DEFAULT NULL COMMENT '报告路径',
        `notify_time` bigint DEFAULT NULL COMMENT '回调时间',
        `result` varchar(30) DEFAULT NULL COMMENT '数据结果，SUCCESS：成功，ERROR：失败',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='分析-批次表';

-- ----------------------------
-- Table structure for grade_batch_paper
-- ----------------------------
CREATE TABLE IF NOT EXISTS `grade_batch_paper` (
        `id` bigint NOT NULL,
        `school_id` bigint NOT NULL COMMENT '学校id',
        `org_id` bigint NOT NULL COMMENT '机构id',
        `exam_id` bigint NOT NULL COMMENT '知学知考考试id',
        `batch_id` bigint NOT NULL COMMENT '分析批次id',
        `grade_paper_id` bigint DEFAULT NULL COMMENT '试卷id',
        `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
        `paper_type` varchar(16) NOT NULL COMMENT '试卷类型',
        `status` varchar(32) NOT NULL COMMENT '分析试卷状态\nSETTING_GRADE_PAPER("设置分析试卷"),PUSH_GRADE_BATCH("推送教研分析批次数据"),SETTING_GRADE_PAPER_PARAM("设置分析试卷参数"),READY_TO_CALCULATE("待计算"),FINISH_CALCULATE("完成计算")',
        `teach_college_id` bigint NOT NULL COMMENT '当前开课学院id',
        `teach_college_name` varchar(100) NOT NULL COMMENT '当前开课学院名称',
        `paper_name` varchar(100) DEFAULT NULL COMMENT '试卷名称',
        `enable` tinyint DEFAULT NULL COMMENT '是否可用',
        `publish_status` tinyint DEFAULT NULL COMMENT '是否发布',
        `grade_course_code` varchar(100) DEFAULT NULL COMMENT '推送教研分析后回传的教研分析课程编号（考试下唯一）',
        `create_id` bigint DEFAULT NULL,
        `create_time` bigint DEFAULT NULL,
        `update_id` bigint DEFAULT NULL,
        `update_time` bigint DEFAULT NULL,
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='分析-批次试卷关联关系表';

-- ----------------------------
-- Table structure for grade_batch_student
-- ----------------------------
CREATE TABLE IF NOT EXISTS `grade_batch_student` (
        `id` bigint NOT NULL,
        `batch_id` bigint DEFAULT NULL COMMENT '分析-批次表ID',
        `course_code` varchar(50) DEFAULT NULL COMMENT '课程编码',
        `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
        `paper_number` varchar(50) DEFAULT NULL COMMENT '试卷编号',
        `paper_type` varchar(16) DEFAULT NULL COMMENT '试卷类型',
        `student_code` varchar(45) DEFAULT NULL COMMENT '学号',
        `student_name` varchar(100) DEFAULT NULL COMMENT '学生姓名',
        `ticket_number` varchar(45) DEFAULT NULL COMMENT '准考证号',
        `teacher_name` varchar(100) DEFAULT NULL COMMENT '任课老师',
        `teacher_number` varchar(45) DEFAULT NULL COMMENT '任课老师工号',
        `create_id` bigint DEFAULT NULL,
        `create_time` bigint DEFAULT NULL,
        `update_id` bigint DEFAULT NULL,
        `update_time` bigint DEFAULT NULL,
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考务数据（包含任课老师）';

-- ----------------------------
-- Table structure for grade_batch_student_clazz
-- ----------------------------
CREATE TABLE IF NOT EXISTS `grade_batch_student_clazz` (
        `id` bigint NOT NULL,
        `batch_id` bigint DEFAULT NULL COMMENT '分析-批次表ID',
        `course_code` varchar(50) DEFAULT NULL COMMENT '课程编码',
        `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
        `paper_number` varchar(50) DEFAULT NULL COMMENT '试卷编号',
        `paper_type` varchar(16) DEFAULT NULL COMMENT '试卷类型',
        `clazz_id` varchar(45) DEFAULT NULL COMMENT '班级ID',
        `clazz_name` varchar(100) DEFAULT NULL COMMENT '班级名称',
        `teacher_name` varchar(100) DEFAULT NULL COMMENT '任课老师',
        `teacher_number` varchar(45) DEFAULT NULL COMMENT '任课老师工号',
        `create_id` bigint DEFAULT NULL,
        `create_time` bigint DEFAULT NULL,
        `update_id` bigint DEFAULT NULL,
        `update_time` bigint DEFAULT NULL,
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考务数据班级（包含任课老师）';

-- ----------------------------
-- Table structure for grade_module_define
-- ----------------------------
CREATE TABLE IF NOT EXISTS `grade_module_define` (
        `id` bigint NOT NULL COMMENT '模块熟练度主键',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `exam_id` bigint NOT NULL COMMENT '考试id',
        `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
        `paper_type` varchar(16) NOT NULL COMMENT '试卷类型',
        `paper_name` varchar(100) DEFAULT NULL COMMENT '试卷名称',
        `module_type` varchar(32) NOT NULL COMMENT '维度模块类型',
        `interpret` varchar(255) DEFAULT NULL COMMENT '解释',
        `remark` varchar(255) DEFAULT NULL COMMENT '备注',
        `define` varchar(255) DEFAULT NULL COMMENT '二级熟练度定义',
        `level` varchar(32) DEFAULT NULL COMMENT '等级',
        `min` varchar(32) DEFAULT NULL COMMENT '展示用最小边界',
        `max` varchar(32) DEFAULT NULL COMMENT '展示用最大边界',
        `scope` varchar(32) DEFAULT NULL COMMENT '取值范围（计算用）',
        `create_id` bigint DEFAULT NULL,
        `create_time` bigint DEFAULT NULL,
        `update_id` bigint DEFAULT NULL,
        `update_time` bigint DEFAULT NULL,
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='分析-模块二级考察点维度熟练度配置，及模块简要描述表';

-- ----------------------------
-- Table structure for grade_module_evaluation
-- ----------------------------
CREATE TABLE IF NOT EXISTS `grade_module_evaluation` (
        `id` bigint NOT NULL,
        `school_id` bigint NOT NULL COMMENT '学校id',
        `exam_id` bigint NOT NULL COMMENT '考试id',
        `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
        `paper_type` varchar(16) NOT NULL COMMENT '试卷类型',
        `paper_name` varchar(100) DEFAULT NULL COMMENT '试卷名称',
        `module_type` varchar(64) NOT NULL COMMENT '模块类型（知识、维度、素养）',
        `formula` varchar(64) NOT NULL COMMENT '计算公式规则',
        `scope` varchar(64) NOT NULL COMMENT '取值范围（计算用）',
        `level_code` varchar(32) NOT NULL COMMENT '等级',
        `level_name` varchar(32) NOT NULL COMMENT '水平等级层次',
        `result` text COMMENT '诊断结果',
        `advice` text COMMENT '学习建议',
        `create_id` bigint DEFAULT NULL,
        `create_time` bigint DEFAULT NULL,
        `update_id` bigint DEFAULT NULL,
        `update_time` bigint DEFAULT NULL,
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='校验-考察点模块评价建议表';

-- ----------------------------
-- Table structure for grade_paper
-- ----------------------------
CREATE TABLE IF NOT EXISTS `grade_paper` (
        `id` bigint NOT NULL COMMENT '试卷id',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `exam_id` bigint NOT NULL COMMENT '知学知考考试id',
        `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
        `paper_type` varchar(16) NOT NULL COMMENT '试卷类型',
        `paper_name` varchar(100) NOT NULL COMMENT '试卷名称',
        `start_time` bigint DEFAULT NULL COMMENT '该试卷考试开始时间',
        `end_time` bigint DEFAULT NULL COMMENT '该试卷考试结束时间',
        `coefficient` decimal(12,4) DEFAULT NULL COMMENT '赋分系数（赋值分=卷面成绩 +（卷面满分-卷面成绩）/ 赋分系数）',
        `total_score` decimal(12,4) NOT NULL COMMENT '试卷满分',
        `pass_score` decimal(12,4) NOT NULL COMMENT '试卷及格分',
        `score_type` varchar(64) NOT NULL COMMENT '赋分类型\\n1.NEED_ASSIGN_SCORE("是","需要且按照公式赋分")\\n2.NO_NEED_ASSIGN_SCORE("否","不需要赋分，标准分即为赋分")\\n3.SPECIAL_ASSIGN_SCORE("特殊赋分","特殊赋分，按照excel表进行赋分")',
        `create_id` bigint DEFAULT NULL,
        `create_time` bigint DEFAULT NULL,
        `update_id` bigint DEFAULT NULL,
        `update_time` bigint DEFAULT NULL,
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='试卷基础信息表（赋分系数、试卷类型、满分、及格分等）';

-- ----------------------------
-- Table structure for grade_paper_dimension
-- ----------------------------
CREATE TABLE IF NOT EXISTS `grade_paper_dimension` (
        `id` bigint NOT NULL,
        `school_id` bigint NOT NULL COMMENT '学校id',
        `exam_id` bigint NOT NULL COMMENT '考试id',
        `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
        `paper_type` varchar(16) NOT NULL COMMENT '试卷类型',
        `paper_name` varchar(100) NOT NULL COMMENT '试卷名称',
        `dimension_type` varchar(32) NOT NULL COMMENT '维度类型 - 知识，能力，素养',
        `code_primary` varchar(32) NOT NULL COMMENT '一级维度编号',
        `name_primary` varchar(255) NOT NULL COMMENT '一级维度名称',
        `code_second` varchar(32) DEFAULT NULL COMMENT '二级维度编号',
        `name_second` varchar(255) DEFAULT NULL COMMENT '二级维度名称',
        `interpretation` varchar(255) DEFAULT NULL COMMENT '一级维度术语解释',
        `create_id` bigint DEFAULT NULL,
        `create_time` bigint DEFAULT NULL,
        `update_id` bigint DEFAULT NULL,
        `update_time` bigint DEFAULT NULL,
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='分析-考察点维度表';

-- ----------------------------
-- Table structure for grade_paper_struct
-- ----------------------------
CREATE TABLE IF NOT EXISTS `grade_paper_struct` (
        `id` bigint NOT NULL,
        `school_id` bigint NOT NULL COMMENT '学校id',
        `exam_id` bigint NOT NULL COMMENT '知学知考考试id',
        `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
        `paper_type` varchar(16) NOT NULL COMMENT '试卷类型',
        `paper_name` varchar(100) NOT NULL COMMENT '试卷名称',
        `question_name` varchar(200) NOT NULL COMMENT '大题名称',
        `number_type` varchar(32) NOT NULL COMMENT '题号类型（客观题、主观题）',
        `big_question_number` varchar(64) NOT NULL COMMENT '大题号',
        `small_question_number` varchar(64) NOT NULL COMMENT '小题号(每一课程试卷的唯一标识)',
        `big_topic_name` varchar(64) NOT NULL COMMENT '大题名称',
        `full_score` decimal(12,4) NOT NULL COMMENT '满分',
        `score_rules` varchar(32) DEFAULT NULL COMMENT '计分规则',
        `rules_desc` varchar(500) DEFAULT NULL COMMENT '规则说明',
        `knowledge_dimension` varchar(255) NOT NULL COMMENT '知识维度',
        `ability_dimension` varchar(255) DEFAULT NULL COMMENT '能力维度',
        `literacy_dimension` varchar(255) DEFAULT NULL COMMENT '素养维度',
        `create_id` bigint DEFAULT NULL,
        `create_time` bigint DEFAULT NULL,
        `update_id` bigint DEFAULT NULL,
        `update_time` bigint DEFAULT NULL,
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='分析-试卷结构（蓝图）表';

-- ----------------------------
-- Table structure for mark_arbitrate_history
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_arbitrate_history` (
        `id` bigint NOT NULL COMMENT '自增主键',
        `exam_id` bigint NOT NULL COMMENT '考试ID',
        `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
        `group_number` int NOT NULL COMMENT '大题号',
        `student_code` varchar(64) NOT NULL COMMENT '准考证号',
        `secret_number` varchar(64) NOT NULL COMMENT '考生密号',
        `student_id` bigint NOT NULL COMMENT '考生ID',
        `status` varchar(16) NOT NULL COMMENT '状态',
        `update_user_id` bigint DEFAULT NULL COMMENT '处理人ID',
        `total_score` double DEFAULT NULL COMMENT '总分',
        `score_list` text COMMENT '给分明细',
        `create_time` bigint NOT NULL COMMENT '创建时间',
        `update_time` bigint DEFAULT NULL COMMENT '处理时间',
        PRIMARY KEY (`id`) USING BTREE,
        KEY `index1` (`exam_id`,`paper_number`,`group_number`,`status`) USING BTREE,
        KEY `index3` (`student_id`,`status`) USING BTREE,
        KEY `index2` (`update_user_id`,`status`,`update_time`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='仲裁记录表';

-- ----------------------------
-- Table structure for mark_group
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_group` (
        `exam_id` bigint NOT NULL COMMENT '考试ID',
        `course_code` varchar(100) DEFAULT NULL COMMENT '课程代码',
        `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
        `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
        `number` int NOT NULL COMMENT '序号',
        `pic_list` text COMMENT '小图配置',
        `total_score` double NOT NULL COMMENT '满分',
        `double_enable` tinyint(1) DEFAULT '0' COMMENT '是否开启双评',
        `double_rate` double DEFAULT NULL COMMENT '双评比例',
        `arbitrate_threshold` double DEFAULT NULL COMMENT '仲裁阈值',
        `score_policy` varchar(16) DEFAULT NULL COMMENT '合分策略',
        `task_count` int NOT NULL COMMENT '任务总量',
        `marked_count` int NOT NULL COMMENT '已评数量',
        `left_count` int DEFAULT NULL COMMENT '剩余数量',
        PRIMARY KEY (`exam_id`,`paper_number`,`number`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='评卷分组表';

-- ----------------------------
-- Table structure for mark_group_student
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_group_student` (
        `student_id` bigint NOT NULL COMMENT '考生ID',
        `group_number` int NOT NULL COMMENT '分组ID',
        `exam_id` bigint NOT NULL COMMENT '考试ID',
        `paper_number` varchar(32) NOT NULL COMMENT '科目代码',
        `status` varchar(16) NOT NULL COMMENT '状态',
        PRIMARY KEY (`student_id`,`group_number`) USING BTREE,
        KEY `index1` (`exam_id`,`paper_number`,`group_number`,`status`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考生分组状态表';

-- ----------------------------
-- Table structure for mark_header_subjective_score
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_header_subjective_score` (
        `student_id` bigint NOT NULL COMMENT '考生ID',
        `main_number` int NOT NULL COMMENT '大题号',
        `sub_number` varchar(32) NOT NULL COMMENT '小题号',
        `exam_id` bigint NOT NULL COMMENT '考试ID',
        `paper_number` varchar(100) NOT NULL COMMENT '课程代码',
        `group_number` int NOT NULL COMMENT '分组序号',
        `group_score` double NOT NULL COMMENT '分组得分',
        `main_score` double NOT NULL COMMENT '大题得分',
        `score` double NOT NULL COMMENT '小题得分',
        `uncalculate` tinyint NOT NULL COMMENT '是否合分',
        `user_id` bigint DEFAULT NULL COMMENT '科组长ID',
        PRIMARY KEY (`student_id`,`main_number`,`sub_number`) USING BTREE,
        KEY `index1` (`exam_id`,`paper_number`,`main_number`,`main_score`) USING BTREE,
        KEY `index11` (`exam_id`,`paper_number`,`main_number`,`main_score`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='主观题得分明细表';

-- ----------------------------
-- Table structure for mark_header_tag
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_header_tag` (
        `id` bigint NOT NULL COMMENT '主键',
        `exam_id` bigint NOT NULL COMMENT '考试ID',
        `paper_number` varchar(45) NOT NULL COMMENT '试卷编号',
        `student_id` bigint NOT NULL COMMENT '考生ID',
        `group_number` int NOT NULL COMMENT '大题题号',
        `user_id` bigint NOT NULL COMMENT '用户ID',
        `tag_name` varchar(64) NOT NULL COMMENT '标记内容',
        `position_x` double NOT NULL COMMENT 'X轴位置',
        `position_y` double NOT NULL COMMENT 'Y轴位置',
        `offset_index` int NOT NULL COMMENT '裁切图序号',
        `offset_x` int NOT NULL COMMENT '裁切图X轴坐标',
        `offset_y` int NOT NULL COMMENT '裁切图Y轴坐标',
        `tag_type` varchar(64) DEFAULT NULL COMMENT '标记类型',
        PRIMARY KEY (`id`) USING BTREE,
        KEY `index1` (`student_id`,`group_number`) USING BTREE,
        KEY `index2` (`user_id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='组长特殊标记表';

-- ----------------------------
-- Table structure for mark_header_track
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_header_track` (
        `student_id` bigint NOT NULL COMMENT '考生ID',
        `question_number` varchar(128) NOT NULL COMMENT '完整题号',
        `number` int NOT NULL COMMENT '序号',
        `exam_id` bigint NOT NULL COMMENT '考试ID',
        `paper_number` varchar(100) NOT NULL COMMENT '科目代码',
        `group_number` int NOT NULL COMMENT '大题题号',
        `user_id` bigint NOT NULL COMMENT '用户ID',
        `score` double NOT NULL COMMENT '给分',
        `position_x` double NOT NULL COMMENT 'X轴位置',
        `position_y` double NOT NULL COMMENT 'Y轴位置',
        `offset_index` int NOT NULL COMMENT '裁切图序号',
        `offset_x` int NOT NULL COMMENT '裁切图X轴坐标',
        `offset_y` int NOT NULL COMMENT '裁切图Y轴坐标',
        PRIMARY KEY (`student_id`,`question_number`,`number`) USING BTREE,
        KEY `index1` (`student_id`,`group_number`) USING BTREE,
        KEY `index2` (`user_id`) USING BTREE,
        KEY `index3` (`exam_id`,`paper_number`,`group_number`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='组长轨迹给分表';

-- ----------------------------
-- Table structure for mark_paper
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_paper` (
        `id` bigint NOT NULL,
        `exam_id` bigint NOT NULL COMMENT '考试ID',
        `course_code` varchar(32) NOT NULL COMMENT '课程代码',
        `course_name` varchar(100) NOT NULL COMMENT '科目名称',
        `paper_number` varchar(64) DEFAULT NULL COMMENT '试卷编号',
        `course_paper_id` varchar(32) DEFAULT NULL COMMENT '试卷编号（交互）',
        `objective_score` double NOT NULL COMMENT '客观题满分',
        `subjective_score` double NOT NULL COMMENT '主观题满分',
        `total_score` double NOT NULL COMMENT '全卷满分',
        `student_count` int DEFAULT NULL COMMENT '考生人数',
        `upload_count` int NOT NULL COMMENT '已上传人数',
        `absent_count` int DEFAULT NULL COMMENT '缺考人数',
        `remark` varchar(128) DEFAULT NULL COMMENT '备注',
        `sheet_config` text COMMENT '原图遮盖配置',
        `pass_score` double DEFAULT NULL COMMENT '及格分',
        `excellent_score` double DEFAULT NULL COMMENT '优秀分',
        `card_type` varchar(16) DEFAULT NULL COMMENT '题卡类型',
        `paper_file_path` varchar(500) DEFAULT NULL COMMENT '试卷文件类型',
        `answer_file_path` varchar(500) DEFAULT NULL COMMENT '标答文件类型',
        `auto_scroll` tinyint(1) DEFAULT NULL COMMENT '评卷是否跳转',
        `mark_mode` varchar(16) DEFAULT NULL COMMENT '评卷模式',
        `force_mode` tinyint(1) NULL DEFAULT 1 COMMENT '强制评卷模式（不限为false）',
        `mark_start_time` bigint DEFAULT NULL COMMENT '评卷开始时间',
        `mark_end_time` bigint DEFAULT NULL COMMENT '评卷结束时间',
        `sheet_view` tinyint DEFAULT NULL COMMENT '是否原卷显示',
        `show_object_score` tinyint DEFAULT NULL COMMENT '评卷是否显示客观分',
        `group_status` tinyint DEFAULT '0' COMMENT '评卷分组是否提交',
        `open_mark_class` tinyint DEFAULT '0' COMMENT '是否开启分班阅',
        `status` varchar(10) DEFAULT NULL COMMENT '评卷状态',
        `user_id` bigint DEFAULT NULL COMMENT '命题老师ID',
        `paper_type` varchar(10) DEFAULT NULL COMMENT '试卷类型',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考试科目表';

-- ----------------------------
-- Table structure for mark_paper_package
-- ----------------------------
CREATE TABLE `mark_paper_package` (
        `exam_id` BIGINT(20) NOT NULL COMMENT '考试ID',
        `course_code` varchar(32) NOT NULL COMMENT '课程代码',
        `course_name` varchar(100) NOT NULL COMMENT '科目名称',
        `paper_number` VARCHAR(50) NULL COMMENT '试卷编号',
        `course_paper_id` VARCHAR(50) NOT NULL COMMENT '课程试卷号',
        `paper_type` varchar(10) NOT NULL COMMENT '试卷类型',
        `package_code` VARCHAR(10) NOT NULL COMMENT '卷袋号',
        PRIMARY KEY (`exam_id`, `course_paper_id`, `paper_type`, `package_code`));


-- ----------------------------
-- Table structure for mark_problem_history
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_problem_history` (
        `id` bigint NOT NULL COMMENT '自增主键',
        `exam_id` bigint NOT NULL COMMENT '考试ID',
        `paper_number` varchar(32) NOT NULL COMMENT '试卷编号',
        `group_number` int NOT NULL COMMENT '大题号',
        `student_code` varchar(64) NOT NULL COMMENT '学号',
        `secret_number` varchar(64) NOT NULL COMMENT '考生密号',
        `type` varchar(10) NOT NULL COMMENT '问题类型',
        `student_id` bigint NOT NULL COMMENT '考生ID',
        `task_id` bigint NOT NULL COMMENT '评卷任务ID',
        `status` varchar(16) DEFAULT NULL COMMENT '状态',
        `update_user_id` bigint DEFAULT NULL COMMENT '处理人ID',
        `user_id` bigint DEFAULT NULL COMMENT '评卷员ID',
        `score_list` text COMMENT '给分明细',
        `create_time` bigint NOT NULL COMMENT '创建时间',
        `update_time` bigint DEFAULT NULL COMMENT '处理时间',
        `remark` varchar(100) DEFAULT NULL COMMENT '问题原因',
        PRIMARY KEY (`id`) USING BTREE,
        KEY `index2` (`update_user_id`,`status`,`update_time`) USING BTREE,
        KEY `index3` (`student_id`,`status`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='问题卷历史表';

-- ----------------------------
-- Table structure for mark_question
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_question` (
        `id` bigint NOT NULL COMMENT '主键',
        `exam_id` bigint NOT NULL COMMENT '考试ID',
        `paper_number` varchar(100) NOT NULL COMMENT '试卷类型',
        `paper_type` varchar(100) DEFAULT NULL COMMENT '卷型',
        `objective` tinyint NOT NULL COMMENT '是否客观题',
        `group_number` int DEFAULT NULL COMMENT '分组序号',
        `main_number` int NOT NULL COMMENT '大题号',
        `sub_number` int NOT NULL COMMENT '小题号',
        `main_title` varchar(128) NOT NULL COMMENT '大题名称',
        `answer` varchar(16) DEFAULT NULL COMMENT '正确答案',
        `total_score` double DEFAULT NULL COMMENT '满分',
        `option_count` int DEFAULT NULL COMMENT '客观题答案数量',
        `interval_score` double DEFAULT NULL COMMENT '评卷间隔分',
        `objective_policy` varchar(16) DEFAULT NULL COMMENT '客观题判分策略',
        `question_type` varchar(2) DEFAULT NULL COMMENT '题型',
        `name` varchar(128) DEFAULT NULL,
        `paper_index` int DEFAULT NULL,
        `page_index` int DEFAULT NULL,
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='小题信息表';

-- ----------------------------
-- Table structure for mark_special_tag
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_special_tag` (
        `id` bigint NOT NULL COMMENT '主键',
        `task_id` bigint NOT NULL COMMENT '评卷任务ID',
        `tag_name` varchar(64) NOT NULL COMMENT '标记内容',
        `position_x` double NOT NULL COMMENT 'X轴位置',
        `position_y` double NOT NULL COMMENT 'Y轴位置',
        `offset_index` int NOT NULL COMMENT '裁切图序号',
        `offset_x` int NOT NULL COMMENT '裁切图X轴坐标',
        `offset_y` int NOT NULL COMMENT '裁切图Y轴坐标',
        `tag_type` varchar(64) DEFAULT NULL COMMENT '标记类型',
        PRIMARY KEY (`id`) USING BTREE,
        KEY `index1` (`task_id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='特殊标记表';

-- ----------------------------
-- Table structure for mark_student
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_student` (
        `id` bigint NOT NULL COMMENT '主键(和exam_student表id一样)',
        `exam_id` bigint NOT NULL COMMENT '考试ID',
        `course_code` varchar(32) NOT NULL COMMENT '科目代码',
        `course_name` varchar(100) NOT NULL COMMENT '科目名称',
        `paper_number` varchar(100) NOT NULL COMMENT '试卷类型',
        `course_paper_id` varchar(45) DEFAULT NULL COMMENT '试卷编号（交互）',
        `paper_type` varchar(10) DEFAULT NULL COMMENT '试卷编号',
        `secret_number` varchar(64) NOT NULL COMMENT '密号',
        `student_code` varchar(64) NOT NULL COMMENT '学号',
        `student_name` varchar(64) NOT NULL COMMENT '姓名',
        `package_code` varchar(64) NOT NULL COMMENT '试卷袋编号',
        `exam_place` varchar(32) DEFAULT NULL COMMENT '校区',
        `exam_room` varchar(32) DEFAULT NULL COMMENT '考场',
        `remark` varchar(128) DEFAULT NULL COMMENT '备注',
        `batch_code` varchar(32) DEFAULT NULL COMMENT '扫描批次号',
        `sheet_count` int NOT NULL COMMENT '原图数量',
        `sheet_path` mediumtext COMMENT '考生所有图片地址',
        `answers` text COMMENT '客观题识别结果',
        `is_upload` tinyint NOT NULL COMMENT '是否已上传',
        `is_absent` tinyint NOT NULL COMMENT '是否缺考',
        `is_manual_absent` tinyint NOT NULL COMMENT '是否人工指定缺考',
        `is_breach` tinyint NOT NULL COMMENT '是否违纪',
        `upload_time` bigint DEFAULT NULL COMMENT '上传时间',
        `check_time` bigint DEFAULT NULL COMMENT '检查时间',
        `check_user_id` bigint DEFAULT NULL COMMENT '检查人ID',
        `objective_score` double DEFAULT NULL COMMENT '客观总分',
        `objective_score_list` text COMMENT '客观得分明细',
        `subjective_status` varchar(16) NOT NULL COMMENT '主观题状态',
        `subjective_score` double DEFAULT NULL COMMENT '主观总分',
        `subjective_score_list` text COMMENT '主观得分明细',
        `college` varchar(64) DEFAULT NULL COMMENT '学院',
        `major_name` VARCHAR(100) NULL COMMENT '专业',
        `class_name` varchar(64) DEFAULT NULL COMMENT '班级',
        `teacher` varchar(64) DEFAULT NULL COMMENT '任课老师',
        `scan_status` varchar(20) DEFAULT NULL COMMENT '扫描状态',
        `question_filled` bit(1) DEFAULT NULL,
        `assigned` bit(1) DEFAULT NULL,
        `assign_confirmed` bit(1) DEFAULT 0 COMMENT '人工绑定确认',
        `absent_suspect` bit(1) DEFAULT NULL,
        `incomplete` bit(1) DEFAULT NULL,
        `breach_code` varchar(32) DEFAULT NULL,
        `omr_absent` bit(1) DEFAULT NULL COMMENT '识别缺考',
        `omr_absent_checked` bit(1) DEFAULT NULL COMMENT '识别缺考检查',
        `card_number` int DEFAULT NULL,
        `exam_start_time` BIGINT(20) NULL COMMENT '考试开始时间',
        `exam_end_time` BIGINT(20) NULL COMMENT '考试结束时间',
        `create_id` BIGINT(20) NULL COMMENT '创建人ID',
        PRIMARY KEY (`id`) USING BTREE,
        UNIQUE KEY `index1` (`exam_id`,`student_code`,`course_paper_id`) USING BTREE,
        UNIQUE KEY `index2` (`exam_id`,`secret_number`) USING BTREE,
        KEY `index3` (`exam_id`,`course_code`,`upload_time`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考试考生库';

-- ----------------------------
-- Table structure for mark_subjective_score
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_subjective_score` (
        `student_id` bigint NOT NULL COMMENT '考生ID',
        `main_number` int NOT NULL COMMENT '大题号',
        `sub_number` varchar(32) NOT NULL COMMENT '小题号',
        `exam_id` bigint NOT NULL COMMENT '考试ID',
        `paper_number` varchar(100) NOT NULL COMMENT '课程代码',
        `group_number` int NOT NULL COMMENT '分组序号',
        `group_score` double NOT NULL COMMENT '分组得分',
        `main_score` double NOT NULL COMMENT '大题得分',
        `score` double NOT NULL COMMENT '小题得分',
        `uncalculate` tinyint(1) NOT NULL COMMENT '是否合分',
        PRIMARY KEY (`student_id`,`main_number`,`sub_number`) USING BTREE,
        KEY `index1` (`exam_id`,`paper_number`,`main_number`,`main_score`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='主观题得分明细表';

-- ----------------------------
-- Table structure for mark_task
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_task` (
        `id` bigint NOT NULL COMMENT '主键',
        `exam_id` bigint NOT NULL COMMENT '考试ID',
        `course_code` varchar(32) NOT NULL COMMENT '课程代码',
        `course_name` varchar(100) NOT NULL COMMENT '课程名称',
        `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
        `group_number` int NOT NULL COMMENT '大题号',
        `student_id` bigint NOT NULL COMMENT '考生ID',
        `student_code` varchar(64) NOT NULL COMMENT '学号',
        `secret_number` varchar(64) NOT NULL COMMENT '考生密号',
        `task_number` int NOT NULL COMMENT '多评任务序号',
        `status` varchar(16) NOT NULL COMMENT '状态',
        `user_id` bigint DEFAULT NULL COMMENT '评卷员ID',
        `marker_time` bigint DEFAULT NULL COMMENT '评卷时间',
        `marker_score` double DEFAULT NULL COMMENT '评卷总分',
        `marker_score_list` text COMMENT '评卷给分明细',
        `marker_spent` int DEFAULT NULL COMMENT '评卷时长',
        `header_id` bigint DEFAULT NULL COMMENT '科组长ID',
        `header_time` bigint DEFAULT NULL COMMENT '科组长评卷时间',
        `header_score` double DEFAULT NULL COMMENT '科组长总分',
        `header_score_list` text COMMENT '科组长给分明细',
        PRIMARY KEY (`id`) USING BTREE,
        UNIQUE KEY `index2` (`student_id`,`group_number`,`task_number`) USING BTREE,
        KEY `index3` (`user_id`,`status`,`marker_time`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='评卷任务表';

-- ----------------------------
-- Table structure for mark_track
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_track` (
        `id` bigint NOT NULL,
        `task_id` bigint NOT NULL COMMENT '评卷任务ID',
        `question_number` varchar(128) NOT NULL COMMENT '完整题号',
        `number` int NOT NULL COMMENT '序号',
        `student_id` bigint NOT NULL COMMENT '考生ID',
        `exam_id` bigint NOT NULL COMMENT '考试ID',
        `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
        `group_number` int NOT NULL COMMENT '大题题号',
        `user_id` bigint NOT NULL COMMENT '评卷员ID',
        `type` varchar(10) DEFAULT NULL COMMENT '标记类型（SCORE：分数，REMARK：备注）',
        `score` double DEFAULT NULL COMMENT '给分',
        `remark` varchar(20) DEFAULT NULL COMMENT '备注',
        `position_x` double NOT NULL COMMENT 'X轴位置',
        `position_y` double NOT NULL COMMENT 'Y轴位置',
        `offset_index` int NOT NULL COMMENT '裁切图序号',
        `offset_x` int NOT NULL COMMENT '裁切图X轴坐标',
        `offset_y` int NOT NULL COMMENT '裁切图Y轴坐标',
        PRIMARY KEY (`id`) USING BTREE,
        KEY `index2` (`user_id`) USING BTREE,
        KEY `index3` (`exam_id`,`paper_number`,`group_number`) USING BTREE,
        KEY `index1` (`exam_id`,`task_id`,`student_id`,`group_number`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='轨迹给分表';

-- ----------------------------
-- Table structure for mark_user_class
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_user_class` (
        `Id` bigint NOT NULL,
        `exam_id` bigint DEFAULT NULL COMMENT '\n考试ID',
        `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷编号',
        `group_number` int DEFAULT NULL COMMENT '分组号',
        `user_id` bigint unsigned DEFAULT NULL COMMENT '评卷员ID',
        `class_name` varchar(100) DEFAULT NULL COMMENT '班级名称',
        PRIMARY KEY (`Id`) USING BTREE,
        KEY `idx_1` (`exam_id`,`paper_number`,`group_number`,`class_name`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='评卷员班级';

-- ----------------------------
-- Table structure for mark_user_group
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_user_group` (
        `id` bigint NOT NULL COMMENT '主键',
        `exam_id` bigint NOT NULL COMMENT '考试ID',
        `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
        `group_number` int NOT NULL COMMENT '分组编号',
        `user_id` bigint NOT NULL COMMENT '用户ID',
        `mode` varchar(16) DEFAULT NULL COMMENT '强制评卷模式',
        `top_count` int DEFAULT NULL COMMENT '评卷数上限',
        `finish_count` int DEFAULT NULL COMMENT '完成数量',
        `header_finish_count` int DEFAULT NULL COMMENT '管理员直接打分数量',
        `valid_count` int DEFAULT NULL COMMENT '有效数量',
        `avg_score` double DEFAULT NULL COMMENT '平均分',
        `avg_speed` double DEFAULT NULL COMMENT '平均时长',
        `stdev_score` double DEFAULT NULL COMMENT '标准差',
        `max_score` double DEFAULT NULL COMMENT '最高分',
        `min_score` double DEFAULT NULL COMMENT '最低分',
        PRIMARY KEY (`id`) USING BTREE,
        KEY `index2` (`exam_id`,`paper_number`,`group_number`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='评卷员表';

-- ----------------------------
-- Table structure for qrtz_blob_triggers
-- ----------------------------
CREATE TABLE IF NOT EXISTS `qrtz_blob_triggers` (
    `SCHED_NAME` varchar(120) NOT NULL,
    `TRIGGER_NAME` varchar(200) NOT NULL,
    `TRIGGER_GROUP` varchar(200) NOT NULL,
    `BLOB_DATA` blob,
    PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE,
    KEY `SCHED_NAME` (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_calendars
-- ----------------------------
CREATE TABLE IF NOT EXISTS `qrtz_calendars` (
    `SCHED_NAME` varchar(120) NOT NULL,
    `CALENDAR_NAME` varchar(200) NOT NULL,
    `CALENDAR` blob NOT NULL,
    PRIMARY KEY (`SCHED_NAME`,`CALENDAR_NAME`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_cron_triggers
-- ----------------------------
CREATE TABLE IF NOT EXISTS `qrtz_cron_triggers` (
    `SCHED_NAME` varchar(120) NOT NULL,
    `TRIGGER_NAME` varchar(200) NOT NULL,
    `TRIGGER_GROUP` varchar(200) NOT NULL,
    `CRON_EXPRESSION` varchar(120) NOT NULL,
    `TIME_ZONE_ID` varchar(80) DEFAULT NULL,
    PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_fired_triggers
-- ----------------------------
CREATE TABLE IF NOT EXISTS `qrtz_fired_triggers` (
    `SCHED_NAME` varchar(120) NOT NULL,
    `ENTRY_ID` varchar(95) NOT NULL,
    `TRIGGER_NAME` varchar(200) NOT NULL,
    `TRIGGER_GROUP` varchar(200) NOT NULL,
    `INSTANCE_NAME` varchar(200) NOT NULL,
    `FIRED_TIME` bigint NOT NULL,
    `SCHED_TIME` bigint NOT NULL,
    `PRIORITY` int NOT NULL,
    `STATE` varchar(16) NOT NULL,
    `JOB_NAME` varchar(200) DEFAULT NULL,
    `JOB_GROUP` varchar(200) DEFAULT NULL,
    `IS_NONCONCURRENT` varchar(1) DEFAULT NULL,
    `REQUESTS_RECOVERY` varchar(1) DEFAULT NULL,
    PRIMARY KEY (`SCHED_NAME`,`ENTRY_ID`) USING BTREE,
    KEY `IDX_QRTZ_FT_TRIG_INST_NAME` (`SCHED_NAME`,`INSTANCE_NAME`) USING BTREE,
    KEY `IDX_QRTZ_FT_INST_JOB_REQ_RCVRY` (`SCHED_NAME`,`INSTANCE_NAME`,`REQUESTS_RECOVERY`) USING BTREE,
    KEY `IDX_QRTZ_FT_J_G` (`SCHED_NAME`,`JOB_NAME`,`JOB_GROUP`) USING BTREE,
    KEY `IDX_QRTZ_FT_JG` (`SCHED_NAME`,`JOB_GROUP`) USING BTREE,
    KEY `IDX_QRTZ_FT_T_G` (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE,
    KEY `IDX_QRTZ_FT_TG` (`SCHED_NAME`,`TRIGGER_GROUP`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_job_details
-- ----------------------------
CREATE TABLE IF NOT EXISTS `qrtz_job_details` (
    `SCHED_NAME` varchar(120) NOT NULL,
    `JOB_NAME` varchar(200) NOT NULL,
    `JOB_GROUP` varchar(200) NOT NULL,
    `DESCRIPTION` varchar(250) DEFAULT NULL,
    `JOB_CLASS_NAME` varchar(250) NOT NULL,
    `IS_DURABLE` varchar(1) NOT NULL,
    `IS_NONCONCURRENT` varchar(1) NOT NULL,
    `IS_UPDATE_DATA` varchar(1) NOT NULL,
    `REQUESTS_RECOVERY` varchar(1) NOT NULL,
    `JOB_DATA` blob,
    PRIMARY KEY (`SCHED_NAME`,`JOB_NAME`,`JOB_GROUP`) USING BTREE,
    KEY `IDX_QRTZ_J_REQ_RECOVERY` (`SCHED_NAME`,`REQUESTS_RECOVERY`) USING BTREE,
    KEY `IDX_QRTZ_J_GRP` (`SCHED_NAME`,`JOB_GROUP`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_locks
-- ----------------------------
CREATE TABLE IF NOT EXISTS `qrtz_locks` (
    `SCHED_NAME` varchar(120) NOT NULL,
    `LOCK_NAME` varchar(40) NOT NULL,
    PRIMARY KEY (`SCHED_NAME`,`LOCK_NAME`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_paused_trigger_grps
-- ----------------------------
CREATE TABLE IF NOT EXISTS `qrtz_paused_trigger_grps` (
    `SCHED_NAME` varchar(120) NOT NULL,
    `TRIGGER_GROUP` varchar(200) NOT NULL,
    PRIMARY KEY (`SCHED_NAME`,`TRIGGER_GROUP`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_scheduler_state
-- ----------------------------
CREATE TABLE IF NOT EXISTS `qrtz_scheduler_state` (
    `SCHED_NAME` varchar(120) NOT NULL,
    `INSTANCE_NAME` varchar(200) NOT NULL,
    `LAST_CHECKIN_TIME` bigint NOT NULL,
    `CHECKIN_INTERVAL` bigint NOT NULL,
    PRIMARY KEY (`SCHED_NAME`,`INSTANCE_NAME`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_simple_triggers
-- ----------------------------
CREATE TABLE IF NOT EXISTS `qrtz_simple_triggers` (
    `SCHED_NAME` varchar(120) NOT NULL,
    `TRIGGER_NAME` varchar(200) NOT NULL,
    `TRIGGER_GROUP` varchar(200) NOT NULL,
    `REPEAT_COUNT` bigint NOT NULL,
    `REPEAT_INTERVAL` bigint NOT NULL,
    `TIMES_TRIGGERED` bigint NOT NULL,
    PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_simprop_triggers
-- ----------------------------
CREATE TABLE IF NOT EXISTS `qrtz_simprop_triggers` (
    `SCHED_NAME` varchar(120) NOT NULL,
    `TRIGGER_NAME` varchar(200) NOT NULL,
    `TRIGGER_GROUP` varchar(200) NOT NULL,
    `STR_PROP_1` varchar(512) DEFAULT NULL,
    `STR_PROP_2` varchar(512) DEFAULT NULL,
    `STR_PROP_3` varchar(512) DEFAULT NULL,
    `INT_PROP_1` int DEFAULT NULL,
    `INT_PROP_2` int DEFAULT NULL,
    `LONG_PROP_1` bigint DEFAULT NULL,
    `LONG_PROP_2` bigint DEFAULT NULL,
    `DEC_PROP_1` decimal(13,4) DEFAULT NULL,
    `DEC_PROP_2` decimal(13,4) DEFAULT NULL,
    `BOOL_PROP_1` varchar(1) DEFAULT NULL,
    `BOOL_PROP_2` varchar(1) DEFAULT NULL,
    PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_triggers
-- ----------------------------
CREATE TABLE IF NOT EXISTS `qrtz_triggers` (
    `SCHED_NAME` varchar(120) NOT NULL,
    `TRIGGER_NAME` varchar(200) NOT NULL,
    `TRIGGER_GROUP` varchar(200) NOT NULL,
    `JOB_NAME` varchar(200) NOT NULL,
    `JOB_GROUP` varchar(200) NOT NULL,
    `DESCRIPTION` varchar(250) DEFAULT NULL,
    `NEXT_FIRE_TIME` bigint DEFAULT NULL,
    `PREV_FIRE_TIME` bigint DEFAULT NULL,
    `PRIORITY` int DEFAULT NULL,
    `TRIGGER_STATE` varchar(16) NOT NULL,
    `TRIGGER_TYPE` varchar(8) NOT NULL,
    `START_TIME` bigint NOT NULL,
    `END_TIME` bigint DEFAULT NULL,
    `CALENDAR_NAME` varchar(200) DEFAULT NULL,
    `MISFIRE_INSTR` smallint DEFAULT NULL,
    `JOB_DATA` blob,
    PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE,
    KEY `IDX_QRTZ_T_J` (`SCHED_NAME`,`JOB_NAME`,`JOB_GROUP`) USING BTREE,
    KEY `IDX_QRTZ_T_JG` (`SCHED_NAME`,`JOB_GROUP`) USING BTREE,
    KEY `IDX_QRTZ_T_C` (`SCHED_NAME`,`CALENDAR_NAME`) USING BTREE,
    KEY `IDX_QRTZ_T_G` (`SCHED_NAME`,`TRIGGER_GROUP`) USING BTREE,
    KEY `IDX_QRTZ_T_STATE` (`SCHED_NAME`,`TRIGGER_STATE`) USING BTREE,
    KEY `IDX_QRTZ_T_N_STATE` (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`,`TRIGGER_STATE`) USING BTREE,
    KEY `IDX_QRTZ_T_N_G_STATE` (`SCHED_NAME`,`TRIGGER_GROUP`,`TRIGGER_STATE`) USING BTREE,
    KEY `IDX_QRTZ_T_NEXT_FIRE_TIME` (`SCHED_NAME`,`NEXT_FIRE_TIME`) USING BTREE,
    KEY `IDX_QRTZ_T_NFT_ST` (`SCHED_NAME`,`TRIGGER_STATE`,`NEXT_FIRE_TIME`) USING BTREE,
    KEY `IDX_QRTZ_T_NFT_MISFIRE` (`SCHED_NAME`,`MISFIRE_INSTR`,`NEXT_FIRE_TIME`) USING BTREE,
    KEY `IDX_QRTZ_T_NFT_ST_MISFIRE` (`SCHED_NAME`,`MISFIRE_INSTR`,`NEXT_FIRE_TIME`,`TRIGGER_STATE`) USING BTREE,
    KEY `IDX_QRTZ_T_NFT_ST_MISFIRE_GRP` (`SCHED_NAME`,`MISFIRE_INSTR`,`NEXT_FIRE_TIME`,`TRIGGER_GROUP`,`TRIGGER_STATE`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for scan_answer_card
-- ----------------------------
CREATE TABLE IF NOT EXISTS `scan_answer_card` (
        `id` bigint NOT NULL,
        `exam_id` bigint NOT NULL COMMENT '考试ID',
        `paper_number` varchar(100) DEFAULT NULL COMMENT '科目代码',
        `course_paper_id` varchar(50) NOT NULL COMMENT '试卷编号（交互）',
        `paper_type` varchar(100) NOT NULL COMMENT '试卷编号',
        `card_id` bigint DEFAULT NULL COMMENT '知学知考题卡ID（自定义卡格式为空）',
        `number` int NOT NULL COMMENT '序号',
        `source` varchar(16) NOT NULL COMMENT '来源',
        `uri` varchar(500) DEFAULT NULL COMMENT '卡格式文件路径',
        `md5` varchar(255) DEFAULT NULL COMMENT '文件md5',
        `parameter` varchar(255) DEFAULT NULL COMMENT '其他属性',
        `remark` varchar(255) DEFAULT NULL COMMENT '备注',
        `update_time` bigint DEFAULT NULL COMMENT '修改时间',
        `need_adapte` tinyint DEFAULT NULL COMMENT '是否需要适配',
        `paper_count` int DEFAULT NULL,
        `single_page` tinyint(1) DEFAULT NULL,
        `slice_config` text COMMENT '裁切配置',
        `dpi` int DEFAULT NULL COMMENT 'dpi',
        `code` varchar(64) DEFAULT NULL COMMENT '混扫代码',
        `adapte_md5` varchar(32) DEFAULT NULL COMMENT '适配后文件md5',
        `adapte_uri` varchar(500) DEFAULT NULL,
        PRIMARY KEY (`id`) USING BTREE,
        UNIQUE KEY `idx_1` (`exam_id`,`number`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='题卡卡格式表';

-- ----------------------------
-- Table structure for scan_answer_card_subject
-- ----------------------------
CREATE TABLE IF NOT EXISTS `scan_answer_card_subject` (
        `id` bigint NOT NULL COMMENT '主键',
        `exam_id` bigint NOT NULL COMMENT '考试ID',
        `card_number` int NOT NULL COMMENT '卡格式号',
        `paper_number` varchar(255) NOT NULL COMMENT '试卷编号',
        `course_paper_id` varchar(50) NOT NULL COMMENT '试卷编号（交互）',
        PRIMARY KEY (`id`) USING BTREE,
        UNIQUE KEY `index1` (`exam_id`,`card_number`,`paper_number`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='卡格式科目关联关系';

-- ----------------------------
-- Table structure for scan_batch
-- ----------------------------
CREATE TABLE IF NOT EXISTS `scan_batch` (
        `id` bigint NOT NULL AUTO_INCREMENT,
        `exam_id` bigint NOT NULL COMMENT '考试ID',
        `user_id` bigint NOT NULL COMMENT '扫描员ID',
        `scan_count` int NOT NULL COMMENT '扫描数量',
        `assigned_count` int NOT NULL,
        `status` varchar(16) NOT NULL COMMENT '状态',
        `creator_id` bigint DEFAULT NULL,
        `updater_id` bigint DEFAULT NULL,
        `create_time` bigint DEFAULT NULL,
        `update_time` bigint DEFAULT NULL,
        `course_paper_id` varchar(64) DEFAULT NULL COMMENT '试卷编号',
        PRIMARY KEY (`id`) USING BTREE,
        KEY `exam_verify_status` (`exam_id`) USING BTREE
        ) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for scan_batch_paper
-- ----------------------------
CREATE TABLE IF NOT EXISTS `scan_batch_paper` (
        `batch_id` bigint NOT NULL COMMENT '批次ID',
        `student_id` bigint NOT NULL COMMENT '考生ID',
        `paper_index` int NOT NULL COMMENT '试卷页数',
        `paper_id` bigint NOT NULL COMMENT '试卷ID',
        `card_number` int NOT NULL COMMENT '题卡编号',
        `assigned` bit(1) NOT NULL,
        PRIMARY KEY (`batch_id`,`student_id`,`paper_index`) USING BTREE,
        UNIQUE KEY `paper_id` (`paper_id`) USING BTREE,
        UNIQUE KEY `paper_id_2` (`paper_id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for scan_file_property
-- ----------------------------
CREATE TABLE IF NOT EXISTS `scan_file_property` (
        `path` varchar(255) NOT NULL COMMENT '文件路径',
        `md5` varchar(32) NOT NULL COMMENT '文件md5',
        `exam_id` bigint NOT NULL COMMENT '考试ID',
        `file_size` bigint NOT NULL COMMENT '文件大小',
        `create_time` bigint DEFAULT NULL,
        `update_time` bigint DEFAULT NULL,
        PRIMARY KEY (`path`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for scan_log
-- ----------------------------
CREATE TABLE  IF NOT EXISTS `scan_log` (
      `school_code` VARCHAR(20) NOT NULL COMMENT '学校代码',
      `device_code` VARCHAR(50) NOT NULL COMMENT '设备号',
      `file_name` VARCHAR(45) NOT NULL COMMENT '文件名',
      `type` VARCHAR(10) NOT NULL COMMENT '错误文件类型，崩溃异常文件：COLLAPSE，正常文件：NORMAL',
      `md5` VARCHAR(45) NOT NULL COMMENT '扫描端日志',
      `path` VARCHAR(500) NULL COMMENT '文件存储路径',
      `login_name` VARCHAR(45) NULL COMMENT '扫描账号',
      `upload_time` BIGINT(20) NULL COMMENT '上传时间',
    PRIMARY KEY (`school_code`, `device_code`, `file_name`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for scan_omr_task
-- ----------------------------
CREATE TABLE IF NOT EXISTS `scan_omr_task` (
        `id` bigint NOT NULL,
        `exam_id` bigint NOT NULL COMMENT '考试ID',
        `conditions` varchar(100) NOT NULL COMMENT '条件',
        `student_id` bigint NOT NULL COMMENT '考生ID',
        `paper_index` int NOT NULL COMMENT '试卷页数',
        `paper_id` bigint NOT NULL COMMENT '试卷ID',
        `card_number` int NOT NULL COMMENT '题卡编号',
        `status` varchar(16) NOT NULL COMMENT '状态',
        `user_id` bigint DEFAULT NULL,
        `pages` text NOT NULL,
        `creator_id` bigint DEFAULT NULL,
        `updater_id` bigint DEFAULT NULL,
        `create_time` bigint DEFAULT NULL,
        `update_time` bigint DEFAULT NULL,
        PRIMARY KEY (`id`) USING BTREE,
        UNIQUE KEY `group_paper` (`conditions`,`paper_id`) USING BTREE,
        KEY `exam_status` (`exam_id`,`status`,`student_id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for scan_package
-- ----------------------------
CREATE TABLE IF NOT EXISTS `scan_package` (
        `id` bigint NOT NULL,
        `exam_id` bigint NOT NULL COMMENT '考试ID',
        `course_paper_id` varchar(30) DEFAULT NULL COMMENT '试卷编号',
        `package_code` varchar(20) DEFAULT NULL COMMENT '卷袋编号',
        `package_no` int DEFAULT NULL COMMENT '页码',
        `path` varchar(255) NOT NULL COMMENT '签到表图片路径',
        `md5` varchar(32) NOT NULL COMMENT '文件md5',
        `assigned` bit(1) DEFAULT NULL,
        `creator_id` bigint DEFAULT NULL,
        `updater_id` bigint DEFAULT NULL,
        `create_time` bigint DEFAULT NULL,
        `update_time` bigint DEFAULT NULL,
        PRIMARY KEY (`id`) USING BTREE,
        KEY `exam` (`exam_id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for scan_paper
-- ----------------------------
CREATE TABLE IF NOT EXISTS `scan_paper` (
        `id` bigint NOT NULL,
        `exam_id` bigint NOT NULL COMMENT '考试ID',
        `card_number` int NOT NULL COMMENT '卡格式编号',
        `number` int NOT NULL COMMENT '题卡编号（对应paper_index）',
        `page_count` int NOT NULL COMMENT '数量',
        `mismatch` bit(1) NOT NULL,
        `assigned` bit(1) NOT NULL,
        `question_filled` bit(1) NOT NULL,
        `checked` varchar(45) DEFAULT NULL COMMENT '已检查',
        `creator_id` bigint DEFAULT NULL,
        `updater_id` bigint DEFAULT NULL,
        `create_time` bigint DEFAULT NULL,
        `update_time` bigint DEFAULT NULL,
        `assign_confirmed` bit(1) DEFAULT NULL COMMENT '人工绑定确认',
        PRIMARY KEY (`id`) USING BTREE,
        KEY `mismatch` (`exam_id`,`mismatch`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for scan_paper_page
-- ----------------------------
CREATE TABLE IF NOT EXISTS `scan_paper_page` (
        `paper_id` bigint NOT NULL,
        `page_index` int NOT NULL,
        `absent` text,
        `breach` text,
        `question` text,
        `selective` text,
        `sheet_path` varchar(1000) NOT NULL,
        `recog_data` longtext,
        `creator_id` bigint DEFAULT NULL,
        `updater_id` bigint DEFAULT NULL,
        `create_time` bigint DEFAULT NULL,
        `update_time` bigint DEFAULT NULL,
        PRIMARY KEY (`paper_id`,`page_index`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for scan_refix_batch
-- ----------------------------
CREATE TABLE IF NOT EXISTS `scan_refix_batch` (
        `id` bigint NOT NULL,
        `exam_id` bigint NOT NULL,
        `card_number` int NOT NULL,
        `conditions` text NOT NULL,
        `update_slice` bit(1) NOT NULL,
        `total_count` int NOT NULL,
        `sucess_count` int NOT NULL,
        `finished` bit(1) NOT NULL,
        `creator_id` bigint DEFAULT NULL,
        `updater_id` bigint DEFAULT NULL,
        `create_time` bigint DEFAULT NULL,
        `update_time` bigint DEFAULT NULL,
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for scan_student_paper
-- ----------------------------
CREATE TABLE IF NOT EXISTS `scan_student_paper` (
        `id` bigint NOT NULL,
        `student_id` bigint NOT NULL,
        `paper_index` int NOT NULL,
        `paper_id` bigint NOT NULL,
        `create_time` bigint DEFAULT NULL,
        `update_time` bigint DEFAULT NULL,
        PRIMARY KEY (`id`) USING BTREE,
        UNIQUE KEY `paper_id` (`paper_id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for sys_config
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sys_config` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint DEFAULT NULL COMMENT '学校id',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `config_key` varchar(100) NOT NULL COMMENT '参数键名',
        `config_name` varchar(100) NOT NULL COMMENT '参数名称',
        `config_value` varchar(100) DEFAULT NULL COMMENT '参数键值',
        `remark` varchar(100) DEFAULT NULL COMMENT '备注',
        `enable` tinyint DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
        `sort` int DEFAULT '1' COMMENT '排序',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='参数表';

INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (1, NULL, NULL, 'sys.user.initPassword', '初始密码', '123456', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (2, NULL, NULL, 'sys.warning.days', '预警天数', '3', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (3, NULL, NULL, 'sys.code.enable', '是否启用短信验证码', 'true', 'true-启用，false-禁用', 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (4, NULL, NULL, 'sys.message.enable', '是否启用短信消息提示', 'true', 'true-启用，false-禁用', 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (5, NULL, NULL, 'sys.message.resendCount', '失败短信重试次数', '3', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (6, NULL, NULL, 'sys.sync.enable', '是否开启数据同步云阅卷', 'true', 'true-启用，false-禁用', 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (8, NULL, NULL, 'sys.pdf.size.list', '全局pdf格式清单', '[A3, A4, A5, B3, B4, B5, 8K]', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (9, NULL, NULL, 'admin.logo.url', '超级管理员logo', 'http://qmth-test.oss-cn-shenzhen.aliyuncs.com/frontend/wx_logo.png', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (10, NULL, NULL, 'attachment.type', '附件类型', '[.xlsx,.xls,.doc,.docx,.pdf,.jpg,.jpeg,.png,.html,.zip,.mp3,.wav,.dll,.exe,.ftl,.bpmn,.xml]', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (11, NULL, NULL, 'attachment.length', '附件名称长度', '100', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (12, NULL, NULL, 'attachment.size', '附件大小 ', '200', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (13, NULL, NULL, 'auto.create.pdf.reset.max.count', '自动重试生成pdf失败最大次数', '5', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (14, NULL, NULL, 'thread.pool.core.size', '自定义线程池大小', '500', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (15, NULL, NULL, 'custom.thread.pool.core.size', '是否自定义线程池大小', 'true', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (16, NULL, NULL, 'session.active', 'session会话时长', '4h', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (18, NULL, NULL, 'sms.normal.code', '万能短信验证码', 'qmth', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (19, NULL, NULL, 'code.expired.time', '短信验证码有效时间', '2', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (20, NULL, NULL, 'code.send.interval', '短信验证码发送间隔', '60', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (21, NULL, NULL, 'teachcloud.print.open.callback.pwd', '分布式印刷回调密码', 'qmth87863577-offline', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (24, NULL, NULL, 'sms.sign.name', '短信签名', '知学知考', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (25, NULL, NULL, 'sms.tpl.code', '登录短信验证码', 'SMS_147416565', NULL, 1, 2, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (26, NULL, NULL, 'sms.audit.pass.code', '命题任务审核通过', 'SMS_216425141', NULL, 1, 3, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (27, NULL, NULL, 'sms.audit.not.pass.code', '命题任务审核不通过', 'SMS_216275156', NULL, 1, 4, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (28, NULL, NULL, 'sms.exam.task.create.code', '命题任务待办生成通知', 'SMS_217436292', NULL, 1, 5, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (29, NULL, NULL, 'sms.exam.task.will.expire.code', '命题任务待办到期预警通知', 'SMS_217436295', NULL, 1, 6, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (30, NULL, NULL, 'sms.exam.task.overdue.code', '命题任务待办逾期通知', 'SMS_217426313', NULL, 1, 7, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (31, NULL, NULL, 'sms.allocation.will.expire.code', '命题分配待办到期预警通知', 'SMS_217406305', NULL, 1, 8, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (32, NULL, NULL, 'sms.allocation.overdue.code', '命题分配待办逾期通知', 'SMS_217406308', NULL, 1, 9, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (33, NULL, NULL, 'sms.audit.create.code', '审核待办生成通知', 'SMS_237216000', NULL, 1, 10, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (34, NULL, NULL, 'sms.audit.review.code', '审核待办修改申请通知', 'SMS_237216002', NULL, 1, 11, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (35, NULL, NULL, 'sms.audit.will.expire.code', '审核待办到期预警通知', 'SMS_217436302', NULL, 1, 12, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (36, NULL, NULL, 'sms.audit.overdue.code', '审核待办逾期通知', 'SMS_217416271', NULL, 1, 13, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (37, NULL, NULL, 'sms.audit.reject.code', '审核驳回通知', 'SMS_237206063', NULL, 1, 14, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (38, NULL, NULL, 'sms.upload.structure.code', '试卷结构推送通知', 'SMS_237201068', NULL, 1, 15, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (39, NULL, NULL, 'sms.audit.copy.user.code', '用户', 'SMS_237206065', NULL, 1, 16, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (40, NULL, NULL, 'sys.txt.charset', 'txt文件编码', 'UTF-8', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (41, NULL, NULL, 'package.barcode.position', '签到表条码坐标', '[0.7,0,0.3,0.1]', NULL, 1, 17, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (401752651068346368, 135, NULL, 'open.flow.message.push', '是否开启消息推送', 'true', NULL, 1, 5, 1, 1686726096298, NULL, 1686726096298);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (401752651068346369, 135, NULL, 'flow.message.type', '消息类型', 'OFF_STANDARD', NULL, 1, 6, 1, 1686726096303, NULL, 1686726096303);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (401752651068346370, 135, NULL, 'mq.host.url', '消息中心地址', 'http://exam.gdpu.edu.cn:7400?code=gdpu', NULL, 1, 7, 1, 1686726096309, NULL, 1686726096309);

-- ----------------------------
-- Table structure for sys_org
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sys_org` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `type` varchar(50) DEFAULT NULL COMMENT '类型，SCHOOL：学校，COLLEGE：学院，FACULTY：院系，TEACHING_ROOM：教研室，PRINTING_HOUSE：印刷厂',
        `code` varchar(20) DEFAULT NULL COMMENT '机构代码',
        `name` varchar(100) NOT NULL COMMENT '机构名称',
        `parent_id` bigint DEFAULT NULL COMMENT '上级机构id',
        `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '修改人',
        `update_time` bigint DEFAULT NULL COMMENT '修改时间',
        `historic_name` varchar(100) DEFAULT NULL COMMENT '历史机构名称',
        `third_relate_id` int DEFAULT NULL COMMENT '云阅卷机构ID',
        PRIMARY KEY (`id`) USING BTREE,
        UNIQUE KEY `org_schoolId_name_idx` (`school_id`,`name`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='学校组织架构';

-- ----------------------------
-- Table structure for sys_privilege
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sys_privilege` (
        `id` bigint NOT NULL COMMENT '主键',
        `name` varchar(100) NOT NULL COMMENT '菜单名称',
        `url` varchar(100) NOT NULL COMMENT 'URL地址',
        `type` varchar(50) NOT NULL COMMENT '类型，MENU：菜单，BUTTON：按钮，LINK：链接，URL：接口地址，LIST：列表，CONDITION：查询条件，TAB：选项卡',
        `parent_id` bigint DEFAULT NULL COMMENT '上级菜单',
        `sequence` int NOT NULL COMMENT '序号',
        `property` varchar(30) DEFAULT NULL COMMENT '属性，NO_AUTH：无需鉴权，AUTH：鉴权，SYS：系统公用',
        `related` text COMMENT '关联属性，type为URL且property为AUTH才有',
        `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
        `default_auth` tinyint DEFAULT '1' COMMENT '默认权限，0：否，1：是',
        `front_display` tinyint(1) DEFAULT '1' COMMENT '是否前端展示',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='菜单权限表';

INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1, '基础配置', 'base', 'MENU', NULL, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2, '用户管理', 'user', 'MENU', 1, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3, '配置管理', 'rule', 'MENU', 1, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (4, '组织架构', 'OrganizationManage', 'MENU', 2, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (5, '用户管理', 'UserManage', 'MENU', 2, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (6, '角色管理', 'RoleManage', 'MENU', 2, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (7, '权限管理', 'MenuManage', 'MENU', 129, 6, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (8, '通用规则', 'CommonRule', 'MENU', 3, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (9, '题卡规则管理', 'CardRuleManage', 'MENU', 3, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (11, '印品管理', 'PrintTemplateManage', 'MENU', 3, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (13, '课程管理', 'CourseManage', 'MENU', 356, 7, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (14, '考试中心', 'exam', 'MENU', NULL, 5, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (38, '印刷管理', 'print', 'MENU', 14, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (39, '命题任务管理', 'ExamTaskManage', 'MENU', 652, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (40, '入库申请', 'TaskApplyManage', 'MENU', 652, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (41, '入库审核', 'TaskReviewManage', 'MENU', 652, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (42, '卷库查询', 'TaskPaperManage', 'MENU', 652, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (43, '印刷计划管理', 'PrintPlanManage', 'MENU', 38, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (44, '考务数据导入', 'BusinessDataExport', 'MENU', 38, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (45, '考务明细查询', 'BusinessDataDetail', 'MENU', 38, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (47, '印刷任务管理', 'PrintTaskManage', 'MENU', 38, 5, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (48, '查询', '/api/admin/exam/print/list', 'URL', 43, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (49, '按类型查询模板', '/api/admin/exam/print/get_template', 'URL', 43, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (50, '新增/修改', '/api/admin/exam/print/save', 'URL', 43, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (51, '删除', '/api/admin/exam/print/remove', 'URL', 43, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (52, '查询', '/api/admin/exam/print/data_list', 'URL', 44, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (53, '汇总数据查询', '/api/admin/exam/print/data_total', 'URL', 44, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (54, '导入', '/api/admin/exam/print/data_import', 'URL', 44, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (55, '查看明细', '/api/admin/exam/print/data_detail_list', 'URL', 44, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (56, '查询', '/api/admin/exam/print/relate_list', 'URL', 43, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (57, '关联/更换试卷', '/api/admin/exam/print/relate_update', 'URL', 43, 2, 'AUTH', '58', 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (58, '查询所有试卷编号/关联卷型', '/api/admin/exam/print/relate_get_paper_numbers', 'URL', 43, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (59, '查询', '/api/admin/exam/print/task_list', 'URL', 47, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (60, '提交印刷', '/api/admin/exam/print/task_submit', 'URL', 47, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (61, '撤回提交', '/api/admin/exam/print/task_cancel', 'URL', 47, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (62, '汇总数据查询', '/api/admin/exam/print/task_total_data', 'URL', 47, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (63, '查询', '/api/admin/exam/task/list', 'URL', 39, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (64, '更改/指派命题老师', '/api/admin/exam/task/assign_user', 'URL', 39, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (65, '启用/禁用', '/api/admin/exam/task/enable', 'URL', 39, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (66, '新建', '/api/admin/exam/task/save', 'URL', 39, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (67, '批量新建-文件导入', '/api/admin/exam/task/import', 'URL', 39, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (68, '批量新建', '/api/admin/exam/task/save_batch', 'URL', 39, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (69, '审核历史', '/api/admin/exam/task/review_list', 'URL', 39, 7, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (70, '查询', '/api/admin/exam/task/apply_list', 'URL', 40, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (71, '提交/暂存', '/api/admin/exam/task/apply_save', 'URL', 40, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (72, '撤回/重新申请', '/api/admin/exam/task/apply_status', 'URL', 40, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (73, '查询未审核', '/api/admin/exam/task/review_list_unaudited', 'URL', 41, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (74, '审核', '/api/admin/exam/task/review_save', 'URL', 41, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (75, '批量审核', '/api/admin/exam/task/review_save_batch', 'URL', 41, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (76, '查询', '/api/admin/exam/task/paper_list', 'URL', 42, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (77, '查询', '/api/admin/sys/user/list', 'URL', 5, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (78, '新增/修改', '/api/admin/sys/user/save', 'URL', 5, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (79, '启用/禁用', '/api/admin/sys/user/enable', 'URL', 5, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (80, '重置密码', '/api/admin/sys/user/reset_password', 'URL', 5, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (81, '导入', '/api/admin/sys/user/import', 'URL', 5, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (82, '绑定角色', '/api/admin/sys/user/bind_roles', 'URL', 5, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (83, '查询', '/api/admin/sys/role/list', 'URL', 6, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (84, '删除', '/api/admin/sys/role/remove', 'URL', 6, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (85, '新增/修改', '/api/admin/sys/role/save', 'URL', 6, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (86, '启用/禁用', '/api/admin/sys/role/enable', 'URL', 6, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (87, '用户已绑定角色列表', '/api/admin/sys/role/get_user_roles', 'URL', 5, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (88, '查询', '/api/admin/sys/privilege/list', 'URL', 7, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (89, '新增/修改', '/api/admin/sys/privilege/save', 'URL', 7, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (90, '删除', '/api/admin/sys/privilege/remove', 'URL', 7, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (91, '角色已绑定权限列表', '/api/admin/sys/privilege/get_role_privileges', 'URL', 6, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (92, '组织机构-查询', '/api/admin/sys/org/list', 'URL', 4, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (93, '组织机构-新增/修改', '/api/admin/sys/org/save', 'URL', 4, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (94, '组织机构-删除', '/api/admin/sys/org/remove', 'URL', 4, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (95, '组织机构-启用/禁用', '/api/admin/sys/org/enable', 'URL', 4, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (96, '查询', '/api/admin/basic/course/list', 'URL', 13, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (97, '新增/修改', '/api/admin/basic/course/save', 'URL', 13, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (98, '删除', '/api/admin/basic/course/remove', 'URL', 13, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (99, '查询', '/api/admin/basic/exam_rule/list', 'URL', 8, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (100, '新增/修改', '/api/admin/basic/exam_rule/save', 'URL', 8, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (101, '查询', '/api/admin/basic/card_rule/list', 'URL', 9, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (102, '查询单个', '/api/admin/basic/card_rule/get_one', 'URL', 9, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (103, '新增/修改', '/api/admin/basic/card_rule/save', 'URL', 9, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (104, '删除', '/api/admin/basic/card_rule/delete', 'URL', 9, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (105, '查询-汇总数据', '/api/admin/exam/print/relate_total', 'URL', 43, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (112, '系统设置', 'data', 'MENU', 1, 5, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (113, '任务管理', 'DataTaskManage', 'MENU', 112, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (114, '查询', '/api/admin/data/task/query', 'URL', 113, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (117, '命题待办接口', '/api/admin/work/exam/task/ready', 'URL', 16, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (118, '命题待办计数接口', '/api/admin/work/exam/task/ready_count', 'URL', 16, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (119, '命题分配待办接口', '/api/admin/work/exam/task/new', 'URL', 16, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (120, '命题分配待办计数接口', '/api/admin/work/exam/task/new_count', 'URL', 16, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (121, '审核待办接口', '/api/admin/work/exam/task/submit', 'URL', 16, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (122, '审核待办计数接口', '/api/admin/work/exam/task/submit_count', 'URL', 16, 6, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (123, '查询', '/api/admin/basic/template/list', 'URL', 11, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (124, '新增/修改', '/api/admin/basic/template/save', 'URL', 11, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (125, '启用/禁用', '/api/admin/basic/template/enable', 'URL', 11, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (129, '公共接口', 'common', 'MENU', NULL, 12, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (130, '用户登录相关', 'login', 'MENU', 129, 1, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (131, '用户登录', '/api/admin/common/login', 'URL', 130, 1, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (132, '查询用户权限', '/api/admin/common/get_menu', 'URL', 130, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (133, '发送验证码', '/api/admin/common/get_verify_code', 'URL', 130, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (134, '修改密码', '/api/admin/sys/user/update_password', 'URL', 130, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (135, '用户登出', '/api/admin/common/logout', 'URL', 130, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (136, '模糊查询', 'query', 'MENU', 129, 2, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (137, '模糊查询-印刷计划', '/api/admin/exam/print/query', 'URL', 136, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (138, '题卡规则', '/api/admin/basic/card_rule/query', 'URL', 136, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (139, '模糊查询-课程', '/api/admin/basic/course/query', 'URL', 136, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (140, '模糊查询-试卷编号', '/api/admin/exam/task/paper_number_query', 'URL', 136, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (141, '模糊查询-命题老师', '/api/admin/exam/task/user_query', 'URL', 136, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (142, '模糊查询-考点', '/api/admin/exam/print/data_place_query', 'URL', 136, 6, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (143, '模糊查询-考场', '/api/admin/exam/print/data_room_query', 'URL', 136, 7, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (144, '文件相关', 'file', 'MENU', 129, 3, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (145, '文件上传', '/api/admin/common/file/upload', 'URL', 144, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (146, '文件查询', '/api/admin/common/file/get_one', 'URL', 144, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (147, '文件下载', '/api/admin/common/file/download', 'URL', 144, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (148, '文件预览', '/api/admin/common/file/preview', 'URL', 144, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (149, '系统相关', 'sys', 'MENU', 129, 4, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (150, '查询枚举类型', '/api/admin/common/get_enums', 'URL', 149, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (151, '查询学校列表', '/api/admin/common/school/list', 'URL', 149, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (152, '查询系统参数', '/api/admin/common/sys_config/get_one', 'URL', 149, 3, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (153, '根据学校代码查询学校信息', '/api/admin/common/school/query_by_school_code', 'URL', 149, 4, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (154, '其他', 'other', 'MENU', 129, 5, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (155, '用户导入（异步）', '/api/admin/sys/user/async/import', 'URL', 154, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (156, '用户导出', '/api/admin/sys/user/export', 'URL', 154, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (157, '模板下载', '/api/admin/exam/print/template_download', 'URL', 44, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (158, '结果导出', '/api/admin/exam/print/data_export', 'URL', 44, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (159, '查询', '/api/admin/exam/print/data_detail', 'URL', 45, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (160, '重新提交印刷', '/api/admin/exam/print/task_resubmit', 'URL', 47, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (161, '查看PDF', '/api/admin/exam/print/task_view_pdf', 'URL', 47, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (162, '批量下载PDF', '/api/admin/exam/print/task_download_pdf', 'URL', 47, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (163, '导出审核样本', '/api/admin/exam/task/review_export', 'URL', 41, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (164, '下载试卷', '/api/admin/exam/task/paper_download', 'URL', 42, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (165, '试卷启用/禁用', '/api/admin/exam/task/paper_enable', 'URL', 42, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (166, '查询已审核', '/api/admin/exam/task/review_list_audited', 'URL', 41, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (167, '查询可选择题卡', '/api/admin/exam/card/select_card_list', 'URL', 40, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (168, '查询申请详情', '/api/admin/exam/task/apply_get_one', 'URL', 40, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (169, '根据课程查询命题老师', '/api/admin/sys/user/user_list', 'URL', 39, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (170, '客服制卡', 'customer', 'MENU', NULL, 9, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (171, '客服制卡', 'customerManage', 'MENU', 170, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (172, '题卡审核', 'CustomerCard', 'MENU', 171, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (173, '客服制卡-查询', '/api/admin/exam/card/cust_list', 'URL', 172, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (174, '导入模板下载', '/api/admin/common/download_import_template', 'URL', 149, 15, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (175, '题卡管理', 'card', 'MENU', NULL, 10, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (176, '查询题卡规则', '/api/admin/basic/card_rule/get_one', 'URL', 175, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (177, '查询题卡详情', '/api/admin/exam/card/get_one', 'URL', 175, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (178, '保存/提交', '/api/admin/exam/card/save', 'URL', 175, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (179, '查询考务规则', '/api/admin/basic/exam_rule/list', 'URL', 149, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (180, '查询可设置权限列表', '/api/admin/sys/privilege/list_auth', 'URL', 6, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (181, '超管中心', 'admin', 'MENU', 129, 7, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (182, '查询客服', '/api/admin/sys/user/list_customer', 'URL', 181, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (183, '新增客服', '/api/admin/sys/user/save_customer', 'URL', 181, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (184, '查询用户可绑定角色', '/api/admin/sys/role/list_to_user', 'URL', 5, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (185, '客服制卡-批量下载文件', '/api/admin/exam/card/download_files', 'URL', 172, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (186, '查看印品', '/api/admin/exam/print/template_view', 'URL', 47, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (187, '客户端登录', '/api/admin/client/user/login', 'URL', NULL, 1, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (188, '学生明细', '/api/admin/exam/print/get_student_detail', 'URL', 45, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (189, '修改', '/api/admin/exam/task/paper_update', 'URL', 42, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (190, '重新生成pdf', '/api/admin/data/task/reset_create_pdf', 'URL', 113, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (191, '批量下载试卷PDF、题卡', '/api/admin/exam/task/paper_card_download_pdf', 'URL', 42, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (192, '预览', '/api/admin/basic/template/read_content', 'URL', 11, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (195, '获取服务器时间', '/api/admin/common/get_system_time', 'URL', 129, 2, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (198, '数据同步', '/api/admin/exam/print/sync_data_cloud', 'URL', 47, 9, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (199, '打印端', 'client', 'MENU', NULL, 11, NULL, '200,201,202,203,204,205,206,207,208,209,210,211,212,213,214,215,216,973,974,975', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (200, '试卷打样-查询列表', '/api/admin/client/paper_try/list', 'URL', 199, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (201, '试卷打样-查看/试印/重印', '/api/admin/client/paper_try/print', 'URL', 199, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (202, '试卷打样-批量试印', '/api/admin/client/paper_try/print_batch', 'URL', 199, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (203, '试卷打样-标记合格状态', '/api/admin/client/paper_try/tag_pass', 'URL', 199, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (204, '查询列表', '/api/admin/client/print/task_list', 'URL', 199, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (205, '导出', '/api/admin/client/print/task_list_export', 'URL', 199, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (206, '汇总数据查询', '/api/admin/client/print/task_total_data', 'URL', 199, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (207, '查看', '/api/admin/client/print/preview', 'URL', 199, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (208, '印刷/缓存数据', '/api/admin/client/print/get_print_data', 'URL', 199, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (209, '批量缓存', '/api/admin/client/print/get_print_data_batch', 'URL', 199, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (210, '缓存后更新状态', '/api/admin/client/print/update_download', 'URL', 199, 12, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (211, '校验', '/api/admin/client/print/validate_data', 'URL', 199, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (212, '更新打印进度', '/api/admin/client/print/update_progress', 'URL', 199, 14, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (213, '重打-查询考生列表', '/api/admin/client/print/list_student', 'URL', 199, 15, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (214, '重打-内容查询', '/api/admin/client/print/get_reprint_data', 'URL', 199, 16, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (215, '统计查询-查询列表', '/api/admin/client/print/statistics_list', 'URL', 199, 16, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (216, '统计查询-汇总数据查询', '/api/admin/client/print/statistics_total_data', 'URL', 199, 16, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (217, '查询', 'Select', 'BUTTON', 5, 1, 'AUTH', '77', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (218, '新增', 'Add', 'BUTTON', 5, 2, 'AUTH', '78', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (219, '编辑', 'Edit', 'LINK', 5, 1, 'AUTH', '78', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (220, '启用/禁用', 'Enable', 'LINK', 5, 2, 'AUTH', '79', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (221, '列表', 'List', 'LIST', 5, 1, 'AUTH', '77', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (222, '查询条件', 'Condition', 'CONDITION', 5, 1, 'AUTH', '184,87', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (223, '重置密码', 'Reset', 'LINK', 5, 3, 'AUTH', '80', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (224, '新增', 'Add', 'BUTTON', 4, 1, 'AUTH', '93', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (225, '子部门', 'Add', 'LINK', 4, 1, 'AUTH', '93', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (226, '编辑', 'Edit', 'LINK', 4, 2, 'AUTH', '93', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (227, '删除', 'Delete', 'LINK', 4, 3, 'AUTH', '94', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (228, '树形列表', 'List', 'LIST', 4, 1, 'AUTH', '92', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (229, '查询', 'Select', 'BUTTON', 6, 1, 'AUTH', '83,88', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (230, '新增', 'Add', 'BUTTON', 6, 2, 'AUTH', '85,89', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (231, '编辑', 'Edit', 'LINK', 6, 1, 'AUTH', '83,91', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (232, '删除', 'Delete', 'LINK', 6, 2, 'AUTH', '84,90', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (233, '列表', 'List', 'LIST', 6, 1, 'AUTH', '83', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (234, '查询条件', 'Condition', 'CONDITION', 4, 1, 'AUTH', '92', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (235, '查询条件', 'Condition', 'CONDITION', 6, 1, 'AUTH', '83', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (236, '考务规则配置', 'Add', 'BUTTON', 8, 1, 'AUTH', '99, 100', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (237, '查询', 'Select', 'BUTTON', 9, 1, 'AUTH', '101', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (238, '新增', 'Add', 'BUTTON', 9, 2, 'AUTH', '103', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (239, '查询条件', 'Condition', 'CONDITION', 9, 1, 'AUTH', '101', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (240, '列表', 'List', 'LIST', 9, 1, 'AUTH', '101', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (243, '导入', 'Import', 'BUTTON', 5, 3, 'AUTH', '81', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (249, '查询', 'Select', 'BUTTON', 11, 1, 'AUTH', '123', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (250, '新增', 'Add', 'BUTTON', 11, 2, 'AUTH', '124', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (251, '预览', 'Preview', 'LINK', 11, 1, 'AUTH', '192', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (252, '编辑', 'Edit', 'LINK', 11, 2, 'AUTH', '124,763', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (253, '启用/禁用', 'Enable', 'LINK', 11, 3, 'AUTH', '125', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (254, '查询条件', 'Condition', 'CONDITION', 11, 1, 'AUTH', '123', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (255, '列表', 'List', 'LIST', 11, 1, 'AUTH', '123', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (263, '查询', 'Select', 'BUTTON', 13, 1, 'AUTH', '96,462', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (268, '编辑', 'Edit', 'LINK', 13, 1, 'AUTH', '97', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (269, '删除', 'Delete', 'LINK', 13, 2, 'AUTH', '368', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (270, '查询条件', 'Condition', 'CONDITION', 13, 2, 'AUTH', '96', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (271, '列表', 'List', 'LIST', 13, 1, 'AUTH', '96', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (272, '查询', 'Select', 'BUTTON', 39, 1, 'AUTH', '63', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (273, '批量新增', 'BatchAdd', 'BUTTON', 39, 2, 'AUTH', '67', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (274, '新增', 'Add', 'BUTTON', 39, 3, 'AUTH', '66,169,441,442,445,482', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (275, '预览', 'Preview', 'LINK', 39, 1, 'AUTH', '68', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (277, '启用/禁用', 'Cancel', 'LINK', 39, 3, 'AUTH', '65', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (278, '查询条件', 'Condition', 'CONDITION', 39, 1, 'AUTH', '63', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (279, '列表', 'List', 'LIST', 39, 1, 'AUTH', '63', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (280, '查询', 'Select', 'BUTTON', 40, 1, 'AUTH', '70', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (281, '预览', 'Preview', 'LINK', 40, 1, 'AUTH', '168,477,568,417,420,421,441,442,640,641', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (282, '撤销申请', 'End', 'LINK', 40, 2, 'AUTH', '72', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (283, '立即申请', 'Edit', 'LINK', 40, 3, 'AUTH', '71,167,424,477,417,420,421,441,442,640,641', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (284, '查询条件', 'Condition', 'CONDITION', 40, 1, 'AUTH', '70', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (285, '列表', 'List', 'LIST', 40, 1, 'AUTH', '70', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (288, '查询（未审核）', 'NotReviewSelect', 'BUTTON', 41, 1, 'AUTH', '73', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (289, '导出审核样本（未审核）', 'NotReviewExport', 'BUTTON', 41, 2, 'AUTH', '163', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (290, '批量不通过（未审核）', 'NotReviewBatchNotPass', 'BUTTON', 41, 3, 'AUTH', '75', 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (291, '批量通过（未审核）', 'NotReviewBatchPass', 'BUTTON', 41, 4, 'AUTH', '75', 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (292, '审核（未审核）', 'NotReviewEdit', 'LINK', 41, 1, 'AUTH', '74,417,420,421,441,442,477,417,640,641', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (293, '查询条件（未审核）', 'NotReviewCondition', 'CONDITION', 41, 1, 'AUTH', '73', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (294, '列表（未审核）', 'NotReviewList', 'LIST', 41, 1, 'AUTH', '73', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (295, '查询（已审核）', 'ReviewSelect', 'BUTTON', 41, 1, 'AUTH', '166', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (296, '查看详情（已审核）', 'ReviewPreview', 'LINK', 41, 1, 'AUTH', '74,477,417,420,421,441,442,640,641', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (297, '查询条件（已审核）', 'ReviewCondition', 'CONDITION', 41, 1, 'AUTH', '166', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (298, '列表（已审核）', 'ReviewList', 'LIST', 41, 1, 'AUTH', '166', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (299, '查询', 'Select', 'BUTTON', 42, 1, 'AUTH', '76', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (300, '批量下载试卷题卡', 'BatchDownload', 'BUTTON', 42, 2, 'AUTH', '191', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (301, '预览', 'Preview', 'LINK', 42, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (302, '启用/禁用', 'Enable', 'LINK', 42, 2, 'AUTH', '165', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (303, '下载', 'Download', 'LINK', 42, 3, 'AUTH', '164', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (304, '查询条件', 'Condition', 'CONDITION', 42, 1, 'AUTH', '76', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (305, '列表', 'List', 'LIST', 42, 1, 'AUTH', '76', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (308, '编辑', 'Edit', 'LINK', 42, 4, 'AUTH', '189', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (309, '查询', 'Select', 'BUTTON', 43, 1, 'AUTH', '48,49', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (310, '新增', 'Add', 'BUTTON', 43, 2, 'AUTH', '50,49,621', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (311, '查看', 'Preview', 'LINK', 43, 1, 'AUTH', '56,57,58,105,825,826', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (312, '编辑', 'Edit', 'LINK', 43, 2, 'AUTH', '50', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (313, '删除', 'Delete', 'LINK', 43, 3, 'AUTH', '51', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (314, '查询条件', 'Condition', 'CONDITION', 43, 1, 'AUTH', '48', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (315, '列表', 'List', 'LIST', 43, 1, 'AUTH', '48,49', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (316, '查询', 'Select', 'BUTTON', 44, 1, 'AUTH', '52,53', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (317, '模版下载', 'TempleteDownload', 'BUTTON', 44, 2, 'AUTH', '157', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (318, '导出', 'Export', 'BUTTON', 44, 3, 'AUTH', '158', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (319, '导入', 'Import', 'BUTTON', 44, 4, 'AUTH', '54', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (320, '查看详情', 'Preview', 'LINK', 44, 1, 'AUTH', '55,327,328,329', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (321, '查询条件', 'Condition', 'CONDITION', 44, 1, 'AUTH', '52', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (322, '列表', 'List', 'LIST', 44, 1, 'AUTH', '52,53', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (323, '查询', 'Select', 'BUTTON', 45, 1, 'AUTH', '159', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (324, '预览', 'Preview', 'LINK', 45, 1, 'AUTH', '188', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (325, '查询条件', 'Condition', 'CONDITION', 45, 1, 'AUTH', '159', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (326, '列表', 'List', 'LIST', 45, 1, 'AUTH', '159', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (327, '新增考生', '/api/admin/exam/print/add_exam_student', 'URL', 44, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (328, '查询考场下试卷列表', '/api/admin/exam/print/list_exam_detail_course', 'URL', 44, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (329, '删除考生', '/api/admin/exam/print/delete_exam_student', 'URL', 44, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (330, '重新生成pdf', 'CreatePdf', 'LINK', 47, 6, 'AUTH', '190', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (331, '查询', 'Select', 'BUTTON', 47, 1, 'AUTH', '59,62', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (332, '查看pdf', 'Download', 'LINK', 47, 1, 'AUTH', '161,162', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (333, '查询条件', 'Condition', 'CONDITION', 47, 1, 'AUTH', '59', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (334, '列表', 'List', 'LIST', 47, 1, 'AUTH', '59', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (335, '撤回提交', 'End', 'LINK', 47, 2, 'AUTH', '61', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (336, '批量下载PDF', 'BatchDownload', 'BUTTON', 47, 2, 'AUTH', '336', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (337, '查询', 'Select', 'BUTTON', 113, 1, 'AUTH', '114', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (338, '导出日志', 'Export', 'LINK', 113, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (339, '下载文件', 'Download', 'LINK', 113, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (340, '查询条件', 'Condition', 'CONDITION', 113, 1, 'AUTH', '114', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (341, '列表', 'List', 'LIST', 113, 1, 'AUTH', '114', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (342, '预览', 'Preview', 'LINK', 9, 1, 'AUTH', '102', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (343, '编辑', 'Edit', 'LINK', 9, 2, 'AUTH', '103', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (345, '删除', 'Delete', 'LINK', 9, 3, 'AUTH', '104', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (346, '题卡审核-未审核查询', 'NotReviewSelect', 'BUTTON', 172, 1, 'AUTH', '173', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (347, '题卡审核-未审核批量下载试卷文件', 'NotReviewBatchDownload', 'BUTTON', 172, 2, 'AUTH', '185', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (348, '题卡审核-未审核查询条件', 'NotReviewCondition', 'CONDITION', 172, 1, 'AUTH', '173', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (349, '题卡审核-未审核列表', 'NotReviewList', 'LIST', 172, 1, 'AUTH', '173', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (350, '题卡审核-未审核预览', 'NotReviewPreview', 'LINK', 172, 1, 'AUTH', '173', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (351, '题卡审核-未审核设计题卡', 'NotReviewDesign', 'LINK', 172, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (352, '题卡审核-已审核查询', 'ReviewSelect', 'BUTTON', 172, 1, 'AUTH', '173', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (353, '题卡审核-已审核查询条件', 'ReviewCondition', 'CONDITION', 172, 1, 'AUTH', '173', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (354, '题卡审核-已审核列表', 'ReviewList', 'LIST', 172, 1, 'AUTH', '173', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (355, '题卡审核-已审核预览', 'ReviewPreview', 'LINK', 172, 1, 'AUTH', '173', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (356, '字典管理', 'dictionary', 'MENU', 1, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (357, '学生学院管理', 'CollegeManage', 'MENU', 356, 3, NULL, NULL, 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (358, '学生管理', 'StudentManage', 'MENU', 356, 6, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (359, '查询', '/api/admin/basic/college/query', 'URL', 357, 1, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (360, '新增/编辑', '/api/admin/basic/college/save', 'URL', 357, 2, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (361, '删除（逻辑）', '/api/admin/basic/college/delete', 'URL', 357, 3, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (362, '查询', '/api/admin/basic/student/query', 'URL', 358, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (363, '增/编辑', '/api/admin/basic/student/save', 'URL', 358, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (364, '批量删除（逻辑）', '/api/admin/basic/student/delete_batch', 'URL', 358, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (365, '批量导入（异步）', '/api/admin/basic/student/data_import', 'URL', 358, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (366, '查询', '/api/admin/basic/course/query', 'URL', 13, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (367, '新增/编辑', '/api/admin/basic/course/save', 'URL', 13, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (368, '批量删除', '/api/admin/basic/course/delete_batch', 'URL', 13, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (369, '批量导入', '/api/admin/basic/course/data_import', 'URL', 13, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (370, '获取系统编码', '/api/admin/common/get_code', 'URL', 149, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (371, '根据类型查询机构', '/api/admin/sys/org/find_by_type', 'URL', 4, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (372, '查询', 'Select', 'BUTTON', 357, 1, 'AUTH', '359', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (373, '列表', 'List', 'LIST', 357, 1, 'AUTH', '359', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (374, '新增', 'Add', 'BUTTON', 357, 1, 'AUTH', '360,370', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (375, '编辑', 'Edit', 'LINK', 357, 1, 'AUTH', '360', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (376, '删除', 'Delete', 'LINK', 357, 1, 'AUTH', '361', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (377, '查询', 'Select', 'BUTTON', 358, 1, 'AUTH', '362,462', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (378, '列表', 'List', 'LIST', 358, 1, 'AUTH', '362', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (379, '新增', 'Add', 'BUTTON', 358, 1, 'AUTH', '359,363,462', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (380, '编辑', 'Edit', 'LINK', 358, 1, 'AUTH', '359,363', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (381, '删除', 'Delete', 'LINK', 358, 1, 'AUTH', '364', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (382, '导入', 'Import', 'BUTTON', 358, 2, 'AUTH', '365', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (383, '启用/禁用', 'Enable', 'LINK', 9, 4, 'AUTH', '384', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (384, '启用/禁用', '/api/admin/basic/card_rule/enable', 'URL', 9, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (385, '新增', 'Add', 'BUTTON', 13, 1, 'AUTH', '367,462', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (388, '导入', 'Import', 'BUTTON', 13, 2, 'AUTH', '369', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (389, '机构列表', 'List', 'LIST', 13, 1, 'AUTH', NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (390, '流程管理', 'flow', 'MENU', 1, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (391, '查询条件', 'Condition', 'CONDITION', 357, 1, 'AUTH', '359', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (400, '查询条件', 'Condition', 'CONDITION', 358, 1, 'AUTH', '377', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (402, '流程管理', 'FlowManage', 'MENU', 390, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (403, '流程审批记录管理', 'ApproveRecordManage', 'MENU', 390, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (404, '查询', 'Select', 'BUTTON', 403, 1, 'AUTH', '418', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (405, '查询条件', 'Condition', 'CONDITION', 403, 1, 'AUTH', '418', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (406, '列表', 'List', 'LIST', 403, 1, 'AUTH', '418', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (407, '查询', 'Select', 'BUTTON', 402, 1, 'AUTH', '416', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (408, '新建流程', 'Register', 'BUTTON', 402, 2, 'AUTH', '414', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (409, '删除', 'Delete', 'LINK', 402, 2, 'AUTH', '415', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (410, '查询条件', 'Condition', 'CONDITION', 402, 1, 'AUTH', '416', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (411, '列表', 'List', 'LIST', 402, 1, 'AUTH', '416', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (413, '重新生成pdf', 'CreatePdf', 'LINK', 113, 3, 'AUTH', '190', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (414, '保存和发布流程', '/api/admin/custom/flow/save', 'URL', 402, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (415, '流程逻辑删除', '/api/admin/custom/flow/enable', 'URL', 402, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (416, '流程列表', '/api/admin/custom/flow/list', 'URL', 402, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (417, '审批流程', '/api/admin/custom/flow/task/approve', 'URL', 402, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (418, '流程审批记录列表', '/api/admin/custom/flow/approve/list', 'URL', 402, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (419, '流程审批记录逻辑删除', '/api/admin/custom/flow/approve/enable', 'URL', 402, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (420, '流程终止', '/api/admin/custom/flow/end', 'URL', 402, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (421, '获取所有流程节点', '/api/admin/custom/flow/task/all', 'URL', 402, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (422, '删除', 'Delete', 'LINK', 403, 1, 'AUTH', '419', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (427, '发布印刷任务-印刷室列表', '/api/admin/exam/task/list_task_print_house', 'URL', 42, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (437, '查询模板', 'List', 'LIST', 43, 1, 'AUTH', '49', 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (439, '题卡标题查询', '/api/admin/exam/task/find_card_title', 'URL', 40, 7, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (440, '题卡标题查询', 'List', 'LIST', 40, 1, 'AUTH', '439', 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (441, '根据流程类型获取流程节点', '/api/admin/custom/flow/get_flow_info_by_type', 'URL', 402, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (442, '流程节点转他人审批', '/api/admin/custom/flow/task/approver/exchange', 'URL', 402, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (446, '做废/恢复', '/api/admin/exam/print/task_normal', 'URL', 47, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (447, '作废/恢复', 'Normal', 'LINK', 47, 3, 'AUTH', '446', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (448, '指派/更改', 'Assign', 'LINK', 39, 4, 'AUTH', '64', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (450, '班级管理', 'ClazzManage', 'MENU', 356, 5, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (451, '查询', '/api/admin/basic/clazz/query', 'URL', 450, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (452, '新增/编辑', '/api/admin/basic/clazz/save', 'URL', 450, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (453, '批量删除（逻辑）', '/api/admin/basic/clazz/delete_batch', 'URL', 450, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (455, '查询', 'Select', 'BUTTON', 450, 1, 'AUTH', '451', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (456, '列表', 'List', 'LIST', 450, 1, 'AUTH', '451', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (457, '新增', 'Add', 'BUTTON', 450, 1, 'AUTH', '452', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (458, '编辑', 'Edit', 'LINK', 450, 1, 'AUTH', '452', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (459, '删除', 'Delete', 'LINK', 450, 1, 'AUTH', '453', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (461, '查询条件', 'Condition', 'CONDITION', 450, 1, 'AUTH', '451', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (462, '班级数据源查询', '/api/admin/basic/clazz/datasource', 'URL', 450, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (463, '新建任务', 'ExamTaskApplyManage', 'BUTTON', 40, 8, 'AUTH', '49,167,174,371,464,465,427,642,417,420,421,441,442,640,641,642', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (464, '新建命题申请', '/api/admin/exam/task/submit_task_apply', 'URL', 40, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (465, '新建命题申请-班级列表', '/api/admin/exam/task/list_task_apply_class', 'URL', 39, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (466, '命题任务-审批-获取试卷审批表数据', '/api/admin/exam/task/find_approve_form_json', 'URL', 39, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (467, '命题任务-审批-获取试卷审批表pdf', '/api/admin/exam/task/find_approve_form_pdf', 'URL', 39, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (468, '下载审批表', 'Download', 'LINK', 40, 5, 'AUTH', '466,467', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (469, '命题计划统计', 'Statistics', 'MENU', 14, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (470, '命题计划统计表', 'StatisticsManage', 'MENU', 469, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (471, '查询', 'Select', 'BUTTON', 470, 1, 'AUTH', '476', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (472, '模板下载', 'Download', 'BUTTON', 470, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (473, '导入', 'Import', 'BUTTON', 470, 3, 'AUTH', '475', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (474, '列表', 'List', 'LIST', 470, 1, 'AUTH', '476', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (475, '导入', '/api/admin/statistics/import', 'URL', 470, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (476, '列表', '/api/admin/statistics/list', 'URL', 470, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (477, '命题任务-考试对象查询', '/api/admin/exam/task/find_exam_object', 'URL', 39, 12, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (478, '刷新数据', 'Freshen', 'BUTTON', 470, 4, 'AUTH', '480', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (479, '删除数据', 'Delete', 'BUTTON', 470, 5, 'AUTH', '481', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (480, '刷新数据', '/api/admin/statistics/freshen', 'URL', 470, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (481, '删除数据', '/api/admin/statistics/delete', 'URL', 470, 4, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (483, '评卷管理', 'stmms', 'MENU', NULL, 6, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (486, '评卷管理', 'MarkTaskManage', 'MENU', 483, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (487, '成绩查询', 'ScoreManage', 'MENU', 561, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (488, '班级详情', 'ClassScoreDetail', 'LINK', 487, 1, 'AUTH', '490,543', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (489, '成绩报告', 'ScoreReport', 'LINK', 487, 2, 'AUTH', '494,544', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (490, '列表查询', '/api/admin/mark/archive/student/list', 'URL', 488, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (491, '评卷入口', 'MarkEntrance', 'MENU', 486, 1, NULL, '492,493,541', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (492, '评卷员登录', '/api/admin/exam/sso/marker_login', 'URL', 491, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (493, '科组长登录', '/api/admin/exam/sso/marker_leader_login', 'URL', 491, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (494, '报告数据', '/api/admin/mark/archive/score/report', 'URL', 489, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (495, '分班阅（开关）', 'OpenClassReading', 'BUTTON', 897, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (498, '列表', 'List', 'LIST', 487, 1, 'AUTH', '542', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (499, '查询条件', 'Condition', 'CONDITION', 487, 1, 'AUTH', '542', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (500, '查询', 'Select', 'BUTTON', 487, 1, 'AUTH', '542', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (506, '归档管理', 'ScoreArchiveManage', 'MENU', NULL, 7, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (507, '菜单管理', 'MenuManage', 'MENU', 1, 7, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (508, '自定义菜单', 'CustomMenu', 'MENU', 507, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (509, '同步', 'SyncUser', 'BUTTON', 5, 1, 'AUTH', '532', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (510, '学期管理', 'SemesterManage', 'MENU', 356, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (511, '新增', 'Add', 'BUTTON', 510, 1, 'AUTH', '534', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (512, '编辑', 'Edit', 'LINK', 510, 1, 'AUTH', '534', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (513, '删除', 'Delete', 'LINK', 510, 2, 'AUTH', '535', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (514, '列表', 'List', 'LIST', 510, 1, 'AUTH', '533', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (515, '专业管理', 'MajorManage', 'MENU', 356, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (516, '新增', 'Add', 'BUTTON', 515, 1, 'AUTH', '537', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (517, '编辑', 'Edit', 'LINK', 515, 1, 'AUTH', '537', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (518, '删除', 'Delete', 'LINK', 515, 2, 'AUTH', '538', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (519, '列表', 'List', 'LIST', 515, 1, 'AUTH', '536', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (520, '查询条件', 'Condition', 'CONDITION', 515, 1, 'AUTH', '536', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (521, '同步管理', 'SyncManage', 'MENU', 112, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (522, '查询', 'Select', 'BUTTON', 515, 1, 'AUTH', '536', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (523, '查询', 'Select', 'BUTTON', 521, 1, 'AUTH', '557', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (524, '查询条件', 'Condition', 'CONDITION', 521, 1, 'AUTH', '557', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (525, '列表', 'List', 'LIST', 521, 1, 'AUTH', '557', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (526, '导出日志', 'Export', 'LINK', 521, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (527, '查询自定义菜单权限', '/api/admin/set/menu/custom/list', 'URL', 508, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (528, '学校已绑定自定义菜单权限列表', '/api/admin/set/menu/custom/get_school_custom_privileges', 'URL', 508, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (529, '学校新增/修改自定义菜单权限', '/api/admin/set/menu/custom/save', 'URL', 508, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (530, '查询', 'Select', 'BUTTON', 508, 1, 'AUTH', '527,528', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (531, '保存', 'Save', 'BUTTON', 508, 2, 'AUTH', '529', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (532, '同步', '/api/admin/sys/user/push', 'URL', 5, 8, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (533, '查询', '/api/admin/basic/semester/query', 'URL', 510, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (534, '新增/编辑', '/api/admin/basic/semester/save', 'URL', 510, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (535, '删除', '/api/admin/basic/semester/delete', 'URL', 510, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (536, '查询', '/api/admin/basic/major/query', 'URL', 515, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (537, '新增/编辑', '/api/admin/basic/major/save', 'URL', 515, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (538, '删除', '/api/admin/basic/major/delete', 'URL', 515, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (539, '共用接口-查询字典', '/api/admin/common/get_dictionary', 'URL', 149, 7, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (541, '获取用户阅卷角色', '/api/admin/common/get_open_role', 'URL', 491, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (542, '查询接口', '/api/admin/mark/archive/score/list', 'URL', 487, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (543, '成绩导出接口', '/api/admin/mark/archive/score/export', 'URL', 488, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (544, '报告下载', '/api/admin/mark/archive/score/report/download', 'URL', 489, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (547, '阅卷试卷上传待办计数接口', '/api/admin/work/structure/task/ready_count', 'URL', 16, 7, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (548, '阅卷试卷上传待办接口', '/api/admin/work/structure/task/ready', 'URL', 16, 8, 'SYS', '490', 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (549, '短信日志', 'SmsManage', 'MENU', 112, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (550, '查询', 'Select', 'BUTTON', 549, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (551, '查询', '/api/admin/basic/message/list', 'URL', 550, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (552, '查询短信类型', '/api/admin/basic/message/list_message_types', 'URL', 550, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (553, '短信失败重发', '/api/admin/basic/message/resend', 'URL', 550, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (554, '查询条件', 'Condition', 'CONDITION', 549, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (555, '列表', 'List', 'LIST', 549, 1, 'AUTH', '551,552', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (556, '短信失败重发', 'Send', 'LINK', 549, 1, 'AUTH', '553', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (557, '查询', '/api/admin/data/sync/query', 'URL', 521, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (558, '模糊查询', '/api/admin/basic/semester/list', 'URL', 510, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (559, '启用/禁用', 'Enable', 'LINK', 510, 1, 'AUTH', '560', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (560, '启用/禁用', '/api/admin/basic/semester/enable', 'URL', 510, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (561, '归档管理', 'ScoreArchivePrimary', 'MENU', 506, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (562, '提交印刷', 'Submit', 'LINK', 47, 5, 'AUTH', '60', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (564, '考试试卷文件上传接口', '/api/admin/common/file/exam_paper_upload', 'URL', 144, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (566, '审核-印刷员驳回后命题老师修改', '/api/admin/exam/task/apply_resave', 'URL', 40, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (567, '查看', 'Preview', 'LINK', 403, 1, 'AUTH', '168,568,640,641,477', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (568, '查询任务详情', '/api/admin/exam/task/exam_task_get_one', 'URL', 40, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (569, '根据条件删除（物理）', '/api/admin/basic/student/delete_by_condition', 'URL', 358, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (570, '批量删除', 'Delete', 'BUTTON', 358, 1, 'AUTH', '569', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (571, '批量删除', 'Delete', 'BUTTON', 450, 1, 'AUTH', '453', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (572, '批量删除', 'Delete', 'BUTTON', 515, 1, 'AUTH', '538', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (573, '根据查询条件批量删除', '/api/admin/basic/course/delete_by_query', 'URL', 13, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (574, '批量删除', 'Delete', 'BUTTON', 13, 5, 'AUTH', '573', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (575, '课程管理', 'basicInfo', 'MENU', 650, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (576, '课程管理', 'CourseSimpleManage', 'MENU', 575, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (598, '教学学生-查询', '/api/admin/teach/student/page', 'URL', 597, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (599, '教学学生-创建教学学生', '/api/admin/teach/student/create', 'URL', 597, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (600, '教学学生-批量删除', '/api/admin/teach/student/delete', 'URL', 597, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (601, '教学学生-批量导入', '/api/admin/teach/student/import', 'URL', 597, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (602, '教学学生-列表', 'List', 'LIST', 597, 1, 'AUTH', '598', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (603, '教学学生-查询', 'Select', 'BUTTON', 597, 2, 'AUTH', '598', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (604, '教学学生-新增', 'Add', 'BUTTON', 597, 3, 'AUTH', '599', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (605, '教学学生-批量导入', 'Import', 'BUTTON', 597, 4, 'AUTH', '601', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (606, '教学学生-删除', 'Delete', 'LINK', 597, 5, 'AUTH', '600', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (607, '教学学生-批量删除', 'DeleteBatch', 'BUTTON', 597, 6, 'AUTH', '600', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (609, '考试管理', 'ExamManage', 'MENU', 356, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (610, '查询', 'Select', 'BUTTON', 609, 1, 'AUTH', '616', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (611, '新增', 'Add', 'BUTTON', 609, 3, 'AUTH', '617', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (612, '启用/禁用', 'Enable', 'LINK', 609, 4, 'AUTH', '618', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (613, '删除', 'Delete', 'LINK', 609, 5, 'AUTH', '619', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (614, '设置', 'Set', 'LINK', 609, 6, 'AUTH', '620,621,622,623,49', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (615, '查询条件', 'Condition', 'CONDITION', 609, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (616, '查询', '/api/admin/basic/exam/page', 'URL', 609, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (617, '新增/修改', '/api/admin/basic/exam/save', 'URL', 609, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (618, '启用/禁用', '/api/admin/basic/exam/enable', 'URL', 609, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (619, '删除', '/api/admin/basic/exam/delete', 'URL', 609, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (620, '设置', '/api/admin/basic/exam/set_in_userd', 'URL', 609, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (621, '查询', '/api/admin/basic/print_config/page', 'URL', 614, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (622, '新增/修改', '/api/admin/basic/print_config/save', 'URL', 614, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (623, '启用/禁用', '/api/admin/basic/print_config/enable', 'URL', 614, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (624, '题卡管理', 'CardManage', 'MENU', 3, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (625, '查询', 'Select', 'BUTTON', 624, 1, 'AUTH', '627', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (626, '查询条件', 'Condition', 'CONDITION', 624, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (627, '查询', '/api/admin/exam/card/page', 'URL', 624, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (628, '编辑', 'Edit', 'LINK', 609, 7, 'AUTH', '617', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (629, '新增', 'Add', 'BUTTON', 624, 3, 'AUTH', '631', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (630, '编辑题卡/编辑信息', 'Edit', 'LINK', 624, 4, 'AUTH', '631', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (631, '新增/修改', '/api/admin/exam/card/save_generic', 'URL', 624, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (632, '列表', 'List', 'LIST', 609, 7, 'AUTH', '616', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (633, '列表', 'List', 'LIST', 624, 5, 'AUTH', '627', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (634, '发送验证码（绑定手机号）', '/api/admin/common/get_verify_code_for_bind', 'URL', 130, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (635, '删除', 'Delete', 'LINK', 624, 6, 'AUTH', '636', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (636, '删除', '/api/admin/exam/card/delete_generic', 'URL', 624, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (637, '模糊查询', '/api/admin/basic/exam/query', 'URL', 609, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (638, '编辑', 'Edit', 'LINK', 402, 1, 'AUTH', '639', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (639, '编辑流程', '/api/admin/custom/flow/edit', 'URL', 402, 14, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (640, '获取当前流程节点信息', '/api/admin/custom/flow/task/info', 'URL', 402, 15, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (641, '流程信息', '/api/admin/custom/flow/view', 'URL', 402, 16, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (642, '根据考试ID和机构ID查询印品', '/api/admin/basic/print_config/get_by_exam_id_and_org_id', 'URL', 614, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (643, '重命名', 'Rename', 'LINK', 402, 3, 'AUTH', '643', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (644, '重命名自定义流程名称', '/api/admin/custom/flow/rename', 'URL', 402, 17, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (647, '成员管理', 'Manager', 'LINK', 6, 3, 'AUTH', '648,649', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (648, '新增/修改', '/api/admin/sys/role/group/save', 'URL', 647, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (649, '查询', '/api/admin/sys/role/group/list', 'URL', 647, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (650, '课程管理', 'baseInfo', 'MENU', NULL, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (651, '电子交卷', 'electronic', 'MENU', NULL, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (652, '电子交卷', 'electronicSubmit', 'MENU', 651, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (653, '下载管理', 'DownloadManage', 'MENU', 112, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (654, '查询', 'Select', 'BUTTON', 653, 1, 'AUTH', '659', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (655, '查询条件', 'Condition', 'CONDITION', 653, 2, 'AUTH', '659', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (656, '下载文件', 'Download', 'LINK', 653, 3, 'AUTH', '660', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (657, '批量下载文件', 'Download', 'BUTTON', 653, 4, 'AUTH', '661', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (658, '列表', 'List', 'LIST', 653, 5, 'AUTH', '659', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (659, '查询', '/api/admin/data/download/page', 'URL', 653, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (660, '下载文件', '/api/admin/data/download/download_one', 'URL', 653, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (661, '批量下载文件', '/api/admin/data/download/download_batch', 'URL', 653, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (662, '教学学生-从课程班级批量添加学生', '/api/admin/teach/student/create_batch', 'URL', 597, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (663, '教学学生-从课程班级添加', 'SaveBatch', 'BUTTON', 597, 7, 'AUTH', '662', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (664, '重新同步', 'Push', 'LINK', 521, 1, 'AUTH', '665', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (665, '重新同步', '/api/admin/data/sync/resync', 'URL', 521, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (672, '获取题卡图片', '/api/admin/exam/card/find_jpg_file', 'URL', 488, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (675, '授权配置', 'AuthManage', 'MENU', 1, 6, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (676, '授权配置-导出硬件信息按钮', 'Export', 'BUTTON', 675, 1, 'AUTH', '677,678,679,794', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (677, '导出硬件信息', '/api/admin/auth/export/device/info', 'URL', 676, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (678, '离线激活', '/api/admin/auth/offline/activation', 'URL', 676, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (679, '查询激活信息', '/api/admin/auth/select', 'URL', 676, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (680, '教研分析', 'analyze', 'MENU', NULL, 8, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (681, '分析批次管理', 'AnalysisBatchManage', 'MENU', 680, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (682, '查询', '/api/admin/grade/batch/page', 'URL', 681, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (683, '新建', '/api/admin/grade/batch/save', 'URL', 681, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (684, '删除', '/api/admin/grade/batch/delete', 'URL', 681, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (685, '推送', '/api/admin/grade/batch/push', 'URL', 681, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (686, '查询', 'Select', 'BUTTON', 681, 1, 'AUTH', '682', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (687, '列表', 'List', 'LIST', 681, 2, 'AUTH', '682', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (688, '新建', 'Add', 'BUTTON', 681, 3, 'AUTH', '683', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (689, '批量删除', 'Delete', 'BUTTON', 681, 4, 'AUTH', '684', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (690, '删除', 'Delete', 'LINK', 681, 5, 'AUTH', '684', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (691, '推送', 'Push', 'LINK', 681, 6, 'AUTH', '685', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (692, '添加分析课程', 'Window', 'LINK', 681, 7, 'AUTH', '693,694,695,696,704,705', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (693, '查询', '/api/admin/grade/batch/paper/page', 'URL', 692, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (694, '新建', '/api/admin/grade/batch/paper/save', 'URL', 692, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (695, '删除', '/api/admin/grade/batch/paper/delete', 'URL', 692, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (696, '查询可分析试卷', '/api/admin/grade/batch/paper/able', 'URL', 692, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (697, '课程查询', 'Select', 'BUTTON', 692, 1, 'AUTH', '693', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (698, '课程列表', 'List', 'LIST', 692, 2, 'AUTH', '693', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (699, '课程添加分析课程', 'Add', 'BUTTON', 692, 3, 'AUTH', '694', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (700, '课程批量删除', 'Delete', 'BUTTON', 692, 4, 'AUTH', '695', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (701, '课程删除', 'Delete', 'BUTTON', 692, 5, 'AUTH', '695', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (702, '可分析课程查询', 'Select', 'BUTTON', 692, 6, 'AUTH', '696', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (703, '可分析课程列表', 'List', 'LIST', 692, 7, 'AUTH', '696', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (704, '推送', '/api/admin/grade/batch/paper/push', 'URL', 692, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (705, '计算', '/api/admin/grade/batch/paper/calculate', 'URL', 692, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (706, '推送', 'Push', 'LINK', 692, 8, 'AUTH', '704', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (707, '计算', 'Calculate', 'LINK', 692, 9, 'AUTH', '705', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (708, '蓝图设置', 'DataInitManage', 'MENU', 680, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (709, '成绩分析课程数据初始化', '/api/admin/grade/paper/initialize', 'URL', 708, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (710, '查询', 'Select', 'BUTTON', 708, 1, 'AUTH', '709', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (711, '列表', 'List', 'LIST', 708, 2, 'AUTH', '709', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (712, '成绩分析基础配置', 'Window', 'LINK', 708, 3, 'AUTH', '717,718,719,720,726,727,728,729,735,736,737,738,755', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (717, '查询', '/api/admin/grade/paper/dimension/list', 'URL', 712, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (718, '新建维度', '/api/admin/grade/paper/dimension/save', 'URL', 712, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (719, '模板导出', '/api/admin/grade/paper/dimension/export', 'URL', 712, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (720, '导入', '/api/admin/grade/paper/dimension/import', 'URL', 712, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (721, '查询', 'Select', 'BUTTON', 712, 1, 'AUTH', '717', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (722, '列表', 'List', 'LIST', 712, 2, 'AUTH', '717', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (723, '新建', 'Add', 'BUTTON', 712, 3, 'AUTH', '718', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (724, '导入', 'Import', 'BUTTON', 712, 4, 'AUTH', '720', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (725, '模板导出', 'Export', 'BUTTON', 712, 5, 'AUTH', '719', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (726, '查询', '/api/admin/grade/paper/struct/list', 'URL', 712, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (727, '新建试卷结构', '/api/admin/grade/paper/struct/save', 'URL', 712, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (728, '导入', '/api/admin/grade/paper/struct/import', 'URL', 712, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (729, '模板导出', '/api/admin/grade/paper/struct/export', 'URL', 712, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (730, '查询', 'Select', 'BUTTON', 712, 1, 'AUTH', '726', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (731, '列表', 'List', 'LIST', 712, 2, 'AUTH', '726', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (732, '导入', 'Import', 'BUTTON', 712, 3, 'AUTH', '728', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (734, '模板导出', 'Export', 'BUTTON', 712, 4, 'AUTH', '729', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (735, '查询定义', '/api/admin/grade/paper/paper_define', 'URL', 712, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (736, '新建定义', '/api/admin/grade/paper/save_define', 'URL', 712, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (737, '查询', '/api/admin/grade/module/evaluation/list', 'URL', 712, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (738, '新建模块评价', '/api/admin/grade/module/evaluation/save', 'URL', 712, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (739, '查看报告', 'AnalysisReportView', 'MENU', 680, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (740, '教研分析单点登录', '/api/admin/exam/sso/analysis_login', 'URL', 149, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (741, '考务数据导出', 'Export', 'LINK', 681, 8, 'AUTH', '743', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (742, '考务数据导入', 'Import', 'LINK', 681, 9, 'AUTH', '744', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (743, '考务数据导出', '/api/admin/grade/batch/download', 'URL', 681, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (744, '考务数据导入', '/api/admin/grade/batch/upload', 'URL', 681, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (745, '教研待办接口', '/api/admin/work/grade/task/ready', 'URL', 16, 9, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (746, '教研待办计数接口', '/api/admin/work/grade/task/ready_count', 'URL', 16, 10, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (747, '开始计算', 'StartCalc', 'LINK', 681, 12, 'AUTH', '748', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (748, '开始计算', '/api/admin/grade/batch/start_calc', 'URL', 681, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (749, '查询条件', 'Condition', 'CONDITION', 708, 4, 'AUTH', '709', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (750, '查询条件', 'Condition', 'CONDITION', 681, 14, 'AUTH', '682', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (751, '课程发布', 'Publish', 'LINK', 681, 6, 'AUTH', '752', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (752, '课程发布', '/api/admin/grade/batch/paper/publish', 'URL', 681, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (755, '更新试卷机构', '/api/admin/grade/paper/struct/change_paper_structure', 'URL', 712, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (756, '查询已审核数量', '/api/admin/exam/task/review_list_audited_count', 'URL', 41, 6, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (757, '批量导入', '/api/admin/sys/org/import', 'URL', 4, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (758, '导入', 'Import', 'BUTTON', 4, 4, 'AUTH', '757', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (759, '考生对象查询', '/api/admin/exam/task/find_exam_task_student_object', 'URL', 40, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (760, '增加考试对象', 'ExamTaskStudentObject', 'BUTTON', 40, 9, 'AUTH', '759', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (761, '导入考生对象', '/api/admin/exam/task/exam_task_exam_student_import', 'URL', 40, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (762, '批量导入考生', 'ExamTaskStudentObjectImport', 'BUTTON', 40, 10, 'AUTH', '761', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (763, '查询详情', '/api/admin/basic/template/get_one', 'URL', 11, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (764, '复制', '/api/admin/exam/card/copy', 'URL', 175, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (765, '查询开课学院', '/api/admin/grade/batch/paper/find_teach_college', 'URL', 692, 7, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (766, '查询条件', 'Condition', 'CONDITION', 470, 1, 'AUTH', '476', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (767, '同步机构数据', '/api/admin/auth/sync/org', 'URL', 676, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (768, '题库单点登录', '/api/admin/exam/sso/question_library_login', 'URL', 149, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (769, '题库管理', 'questionLibrary', 'MENU', NULL, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (770, '题库管理', 'QuestionLibraryManage', 'MENU', 769, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (771, '用户验证查询', '/api/admin/set/user/select', 'URL', 508, 4, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (772, '用户验证保存', '/api/admin/set/user/save', 'URL', 508, 5, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (773, '试卷规格配置保存', '/api/admin/set/paper/save', 'URL', 508, 6, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (774, '试卷规格配置查询', '/api/admin/set/paper/select', 'URL', 508, 7, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (775, '系统试卷规格配置查询', '/api/admin/set/paper/sys/select', 'URL', 508, 8, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (776, '同步配置保存', '/api/admin/set/sync/save', 'URL', 508, 9, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (777, '同步配置查询', '/api/admin/set/sync/select', 'URL', 508, 10, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (778, '数据还原', '/api/admin/set/backup', 'URL', 508, 11, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (779, '查询自定义角色权限', '/api/admin/set/role/custom/list', 'URL', 508, 12, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (780, '学校新增/修改自定义角色权限', '/api/admin/set/role/custom/save', 'URL', 508, 13, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (781, '学校已绑定自定义角色权限列表', '/api/admin/set/role/custom/get_school_custom_roles', 'URL', 508, 14, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (782, '操作日志', 'ActionLogManage', 'MENU', 112, 5, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (783, '查询', '/api/admin/basic/operation_log/query', 'URL', 782, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (784, '查询条件', 'Condition', 'CONDITION', 782, 1, 'AUTH', '783', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (785, '查询', 'Select', 'BUTTON', 782, 2, 'AUTH', '783', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (788, '删除', 'Delete', 'LINK', 11, 4, 'AUTH', '789', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (789, '删除', '/api/admin/basic/template/remove', 'URL', 11, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (790, '查询异步结果', '/api/admin/data/task/query_result', 'URL', 113, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (794, '在线激活', '/api/admin/auth/online/activation', 'URL', 676, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (795, '学校管理', 'SchoolManage', 'MENU', 1, 7, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (796, '查询', 'Select', 'BUTTON', 795, 1, 'AUTH', '798', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (797, '查询条件', 'Condition', 'CONDITION', 795, 2, 'AUTH', '798', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (798, '查询', '/api/admin/commom/school/list', 'URL', 795, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (799, '机构同步', 'Sync', 'BUTTON', 795, 3, 'AUTH', '800', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (800, '机构同步', '/api/admin/auth/sync/org', 'URL', 795, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (801, '编辑', 'Edit', 'LINK', 795, 1, 'AUTH', '802', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (802, '编辑', '/api/admin/commom/school/update', 'URL', 795, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (803, '列表', 'List', 'LIST', 782, 3, 'AUTH', '783', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (804, '删除', 'Delete', 'LINK', 39, 5, 'AUTH', '805', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (805, '删除', '/api/admin/exam/task/delete', 'URL', 39, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (806, '删除', 'Delete', 'LINK', 40, 6, 'AUTH', '807', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (807, '删除', '/api/admin/exam/task/apply_delete', 'URL', 40, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (808, '删除', 'Delete', 'LINK', 44, 2, 'AUTH', '809', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (809, '删除', '/api/admin/exam/print/data_delete', 'URL', 44, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (810, '导出', 'Export', 'BUTTON', 4, 5, 'AUTH', '811', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (811, '导出', '/api/admin/sys/org/export', 'URL', 4, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (812, '导出', 'Export', 'BUTTON', 5, 4, 'AUTH', '813', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (813, '导出', '/api/admin/sys/user/export', 'URL', 5, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (814, '导出', 'Export', 'BUTTON', 358, 3, 'AUTH', '815', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (815, '导出', '/api/admin/basic/student/export', 'URL', 358, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (816, '导出', 'Export', 'BUTTON', 13, 6, 'AUTH', '817', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (817, '导出', '/api/admin/basic/course/export', 'URL', 13, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (825, '命题任务公共卷型查询接口', '/api/admin/exam/print/list_relate_common_types', 'URL', 43, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (826, '批量关联接口', '/api/admin/exam/print/relate_update_batch', 'URL', 43, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (827, '超管-角色管理查询', '/api/admin/sys/role/list_to_admin', 'URL', 5, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (828, '生成图片/预览图片', 'ConvertImage', 'LINK', 624, 7, 'AUTH', '829', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (829, '生成图片', '/api/admin/exam/card/convert_image', 'URL', 624, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (830, '教学课程-分页查询', '/api/admin/teach/course/page', 'URL', 576, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (831, '教学课程-手动添加', '/api/admin/teach/course/manual_create', 'URL', 576, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (832, '教学课程-课程表选择并添加', '/api/admin/teach/course/select_create', 'URL', 576, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (833, '教学课程-批量导入', '/api/admin/teach/course/import', 'URL', 576, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (834, '教学课程-批量删除', '/api/admin/teach/course/delete', 'URL', 576, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (835, '教学课程-列表', 'List', 'LIST', 576, 1, 'AUTH', '830', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (836, '查询', 'Select', 'BUTTON', 576, 2, 'AUTH', '830', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (837, '删除', 'Delete', 'LINK', 576, 3, 'AUTH', '834', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (838, '查询条件', 'Condition', 'CONDITION', 576, 4, 'AUTH', '830', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (839, '基础课程-根据学院查询课程', '/api/admin/basic/course/list_by_org', 'URL', 13, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (840, '新增', 'AddCourse', 'BUTTON', 576, 5, 'AUTH', '831,832,833,839', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (841, '管理', 'Link', 'LINK', 576, 6, 'AUTH', '77,842,843,844,845,846,847,848,849,850,851,852,853,866', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (842, '教学班-分页查询', '/api/admin/teach/clazz/page', 'URL', 841, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (843, '教学班-编辑', '/api/admin/teach/clazz/edit', 'URL', 841, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (844, '教学班-删除', '/api/admin/teach/clazz/delete', 'URL', 841, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (845, '教师团队-分页查询', '/api/admin/teach/course/teacher_team_page', 'URL', 841, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (846, '教师团队-手动添加', '/api/admin/teach/course/manual_create_teacher', 'URL', 841, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (847, '教师团队-教师库添加', '/api/admin/teach/course/select_create_teacher', 'URL', 841, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (848, '教师团队-批量导入教师', '/api/admin/teach/course/import_teacher', 'URL', 841, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (849, '教学学生-创建教学学生', '/api/admin/teach/student/create', 'URL', 841, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (850, '教学学生-查询', '/api/admin/teach/student/page', 'URL', 841, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (851, '教学学生-从课程班级批量添加学生', '/api/admin/teach/student/create_batch', 'URL', 841, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (852, '教学学生-批量导入', '/api/admin/teach/student/import', 'URL', 841, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (853, '教学学生-批量删除', '/api/admin/teach/student/delete', 'URL', 841, 12, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (856, '查看', 'Preview', 'LINK', 624, 8, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (860, '共用接口-查询学期', '/api/admin/basic/condition/list_semester', 'URL', 149, 8, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (861, '共用接口-查询考试', '/api/admin/basic/condition/list_exam', 'URL', 149, 9, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (862, '共用接口-查询计划', '/api/admin/basic/condition/list_print_plan', 'URL', 149, 10, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (863, '共用接口-查询课程', '/api/admin/basic/condition/list_course', 'URL', 149, 11, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (864, '共用接口-查询试卷编号', '/api/admin/basic/condition/list_paper_number', 'URL', 149, 12, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (865, '共用接口-查询班级', '/api/admin/basic/condition/list_clazz', 'URL', 149, 13, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (866, '教师团队-根据机构查询', '/api/admin/teach/course/find_teacher_by_org', 'URL', 841, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (867, '下载', 'Download', 'LINK', 624, 9, 'AUTH', '868', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (868, '下载', '/api/admin/exam/card/download_card', 'URL', 624, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (869, '试卷题卡预览日志记录接口', '/api/admin/basic/operation_log/save_preview_log', 'URL', 149, 14, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (872, '批量撤回', 'BatchEnd', 'BUTTON', 47, 3, 'AUTH', '873', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (873, '批量撤回', '/api/admin/exam/print/task_batch_cancel', 'URL', 47, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (874, '共用接口-查询学院', '/api/admin/basic/condition/list_college', 'URL', 149, 14, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (875, '我教课程选择', 'MyTeachCourse', 'BUTTON', 40, 10, 'AUTH', '759', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (876, '学生库选择', 'StudentLibrary', 'BUTTON', 40, 11, 'AUTH', '759', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (877, '选择学生（学生库选择）', 'SelectStudent', 'BUTTON', 40, 12, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (878, '作废', 'Cancel', 'LINK', 42, 5, 'AUTH', '879', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (879, '作废', '/api/admin/exam/task/cancel', 'URL', 42, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (880, '重启流程', '/api/admin/exam/task/restart', 'URL', 42, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (881, '打回', 'Restart', 'LINK', 42, 6, 'AUTH', '880', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (884, '列表', 'List', 'LIST', 491, 1, 'AUTH', '887', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (885, '查询', 'Select', 'BUTTON', 491, 2, 'AUTH', '887', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (886, '点击评卷', 'Marking', 'LINK', 491, 3, 'AUTH', '888,889,890,891,892,893,894,895,896', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (887, '查询', '/api/admin/mark/entrance/list', 'URL', 491, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (888, '评卷页面', '/api/mark/getSetting', 'URL', 491, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (889, '获取分组列表', '/api/mark/getGroup', 'URL', 491, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (890, '获取评卷状态', '/api/mark/getStatus', 'URL', 491, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (891, '获取任务', '/api/mark/getTask', 'URL', 491, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (892, '保存任务', '/api/mark/saveTask', 'URL', 491, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (893, '获取评卷历史', '/api/mark/getHistory', 'URL', 491, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (894, '更新个人评卷配置', '/api/mark/updateSetting', 'URL', 491, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (895, '选择评卷分组', '/api/mark/subjectSelect', 'URL', 491, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (896, '清理评卷员的评卷任务缓存', '/api/mark/clear', 'URL', 491, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (897, '评卷设置', 'MarkSetting', 'MENU', 486, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (898, '列表', 'List', 'LIST', 897, 1, 'AUTH', '900', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (899, '查询', 'Select', 'BUTTON', 897, 2, 'AUTH', '900', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (900, '查询', '/api/admin/mark/setting/list', 'URL', 897, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (901, '评卷参数设置', 'MarkParamSetting', 'LINK', 897, 3, 'AUTH', '672,902,903,904,905,906,907,908,909,910,911,912,913,962,963', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (902, '结构查询', '/api/admin/mark/question/list', 'URL', 897, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (903, '结构提交', '/api/admin/mark/question/save', 'URL', 897, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (904, '评卷任务查询', '/api/admin/mark/group/list', 'URL', 897, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (905, '评卷任务新增', '/api/admin/mark/group/save', 'URL', 897, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (906, '评卷任务修改', '/api/admin/mark/group/update', 'URL', 897, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (907, '评卷任务删除', '/api/admin/mark/group/delete', 'URL', 897, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (908, '分班阅查询', '/api/admin/mark/class/list', 'URL', 897, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (909, '分班阅提交', '/api/admin/mark/class/save', 'URL', 897, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (910, '客观题标答查询', '/api/admin/mark/question/objective/list', 'URL', 897, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (911, '客观题标答提交', '/api/admin/mark/question/objective/save', 'URL', 897, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (912, '主观题标答查询', '/api/admin/mark/question/subjective/get', 'URL', 897, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (913, '主观题标答提交', '/api/admin/mark/question/subjective/upload', 'URL', 897, 12, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (914, '评卷设置', 'MarkConfig', 'LINK', 897, 4, 'AUTH', '915,916', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (915, '数据获取', '/api/admin/mark/setting/paper/get', 'URL', 897, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (916, '数据保存', '/api/admin/mark/setting/paper/save', 'URL', 897, 14, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (917, '评卷管理', 'MarkManage', 'MENU', 486, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (918, '列表', 'List', 'LIST', 917, 1, 'AUTH', '925', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (919, '查询', 'Select', 'BUTTON', 917, 2, 'AUTH', '925', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (920, '查询条件', 'Condition', 'CONDITION', 917, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (921, '查询条件', 'Condition', 'CONDITION', 491, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (922, '查询条件', 'Condition', 'CONDITION', 897, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (923, '导出评卷工作量', 'Export', 'BUTTON', 917, 4, 'AUTH', '926', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (924, '结束评卷', 'BatchFinish', 'BUTTON', 917, 5, 'AUTH', '927', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (925, '列表获取', '/api/admin/mark/setting/summary', 'URL', 917, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (926, '导出评卷员工作量', '/api/admin/mark/setting/export_marker', 'URL', 917, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (927, '结束评卷', '/api/admin/mark/paper/finish', 'URL', 917, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (928, '进度列表查询', '/api/admin/mark/group/summary', 'URL', 917, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (929, '评卷员管理列表查询', '/api/admin/mark/marker/list', 'URL', 917, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (930, '重置', '/api/admin/mark/marker/reset', 'URL', 917, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (931, '解绑', '/api/admin/mark/marker/delete', 'URL', 917, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (932, '回收', '/api/admin/mark/marker/release', 'URL', 917, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (933, '设置评卷数', '/api/admin/mark/marker/setTaskCount', 'URL', 917, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (934, '评卷质量查询', '/api/admin/mark/quality/list', 'URL', 917, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (935, '问题试卷列表查询', '/api/admin/mark/problem/list', 'URL', 917, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (936, '问题试卷重置', '/api/admin/mark/problem/reset', 'URL', 917, 12, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (937, '仲裁管理列表查询', '/api/admin/mark/arbitrate/list', 'URL', 917, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (938, '仲裁-获取任务', '/api/admin/mark/arbitrate/getTask', 'URL', 917, 14, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (939, '仲裁-左边的列表', '/api/admin/mark/arbitrate/getArbitrationList', 'URL', 917, 15, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (940, '仲裁-清理锁卷缓存', '/api/admin/mark/arbitrate/clear', 'URL', 917, 16, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (941, '仲裁-获取评卷状态', '/api/admin/mark/arbitrate/getStatus', 'URL', 917, 17, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (942, '仲裁-获取配置信息', '/api/admin/mark/arbitrate/getSetting', 'URL', 917, 18, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (943, '仲裁-保存任务', '/api/admin/mark/arbitrate/saveTask', 'URL', 917, 19, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (944, '查看详情', 'Detail', 'LINK', 917, 6, 'AUTH', '928,929,930,931,932,933,934,935,936,937,938,939,940,941,942,943,964,965,966,967,968,1008', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (945, '结束评卷', 'Finish', 'LINK', 917, 7, 'AUTH', '927', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (946, '成绩检查', 'ScoreCheck', 'MENU', 486, 4, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (947, '列表', 'List', 'LIST', 946, 1, 'AUTH', '952', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (948, '查询条件', 'Condition', 'CONDITION', 946, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (949, '查询', 'Select', 'BUTTON', 946, 3, 'AUTH', '952', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (950, '查看详情', 'Detail', 'LINK', 946, 4, 'AUTH', '953,954,955,956,957,958,959,960,969', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (951, '查看签到表', 'PackageDetail', 'LINK', 946, 5, 'AUTH', '961', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (952, '列表获取', '/api/admin/mark/setting/scoreList', 'URL', 946, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (953, '详情列表', '/api/admin/mark/student/score', 'URL', 946, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (954, '获取配置', '/api/admin/mark/track/getSetting', 'URL', 946, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (955, '获取任务', '/api/admin/mark/track/getTask', 'URL', 946, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (956, '主观题检查-获取配置', '/api/admin/mark/inspected/getSetting', 'URL', 946, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (957, '主观题检查-获取任务', '/api/admin/mark/inspected/subjective/getTask', 'URL', 946, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (958, '主观题检查-保存任务', '/api/admin/mark/inspected/subjective/saveTask', 'URL', 946, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (959, '客观题检查-获取任务', '/api/admin/mark/inspected/objective/getTask', 'URL', 946, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (960, '客观题检查-保存任务', '/api/admin/mark/inspected/objective/saveTask', 'URL', 946, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (961, '签到表列表', '/api/admin/mark/paper/package/list', 'URL', 946, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (962, '评卷区设置', '/api/admin/mark/group/update_picture_config', 'URL', 897, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (963, '开启/关闭分班阅', '/api/admin/mark/group/update_open_mark_class', 'URL', 897, 14, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (964, '班级阅卷进度', '/api/admin/mark/group/class/summary', 'URL', 917, 20, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (965, '绑定评卷员', '/api/admin/mark/marker/add', 'URL', 917, 21, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (966, '评卷质量重新计算', '/api/admin/mark/quality/update', 'URL', 917, 22, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (967, '评卷质量给分曲线', '/api/admin/mark/quality/chart', 'URL', 917, 23, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `enable`, `default_auth`, `front_display`) VALUES ('968', '绑定评卷员-已绑定评卷员集合', '/api/admin/mark/marker/list_bind_marker', 'URL', '917', '20', 'AUTH', '1', '1', '1');
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `enable`, `default_auth`, `front_display`) VALUES ('969', '客观题统分', '/api/admin/mark/student/objective/calculate', 'URL', '946', '11', 'AUTH', '1', '1', '1');
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (970, '扫描端', 'scan', 'MENU', NULL, 11, NULL, '971,972,973,974,975,976,977,978,979,980,981,982,983,984,985,986,987,988,989,990,991,992,993,994,995,996,997,998,999,1000,1001,1002,1003,1004,1005,1007,1009,1010', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (971, '心跳接口', '/api/scan/server/heartbeat', 'URL', 970, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (972, '登出接口', '/api/scan/logout', 'URL', 970, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (973, '获取学期、考试列表', '/api/admin/basic/condition/scan/semester/list', 'URL', 970, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (974, '获取课程、试卷编号列表', '/api/admin/basic/condition/scan/course/list', 'URL', 970, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (975, '获取校区、考场', '/api/admin/basic/condition/scan/campus/list', 'URL', 970, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (976, '扫描汇总-答题卡扫描、签到表扫描', '/api/scan/exam/card/info', 'URL', 970, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (977, '扫描汇总-数据校对', '/api/scan/exam/check/info', 'URL', 970, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (978, '答题卡卡格式列表', '/api/scan/answer/card/list', 'URL', 970, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (979, '创建/修改题卡卡格式', '/api/scan/answer/card/save', 'URL', 970, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (980, '删除答题卡卡格式', '/api/scan/answer/card/delete', 'URL', 970, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (981, '答题卡适配卡格式上传', '/api/scan/answer/card/adapte/upload', 'URL', 970, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (982, '答题卡卡格式查询', '/api/scan/answer/card/get', 'URL', 970, 12, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (983, '答题卡扫描批次创建', '/api/scan/answer/batch/create', 'URL', 970, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (984, '答题卡扫描批次完成', '/api/scan/answer/batch/finish', 'URL', 970, 14, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (985, '答题卡扫描结果保存', '/api/scan/answer/batch/save', 'URL', 970, 15, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (986, '获取考生信息', '/api/scan/student/find', 'URL', 970, 16, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (987, '答题卡扫描原图上传', '/api/scan/answer/sheet/upload', 'URL', 970, 17, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (988, '答题卡条码识别坐标获取', '/api/scan/package/position/get', 'URL', 970, 18, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (989, '签到表扫描图片上传', '/api/scan/package/image/upload', 'URL', 970, 19, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (990, '签到表扫描结果保存', '/api/scan/package/save', 'URL', 970, 20, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (991, '签到表扫描结果列表', '/api/scan/package/image/list', 'URL', 970, 21, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (992, '签到表编号查询', '/api/scan/package/find', 'URL', 970, 22, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (993, '签到表详情', '/api/scan/package/image/get', 'URL', 970, 23, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (994, '查询答题卡扫描详情', '/api/scan/answer/query', 'URL', 970, 24, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (995, '查询答题卡扫描概要', '/api/scan/answer/summary', 'URL', 970, 25, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (996, '修改答题卡识别结果', '/api/scan/answer/omr/edit', 'URL', 970, 26, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (997, '更新缺考状态', '/api/scan/student/absent/update', 'URL', 970, 27, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (998, '修改答题卡扫描图片绑定考生', '/api/scan/answer/paper/migrate', 'URL', 970, 28, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (999, '任务确认', '/api/scan/student/confirm', 'URL', 970, 29, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1000, '任务待处理列表', '/api/scan/task/omr/get', 'URL', 970, 30, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1001, '任务提交', '/api/scan/task/omr/save', 'URL', 970, 31, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1002, '任务状态', '/api/scan/task/omr/status', 'URL', 970, 32, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1003, '任务释放', '/api/scan/task/omr/release', 'URL', 970, 33, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1004, '任务历史', '/api/scan/task/omr/history', 'URL', 970, 34, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1005, '答题卡二次识别', '/api/scan/answer/refix', 'URL', 970, 35, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1006, '查询条件', 'Condition', 'CONDITION', 970, 1, 'AUTH', '971,972,973,974,975,976,977,978,979,980,981,982,983,984,985,986,987,988,989,990,991,992,993,994,995,996,997,998,999,1000,1001,1002,1003,1004,1005,1007,1009,1010', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1007, '未扫描-导出', '/api/scan/student/unexist/export', 'URL', 970, 36, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1008, '仲裁-评卷历史', '/api/admin/mark/arbitrate/getHistory', 'URL', 917, 20, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1009, '任务列表', '/api/scan/task/omr/list', 'URL', 970, 37, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `enable`, `default_auth`, `front_display`) VALUES ('1010', '人工绑定确认', '/api/scan/student/assign/confirm', 'URL', '970', '38', 'AUTH', '1', '1', '1');

-- ----------------------------
-- Table structure for sys_role
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sys_role` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint DEFAULT NULL COMMENT '学校id',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `name` varchar(50) NOT NULL COMMENT '角色名称',
        `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
        `type` varchar(20) DEFAULT NULL COMMENT '角色类别，ADMIN：超级管理员，SCHOOL_ADMIN：管理员：EXAM_TEACHER：考务老师，QUESTION_TEACHER：命题老师，CUSTOMER：客服人员，PRINTER：印刷人员，CUSTOM：自定义',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '修改人',
        `update_time` bigint DEFAULT NULL COMMENT '修改时间',
        `default_role` tinyint NOT NULL DEFAULT '0' COMMENT '是否系统内置角色，1：是，0：不是',
        `source` varchar(100) DEFAULT NULL COMMENT '角色来源，ANALYSIS：教研分析',
        `interpret` varchar(1000) DEFAULT NULL COMMENT '角色说明',
        PRIMARY KEY (`id`) USING BTREE,
        UNIQUE KEY `role_schoolId_name_idx` (`school_id`,`name`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='角色表';

INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (1, NULL, NULL, '系统管理员', 1, 'ADMIN', NULL, NULL, NULL, NULL, 1, NULL, NULL);
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (2, NULL, NULL, '学校管理员', 1, 'SCHOOL_ADMIN', NULL, NULL, 1, 1700550684059, 1, 'SYS', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (3, NULL, NULL, '老师', 1, 'SCHOOL_TEACHER', NULL, NULL, 1, 1700550794366, 1, 'SYS', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (4, NULL, NULL, '学院秘书', 1, 'ASSISTANT', NULL, NULL, 1, 1700550851162, 1, 'SYS', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (5, NULL, NULL, '印刷员', 1, 'PRINT', NULL, NULL, 1, 1700550860415, 1, 'SYS', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (6, NULL, NULL, '扫描员', 1, 'SCANNER', NULL, NULL, 1, 1700550926875, 1, 'SYS', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (7, NULL, NULL, '评卷员', 1, 'MARKER', NULL, NULL, 1, 1700550961575, 1, 'CLOUD_MARK', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (8, NULL, NULL, '学科老师', 1, 'SUBJECT_TEACHER', NULL, NULL, 1, 1700550974820, 1, 'QUESTION_LIBRARY', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (9, NULL, NULL, '命题老师', 1, 'ASSIGN_TEACHER', NULL, NULL, 1, 1700551062095, 1, 'QUESTION_LIBRARY', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (10, NULL, NULL, '教务处老师', 1, 'OFFICE_TEACHER', NULL, NULL, 1, 1700551053976, 1, 'ANALYSIS', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (11, NULL, NULL, '学院院长', 1, 'PRESIDENT', NULL, NULL, 1, 1700551047527, 1, 'ANALYSIS', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (12, NULL, NULL, '任课老师', 1, 'TEACHER', NULL, NULL, 1, 1700551038560, 1, 'ANALYSIS', '系统内置');

-- ----------------------------
-- Table structure for sys_role_group_member
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sys_role_group_member` (
        `id` bigint NOT NULL COMMENT '主键',
        `role_id` bigint NOT NULL COMMENT '角色id',
        `user_id` bigint NOT NULL COMMENT '用户id',
        `member_id` bigint NOT NULL COMMENT '成员id',
        `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
        PRIMARY KEY (`id`) USING BTREE,
        UNIQUE KEY `role_group_roleId_memberId_idx` (`role_id`,`member_id`,`enable`) USING BTREE,
        KEY `role_group_member_idx` (`role_id`,`user_id`,`member_id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='角色组成员表';

-- ----------------------------
-- Table structure for sys_role_privilege
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sys_role_privilege` (
        `id` bigint NOT NULL COMMENT '主键',
        `role_id` bigint NOT NULL COMMENT '角色id',
        `privilege_id` bigint NOT NULL COMMENT '菜单id',
        `enable` tinyint DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
        `school_id` bigint DEFAULT NULL COMMENT '内置角色，按学校自定义的权限',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='角色和菜单关联表';

INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (1,1,1,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (2,1,2,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (3,1,4,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (4,1,5,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (5,1,6,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (6,1,92,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (7,1,93,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (8,1,94,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (9,1,95,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (10,1,224,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (11,1,225,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (12,1,226,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (13,1,227,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (14,1,228,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (15,1,234,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (16,1,77,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (17,1,78,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (18,1,79,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (19,1,80,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (20,1,81,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (21,1,82,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (22,1,87,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (23,1,184,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (24,1,217,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (25,1,218,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (26,1,219,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (27,1,220,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (28,1,221,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (29,1,222,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (30,1,223,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (31,1,243,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (32,1,83,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (33,1,84,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (34,1,85,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (35,1,86,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (36,1,91,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (37,1,180,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (38,1,229,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (39,1,230,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (40,1,231,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (41,1,232,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (42,1,233,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (43,1,235,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (44,1,390,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (45,1,402,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (46,1,407,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (47,1,408,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (48,1,409,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (49,1,410,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (50,1,411,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (53,1,527,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (54,1,528,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (55,1,529,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (56,1,638,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (57,1,643,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (58,1,675,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (59,1,676,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (60,1,677,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (61,1,678,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (62,1,679,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (81,1,767,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (88,1,771,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (89,1,772,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (90,1,773,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (91,1,774,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (92,1,775,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (93,1,776,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (94,1,777,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (95,1,778,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (96,1,779,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (97,1,780,1,NULL);
INSERT INTO `sys_role_privilege` (`id`,`role_id`,`privilege_id`,`enable`,`school_id`) VALUES (98,1,781,1,NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847252312066, 2, 1, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847273283585, 2, 2, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847273283586, 2, 3, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847277477890, 2, 4, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847277477891, 2, 5, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847281672193, 2, 6, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847281672194, 2, 8, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847285866498, 2, 9, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847285866499, 2, 11, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847290060801, 2, 13, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847290060802, 2, 14, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847290060803, 2, 38, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847294255105, 2, 39, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847294255106, 2, 40, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847294255107, 2, 41, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847298449409, 2, 42, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847298449410, 2, 43, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847302643713, 2, 44, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847302643714, 2, 45, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847306838018, 2, 47, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847306838019, 2, 48, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847306838020, 2, 49, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847306838021, 2, 50, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847311032321, 2, 51, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847311032322, 2, 52, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847311032323, 2, 53, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847315226626, 2, 54, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847315226627, 2, 55, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847319420930, 2, 56, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847319420931, 2, 57, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847319420932, 2, 58, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847323615234, 2, 59, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847323615235, 2, 60, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847323615236, 2, 61, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847327809538, 2, 62, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847327809539, 2, 63, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847327809540, 2, 64, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847327809541, 2, 65, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847332003841, 2, 66, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847336198145, 2, 67, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847336198146, 2, 68, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847336198147, 2, 70, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847340392450, 2, 71, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847340392451, 2, 72, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847340392452, 2, 73, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847344586754, 2, 74, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847344586755, 2, 76, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847344586756, 2, 77, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847348781057, 2, 78, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847348781058, 2, 79, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847348781059, 2, 80, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847352975361, 2, 81, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847352975362, 2, 83, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847352975363, 2, 84, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847352975364, 2, 85, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847357169665, 2, 87, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847357169666, 2, 88, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847357169667, 2, 89, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847361363969, 2, 90, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847361363970, 2, 91, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847361363971, 2, 92, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847365558274, 2, 93, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847365558275, 2, 94, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847365558276, 2, 96, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847369752578, 2, 97, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847369752579, 2, 99, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847369752580, 2, 100, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847373946882, 2, 101, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847373946883, 2, 102, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847373946884, 2, 103, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847378141186, 2, 104, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847378141187, 2, 105, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847378141188, 2, 112, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847378141189, 2, 113, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847382335489, 2, 114, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847382335490, 2, 123, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847382335491, 2, 124, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847386529794, 2, 125, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847386529795, 2, 157, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847386529796, 2, 158, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847394918402, 2, 159, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847394918403, 2, 161, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847394918404, 2, 162, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847399112705, 2, 163, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847399112706, 2, 164, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847399112707, 2, 165, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847403307009, 2, 166, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847403307010, 2, 167, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847403307011, 2, 168, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847411695618, 2, 169, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847415889921, 2, 174, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847415889922, 2, 184, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847415889923, 2, 188, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847420084226, 2, 190, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847420084227, 2, 191, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847420084228, 2, 192, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847424278530, 2, 217, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847424278531, 2, 218, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847424278532, 2, 219, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847428472834, 2, 220, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847428472835, 2, 221, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847428472836, 2, 222, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847428472837, 2, 223, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847436861442, 2, 224, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847441055746, 2, 225, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847441055747, 2, 226, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847441055748, 2, 227, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847445250049, 2, 228, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847445250050, 2, 229, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847445250051, 2, 230, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847449444354, 2, 231, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847449444355, 2, 232, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847449444356, 2, 233, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847449444357, 2, 235, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847453638658, 2, 236, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847453638659, 2, 237, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847453638660, 2, 238, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847457832962, 2, 239, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847457832963, 2, 240, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847462027266, 2, 243, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847462027267, 2, 249, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847462027268, 2, 250, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847462027269, 2, 251, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847466221570, 2, 252, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847466221571, 2, 253, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847466221572, 2, 254, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847470415874, 2, 255, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847470415875, 2, 263, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847470415876, 2, 268, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847474610178, 2, 269, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847474610179, 2, 270, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847474610180, 2, 271, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847478804481, 2, 272, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847478804482, 2, 273, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847478804483, 2, 274, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847482998786, 2, 275, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847482998787, 2, 277, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847482998788, 2, 278, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847487193089, 2, 279, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847487193090, 2, 280, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847487193091, 2, 281, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847487193092, 2, 282, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847491387394, 2, 283, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847495581698, 2, 284, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847495581699, 2, 285, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847495581700, 2, 288, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847499776001, 2, 289, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847499776002, 2, 292, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847499776003, 2, 293, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847503970305, 2, 294, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847503970306, 2, 295, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847503970307, 2, 296, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847508164609, 2, 297, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847508164610, 2, 298, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847508164611, 2, 299, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847508164612, 2, 300, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847512358914, 2, 301, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847512358915, 2, 302, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847512358916, 2, 303, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847516553217, 2, 304, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847516553218, 2, 305, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847516553219, 2, 309, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847516553220, 2, 310, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847520747522, 2, 311, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847520747523, 2, 312, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847520747524, 2, 313, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847524941826, 2, 314, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847524941827, 2, 315, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847524941828, 2, 316, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847524941829, 2, 317, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847529136130, 2, 318, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847529136131, 2, 319, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847529136132, 2, 320, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847533330433, 2, 321, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847533330434, 2, 322, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847533330435, 2, 323, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847533330436, 2, 324, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847537524738, 2, 325, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847537524739, 2, 326, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847537524740, 2, 327, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847541719042, 2, 328, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847541719043, 2, 329, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847541719044, 2, 330, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847545913345, 2, 331, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847545913346, 2, 332, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847545913347, 2, 333, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847545913348, 2, 334, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847550107649, 2, 335, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847550107650, 2, 336, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847550107651, 2, 337, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847554301953, 2, 338, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847554301954, 2, 339, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847554301955, 2, 340, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847554301956, 2, 341, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847558496258, 2, 342, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847558496259, 2, 343, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847558496260, 2, 345, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847562690561, 2, 356, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847562690562, 2, 358, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847562690563, 2, 359, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847562690564, 2, 362, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847566884865, 2, 363, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847566884866, 2, 364, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847566884867, 2, 365, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847571079170, 2, 367, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847571079171, 2, 368, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847571079172, 2, 369, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847571079173, 2, 371, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847575273474, 2, 377, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847575273475, 2, 378, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847575273476, 2, 379, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847579467778, 2, 380, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847579467779, 2, 381, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847579467780, 2, 382, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847579467781, 2, 383, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847583662082, 2, 384, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847583662083, 2, 385, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847583662084, 2, 388, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847587856386, 2, 390, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847587856387, 2, 400, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847587856388, 2, 402, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847587856389, 2, 403, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847592050690, 2, 404, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847592050691, 2, 405, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847592050692, 2, 406, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847596244993, 2, 407, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847596244994, 2, 408, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847596244995, 2, 409, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847600439297, 2, 410, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847600439298, 2, 411, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847600439299, 2, 413, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847600439300, 2, 414, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847604633601, 2, 415, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847604633602, 2, 416, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847604633603, 2, 417, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847608827905, 2, 418, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847608827906, 2, 419, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847608827907, 2, 420, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847608827908, 2, 421, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847613022209, 2, 422, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847613022210, 2, 424, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847617216513, 2, 427, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847617216514, 2, 441, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847617216515, 2, 442, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847625605122, 2, 445, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847625605123, 2, 446, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847625605124, 2, 447, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847629799425, 2, 448, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847633993729, 2, 450, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847633993730, 2, 451, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847638188034, 2, 452, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847638188035, 2, 453, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847638188036, 2, 455, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847642382338, 2, 456, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847642382339, 2, 457, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847650770945, 2, 458, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847654965250, 2, 459, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847659159553, 2, 461, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847667548162, 2, 462, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847667548163, 2, 463, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847667548164, 2, 464, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847671742466, 2, 465, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847671742467, 2, 466, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847671742468, 2, 467, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847675936770, 2, 468, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847675936771, 2, 477, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847675936772, 2, 482, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847680131074, 2, 483, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847680131075, 2, 486, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847680131076, 2, 487, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847680131077, 2, 488, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847684325377, 2, 489, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847684325378, 2, 490, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847688519681, 2, 491, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847692713986, 2, 492, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847692713987, 2, 493, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847692713988, 2, 494, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847696908290, 2, 495, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847696908291, 2, 498, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847696908292, 2, 499, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847701102593, 2, 500, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847701102594, 2, 506, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847701102595, 2, 510, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847701102596, 2, 511, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847705296897, 2, 513, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847705296898, 2, 514, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847705296899, 2, 515, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847709491201, 2, 516, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847709491202, 2, 517, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847709491203, 2, 518, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847709491204, 2, 519, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847713685506, 2, 520, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847713685507, 2, 521, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847713685508, 2, 522, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847717879809, 2, 523, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847717879810, 2, 524, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847717879811, 2, 525, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847717879812, 2, 526, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847722074113, 2, 533, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847722074114, 2, 534, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847722074115, 2, 535, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847726268418, 2, 536, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847726268419, 2, 537, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847726268420, 2, 538, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847726268421, 2, 541, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847730462721, 2, 542, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847730462722, 2, 543, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847730462723, 2, 544, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847734657025, 2, 549, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847734657026, 2, 550, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847734657027, 2, 551, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847734657028, 2, 552, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847738851329, 2, 553, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847738851330, 2, 554, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847738851331, 2, 555, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847743045633, 2, 556, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847743045634, 2, 557, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847743045635, 2, 559, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847747239938, 2, 560, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847747239939, 2, 561, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847747239940, 2, 562, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847747239941, 2, 567, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847751434241, 2, 568, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847751434242, 2, 569, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847751434243, 2, 570, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847755628546, 2, 571, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847755628547, 2, 572, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847755628548, 2, 573, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847755628549, 2, 574, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847759822850, 2, 575, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847759822851, 2, 576, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847759822852, 2, 609, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847764017153, 2, 610, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847764017154, 2, 611, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847764017155, 2, 612, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847764017156, 2, 613, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847768211458, 2, 614, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847768211459, 2, 615, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847768211460, 2, 616, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847772405762, 2, 617, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847772405763, 2, 618, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847772405764, 2, 619, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847772405765, 2, 620, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847776600065, 2, 621, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847780794369, 2, 622, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847780794370, 2, 623, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847780794371, 2, 624, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847780794372, 2, 625, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847784988674, 2, 626, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847784988675, 2, 627, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847784988676, 2, 628, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847789182978, 2, 629, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847789182979, 2, 630, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847789182980, 2, 631, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847789182981, 2, 632, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847793377281, 2, 633, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847793377282, 2, 635, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847793377283, 2, 636, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847797571586, 2, 638, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847797571587, 2, 639, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847797571588, 2, 640, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847797571589, 2, 641, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847801765890, 2, 642, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847801765891, 2, 643, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847801765892, 2, 647, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847805960194, 2, 648, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847805960195, 2, 649, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847805960196, 2, 650, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847805960197, 2, 651, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847810154497, 2, 652, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847810154498, 2, 653, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847810154499, 2, 654, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847814348802, 2, 655, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847814348803, 2, 656, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847814348804, 2, 657, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847818543105, 2, 658, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847826931714, 2, 659, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847831126017, 2, 660, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847835320322, 2, 661, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847835320323, 2, 664, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847839514626, 2, 665, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847839514627, 2, 672, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847839514628, 2, 757, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847843708929, 2, 758, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847843708930, 2, 759, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847843708931, 2, 760, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847847903233, 2, 761, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847856291842, 2, 762, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847860486146, 2, 763, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847860486147, 2, 782, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847860486148, 2, 783, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847864680450, 2, 784, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847864680451, 2, 785, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847864680452, 2, 788, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847864680453, 2, 789, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847868874754, 2, 803, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847868874755, 2, 804, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847868874756, 2, 805, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847873069057, 2, 806, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847873069058, 2, 807, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847873069059, 2, 808, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847873069060, 2, 809, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847877263361, 2, 810, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847877263362, 2, 811, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847877263363, 2, 812, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847877263364, 2, 813, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847881457666, 2, 814, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847881457667, 2, 815, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847881457668, 2, 816, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847885651969, 2, 817, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847885651970, 2, 825, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847885651971, 2, 826, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847885651972, 2, 828, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847889846274, 2, 829, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847889846275, 2, 830, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847889846276, 2, 831, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847894040577, 2, 832, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847894040578, 2, 833, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847894040579, 2, 834, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847894040580, 2, 835, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847898234881, 2, 836, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847898234882, 2, 837, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847898234883, 2, 838, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847898234884, 2, 839, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847902429186, 2, 840, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847902429187, 2, 841, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847902429188, 2, 842, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847906623489, 2, 843, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847906623490, 2, 844, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847910817793, 2, 845, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847910817794, 2, 846, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847910817795, 2, 847, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847915012097, 2, 848, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847915012098, 2, 849, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847915012099, 2, 850, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847919206401, 2, 851, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847919206402, 2, 852, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847919206403, 2, 853, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847919206404, 2, 856, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847923400706, 2, 866, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847923400707, 2, 867, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847923400708, 2, 868, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847927595009, 2, 872, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847927595010, 2, 873, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847927595011, 2, 875, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847927595012, 2, 876, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847931789313, 2, 877, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847931789314, 2, 880, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847931789315, 2, 881, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847935983618, 2, 884, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847935983619, 2, 885, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847935983620, 2, 886, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847935983621, 2, 887, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847940177922, 2, 888, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847940177923, 2, 889, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847940177924, 2, 890, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847940177925, 2, 891, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847944372225, 2, 892, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847944372226, 2, 893, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847944372227, 2, 894, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847948566529, 2, 895, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847948566530, 2, 896, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847948566531, 2, 897, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847948566532, 2, 898, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847952760833, 2, 899, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847952760834, 2, 900, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847952760835, 2, 901, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847956955138, 2, 902, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847956955139, 2, 903, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847956955140, 2, 904, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847956955141, 2, 905, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847961149442, 2, 906, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847961149443, 2, 907, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847961149444, 2, 908, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847965343746, 2, 909, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847965343747, 2, 910, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847965343748, 2, 911, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847965343749, 2, 912, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847969538049, 2, 913, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847969538050, 2, 914, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847969538051, 2, 915, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847969538052, 2, 916, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847973732354, 2, 917, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847973732355, 2, 918, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847973732356, 2, 919, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847977926658, 2, 920, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847977926659, 2, 921, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847977926660, 2, 922, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847977926661, 2, 923, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847982120962, 2, 924, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847982120963, 2, 925, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847982120964, 2, 926, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847986315265, 2, 927, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847986315266, 2, 928, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847986315267, 2, 929, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847986315268, 2, 930, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847990509569, 2, 931, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847990509570, 2, 932, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847990509571, 2, 933, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847990509572, 2, 934, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847994703873, 2, 935, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847994703874, 2, 936, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847998898177, 2, 937, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847998898178, 2, 938, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860847998898179, 2, 939, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848003092482, 2, 940, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848003092483, 2, 941, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848003092484, 2, 942, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848007286785, 2, 943, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848007286786, 2, 944, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848007286787, 2, 945, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848011481089, 2, 946, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848011481090, 2, 947, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848019869697, 2, 948, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848024064001, 2, 949, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848024064002, 2, 950, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848028258306, 2, 951, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848028258307, 2, 952, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848028258308, 2, 953, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848028258309, 2, 954, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848032452609, 2, 955, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848032452610, 2, 956, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848032452611, 2, 957, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848036646913, 2, 958, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848036646914, 2, 959, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848036646915, 2, 960, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848036646916, 2, 961, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848040841217, 2, 962, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848040841218, 2, 963, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848040841219, 2, 964, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848045035522, 2, 965, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848045035523, 2, 966, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848045035524, 2, 967, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1626860847045035525, 2, 968, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1626860847045035526, 2, 969, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726860848045035525, 2, 1008, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309665939457, 3, 280, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309691105281, 3, 281, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309699493889, 3, 282, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309699493890, 3, 283, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309703688194, 3, 284, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309703688195, 3, 285, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309703688196, 3, 541, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309707882498, 3, 542, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309707882499, 3, 543, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309707882500, 3, 544, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309707882501, 3, 806, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309712076802, 3, 807, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309712076803, 3, 40, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309712076804, 3, 561, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309712076805, 3, 49, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309720465409, 3, 568, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309720465410, 3, 830, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309720465411, 3, 575, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309720465412, 3, 831, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309720465413, 3, 576, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309728854018, 3, 832, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309728854019, 3, 833, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309733048322, 3, 834, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309733048323, 3, 835, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309733048324, 3, 836, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309733048325, 3, 837, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309737242626, 3, 838, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309737242627, 3, 70, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309737242628, 3, 71, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309741436930, 3, 839, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309745631233, 3, 840, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309745631234, 3, 72, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309749825537, 3, 841, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309749825538, 3, 842, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309749825539, 3, 843, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309749825540, 3, 844, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309754019842, 3, 77, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309754019843, 3, 845, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309754019844, 3, 846, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309754019845, 3, 847, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309754019846, 3, 848, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309754019847, 3, 849, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309762408450, 3, 850, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309762408451, 3, 851, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309762408452, 3, 852, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309762408453, 3, 853, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309762408454, 3, 866, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309762408455, 3, 876, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309770797058, 3, 371, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309770797059, 3, 884, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309770797060, 3, 885, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309770797061, 3, 886, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309770797062, 3, 887, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309770797063, 3, 888, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309779185665, 3, 889, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309779185666, 3, 890, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309779185667, 3, 891, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309783379969, 3, 892, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309783379970, 3, 893, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309783379971, 3, 894, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309787574273, 3, 895, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309795962881, 3, 640, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309795962882, 3, 896, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309800157185, 3, 897, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309800157186, 3, 641, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309800157187, 3, 898, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309800157188, 3, 642, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309800157189, 3, 899, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309800157190, 3, 900, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309800157191, 3, 901, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309808545794, 3, 902, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309808545795, 3, 903, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309808545796, 3, 904, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309808545797, 3, 905, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309808545798, 3, 650, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309808545799, 3, 906, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309808545800, 3, 651, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309816934401, 3, 907, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309816934402, 3, 652, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309816934403, 3, 908, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309821128706, 3, 909, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309821128707, 3, 910, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309821128708, 3, 911, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309821128709, 3, 912, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309825323010, 3, 913, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309825323011, 3, 914, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309825323012, 3, 915, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309829517314, 3, 916, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309829517315, 3, 917, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309829517316, 3, 918, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309829517317, 3, 919, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309833711618, 3, 920, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309833711619, 3, 921, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309833711620, 3, 922, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309833711621, 3, 923, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309837905921, 3, 924, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309837905922, 3, 925, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309837905923, 3, 926, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309842100225, 3, 927, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309842100226, 3, 672, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309842100227, 3, 928, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309842100228, 3, 417, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309846294529, 3, 929, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309846294530, 3, 930, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309846294531, 3, 931, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309850488833, 3, 420, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309850488834, 3, 932, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309850488835, 3, 421, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309850488836, 3, 933, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309854683138, 3, 934, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309854683139, 3, 167, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309854683140, 3, 935, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309854683141, 3, 168, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309858877442, 3, 424, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309858877443, 3, 936, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309858877444, 3, 937, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309863071745, 3, 938, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309863071746, 3, 427, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309863071747, 3, 939, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309863071748, 3, 940, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309867266049, 3, 941, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309867266050, 3, 174, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309867266051, 3, 942, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309871460354, 3, 943, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309871460355, 3, 944, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309871460356, 3, 945, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309871460357, 3, 946, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309875654657, 3, 947, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309875654658, 3, 948, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309875654659, 3, 949, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309875654660, 3, 950, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309879848962, 3, 951, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309879848963, 3, 952, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309879848964, 3, 441, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309884043265, 3, 953, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309884043266, 3, 442, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309884043267, 3, 954, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309884043268, 3, 955, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309888237569, 3, 956, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309888237570, 3, 957, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309888237571, 3, 958, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309892431873, 3, 959, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309892431874, 3, 960, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309892431875, 3, 961, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309892431876, 3, 962, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309896626178, 3, 963, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309896626179, 3, 964, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309896626180, 3, 965, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309900820482, 3, 966, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309900820483, 3, 967, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1716811119900820483, 3, 968, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1716811119900820484, 3, 969, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309900820484, 3, 463, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309900820485, 3, 464, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309905014786, 3, 465, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309905014787, 3, 466, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309905014788, 3, 467, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309905014789, 3, 468, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309909209089, 3, 477, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309909209090, 3, 483, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309909209091, 3, 486, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309913403394, 3, 487, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309913403395, 3, 488, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309913403396, 3, 489, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309913403397, 3, 490, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309917597698, 3, 491, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309917597699, 3, 492, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309917597700, 3, 493, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309921792002, 3, 494, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309921792003, 3, 495, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309921792004, 3, 1008, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309921792005, 3, 498, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309925986306, 3, 499, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309925986307, 3, 500, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309925986308, 3, 759, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309925986309, 3, 760, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861309930180609, 3, 506, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547864657922, 4, 650, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547877240833, 4, 651, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547877240834, 4, 652, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547877240835, 4, 272, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547881435138, 4, 273, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547881435139, 4, 274, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547881435140, 4, 275, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547885629441, 4, 277, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547885629442, 4, 278, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547885629443, 4, 279, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547889823746, 4, 804, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547889823747, 4, 165, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547894018049, 4, 805, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547894018050, 4, 39, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547894018051, 4, 169, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547898212353, 4, 42, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547898212354, 4, 299, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547898212355, 4, 300, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547902406658, 4, 301, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547902406659, 4, 302, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547906600962, 4, 304, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547906600963, 4, 305, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547906600964, 4, 441, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547910795266, 4, 442, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547910795267, 4, 445, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547910795268, 4, 830, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547914989569, 4, 575, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547914989570, 4, 191, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547914989571, 4, 831, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547914989572, 4, 63, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547919183874, 4, 576, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547919183875, 4, 448, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547923378177, 4, 832, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547927572482, 4, 64, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547927572483, 4, 833, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547927572484, 4, 65, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547927572485, 4, 66, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547931766785, 4, 834, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547931766786, 4, 835, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547935961089, 4, 67, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547935961090, 4, 836, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547935961091, 4, 68, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547935961092, 4, 837, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547940155393, 4, 838, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547940155394, 4, 839, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547940155395, 4, 840, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547944349698, 4, 841, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547944349699, 4, 842, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547944349700, 4, 843, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547948544001, 4, 844, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547948544002, 4, 76, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547948544003, 4, 77, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547948544004, 4, 845, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547952738306, 4, 846, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547952738307, 4, 847, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547952738308, 4, 848, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547956932609, 4, 849, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547956932610, 4, 850, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547956932611, 4, 851, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547961126913, 4, 852, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547961126914, 4, 853, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547961126915, 4, 482, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861547961126916, 4, 866, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861586607443970, 5, 199, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861586615832578, 5, 200, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861586620026881, 5, 201, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861586620026882, 5, 202, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861586620026883, 5, 203, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861586624221185, 5, 204, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861586624221186, 5, 205, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861586628415489, 5, 973, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861586628415490, 5, 206, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861586628415491, 5, 974, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861586632609793, 5, 207, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861586632609794, 5, 975, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861586632609795, 5, 208, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861586636804097, 5, 209, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861586636804098, 5, 210, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861586636804099, 5, 211, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861586640998401, 5, 212, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861586640998402, 5, 213, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861586640998403, 5, 214, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861586645192706, 5, 215, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861586645192707, 5, 216, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744300691457, 6, 970, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744309080066, 6, 971, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744313274369, 6, 972, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744313274370, 6, 973, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744317468673, 6, 974, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744321662978, 6, 975, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744325857281, 6, 976, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744325857282, 6, 977, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744330051585, 6, 978, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744330051586, 6, 979, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744334245889, 6, 980, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744334245890, 6, 981, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744334245891, 6, 982, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744338440193, 6, 983, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744338440194, 6, 984, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744338440195, 6, 985, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744342634498, 6, 986, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744342634499, 6, 987, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744342634500, 6, 988, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744346828802, 6, 989, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744346828803, 6, 990, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744346828804, 6, 991, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744346828805, 6, 992, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744351023105, 6, 993, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744351023106, 6, 994, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744355217409, 6, 995, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744355217410, 6, 996, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744359411714, 6, 997, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744359411715, 6, 998, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744359411716, 6, 999, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744363606017, 6, 1000, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744363606018, 6, 1001, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744367800322, 6, 1002, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744367800323, 6, 1003, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744367800324, 6, 1004, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744371994626, 6, 1005, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744371994627, 6, 1006, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744371994628, 6, 1007, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726861744376188929, 6, 1009, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1626767744376188929, 6, 1010, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862011016482818, 7, 896, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862011029065729, 7, 483, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862011033260034, 7, 486, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862011033260035, 7, 491, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862011033260036, 7, 492, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862011037454337, 7, 493, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862011037454338, 7, 884, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862011037454339, 7, 885, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862011037454340, 7, 886, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862011041648642, 7, 887, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862011041648643, 7, 888, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862011041648644, 7, 921, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862011045842945, 7, 889, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862011045842946, 7, 890, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862011045842947, 7, 891, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862011045842948, 7, 892, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862011050037250, 7, 893, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862011050037251, 7, 541, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862011050037252, 7, 894, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862011054231554, 7, 895, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862066481958914, 8, 769, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862066490347521, 8, 770, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862097310093314, 9, 769, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862097331064833, 9, 770, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862232698032130, 11, 739, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862232706420737, 11, 680, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862255003340802, 12, 739, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862255011729410, 12, 680, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862296006856706, 10, 739, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1726862296015245314, 10, 680, 1, NULL);

-- ----------------------------
-- Table structure for sys_user
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sys_user` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint DEFAULT NULL COMMENT '学校id',
        `login_name` varchar(100) NOT NULL COMMENT '用户名',
        `real_name` varchar(100) NOT NULL COMMENT '姓名',
        `code` varchar(100) DEFAULT NULL COMMENT '工号',
        `password` varchar(50) NOT NULL COMMENT '密码',
        `mobile_number` varchar(30) DEFAULT NULL COMMENT '手机号',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
        `pwd_count` tinyint DEFAULT '0' COMMENT '密码修改次数，默认为0',
        `pwd_update_time` bigint DEFAULT NULL COMMENT '密码修改时间',
        `remark` varchar(200) DEFAULT NULL COMMENT '备注',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '修改人',
        `update_time` bigint DEFAULT NULL COMMENT '修改时间',
        `historic_name` varchar(100) DEFAULT NULL COMMENT '历史用户名称',
        `ui_setting` text COMMENT '阅卷参数',
        `allow_manual_card` tinyint(1) DEFAULT '0' COMMENT '是否允许新增卡格式',
        PRIMARY KEY (`id`) USING BTREE,
        UNIQUE KEY `user_schoolId_loginName_idx` (`school_id`,`login_name`) USING BTREE,
        UNIQUE KEY `uq_school_id_mobile_number` (`school_id`,`mobile_number`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='用户表';

INSERT INTO `sys_user` (`id`, `school_id`, `login_name`, `real_name`, `code`, `password`, `mobile_number`, `org_id`, `enable`, `pwd_count`, `pwd_update_time`, `remark`, `create_id`, `create_time`, `update_id`, `update_time`, `historic_name`, `ui_setting`, `allow_manual_card`) VALUES (1, NULL, 'sysadmin', 'sysadmin', NULL, 'UW10aDg3ODYzNTc3', NULL, NULL, 1, 2, 1700550554413, NULL, NULL, NULL, NULL, 1700550554425, NULL, NULL, 0);

-- ----------------------------
-- Table structure for sys_user_role
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sys_user_role` (
        `id` bigint NOT NULL COMMENT '主键',
        `user_id` bigint NOT NULL COMMENT '用户id',
        `role_id` bigint NOT NULL COMMENT '角色id',
        `privilege_id` bigint DEFAULT NULL COMMENT '权限id',
        `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='用户和角色关联表';

INSERT INTO `sys_user_role` (`id`, `user_id`, `role_id`, `privilege_id`, `enable`) VALUES (1, 1, 1, NULL, 1);

-- ----------------------------
-- Table structure for t_b_session
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_b_session` (
        `id` varchar(100) NOT NULL COMMENT '主键',
        `school_id` bigint DEFAULT NULL COMMENT '学校id',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `identity` varchar(100) NOT NULL COMMENT '用户标识',
        `type` varchar(50) NOT NULL COMMENT '用户类型',
        `source` varchar(50) NOT NULL COMMENT '访问来源',
        `platform` varchar(50) NOT NULL COMMENT '设备分类',
        `device_id` varchar(100) NOT NULL COMMENT '设备标识',
        `address` varchar(100) NOT NULL COMMENT '登录IP地址',
        `access_token` varchar(50) NOT NULL COMMENT '访问令牌',
        `last_access_time` bigint DEFAULT NULL COMMENT '最近访问时间',
        `last_access_ip` varchar(100) DEFAULT NULL COMMENT '最近访问IP地址',
        `update_time` bigint DEFAULT NULL COMMENT '令牌更新时间',
        `expire_time` bigint DEFAULT NULL COMMENT '令牌强制失效时间',
        `app_source` varchar(50) DEFAULT NULL COMMENT '应用来源',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='会话表';

-- ----------------------------
-- Table structure for t_b_sync_task
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_b_sync_task` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint DEFAULT NULL COMMENT '学校id',
        `semester_id` bigint DEFAULT NULL COMMENT '学期',
        `exam_id` bigint DEFAULT NULL COMMENT '考试',
        `course_code` varchar(50) DEFAULT NULL COMMENT '课程代码',
        `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
        `paper_number` varchar(45) DEFAULT NULL COMMENT '试卷编号',
        `paper_type` varchar(10) DEFAULT NULL COMMENT '试卷类型',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `type` varchar(50) NOT NULL COMMENT '推送类型，USER_PUSH：用户推送',
        `exam_time` varchar(20) DEFAULT NULL COMMENT '考试时间',
        `object_id` bigint DEFAULT NULL COMMENT '印刷计划id',
        `status` varchar(30) NOT NULL COMMENT '任务状态，INIT：未开始，RUNNING：进行中，FINISH：已完成',
        `summary` text COMMENT '实时摘要信息',
        `result` varchar(30) DEFAULT NULL COMMENT '数据结果，SUCCESS：成功，ERROR：失败',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '修改人',
        `update_time` bigint DEFAULT NULL COMMENT '修改时间',
        `remark` mediumtext COMMENT '备注',
        `reset_count` int DEFAULT '0' COMMENT '重试次数',
        `error_message` varchar(500) DEFAULT NULL COMMENT '人工错误原因',
        `report_file_path` varchar(500) DEFAULT NULL COMMENT '报告路径',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='云阅卷同步任务表';

-- ----------------------------
-- Table structure for t_b_task
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_b_task` (
        `id` bigint NOT NULL COMMENT '主键',
        `semester_id` bigint DEFAULT NULL COMMENT '学期',
        `exam_id` bigint DEFAULT NULL COMMENT '考试',
        `course_code` varchar(50) DEFAULT NULL COMMENT '课程代码',
        `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
        `paper_number` varchar(45) DEFAULT NULL COMMENT '试卷编号',
        `type` varchar(50) NOT NULL COMMENT '任务类型，USER_IMPORT：用户导入，\nQUESTION_MISSION_BATCH_CREATE：批量新建命题任务，\nSAMPLE_EXPORT：导出审核样本，\nPAPER_DOWNLOAD：卷库下载，\nEXAMINATION_IMPORT：考务数据导入，\nEXAMINATION_EXPORT：考务数据导出，\nPRINT_PDF_DOWNLOAD：批量下载pdf，\nCREATE_PDF：生成pdf，\nCOURSE_IMPORT：课程导入',
        `entity_id` bigint DEFAULT NULL COMMENT '关联业务对象id',
        `status` varchar(30) NOT NULL COMMENT '任务状态，INIT：未开始，RUNNING：进行中，FINISH：已完成',
        `summary` text COMMENT '实时摘要信息',
        `progress` double DEFAULT NULL COMMENT '执行进度',
        `result` varchar(30) DEFAULT NULL COMMENT '数据结果，SUCCESS：成功，ERROR：失败',
        `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，false：停用，true：启用',
        `import_file_name` varchar(100) DEFAULT NULL COMMENT '导入文件名',
        `import_file_path` varchar(500) DEFAULT NULL COMMENT '导入文件路径',
        `result_file_path` varchar(500) DEFAULT NULL COMMENT '导出文件路径',
        `error_file_path` varchar(500) DEFAULT NULL COMMENT '错误数据文件路径',
        `report_file_path` varchar(500) DEFAULT NULL COMMENT '报告路径',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `school_id` bigint DEFAULT NULL COMMENT '学校id',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `print_plan_id` bigint DEFAULT NULL COMMENT '印刷计划id',
        `remark` mediumtext COMMENT '备注',
        `obj_name` varchar(500) DEFAULT NULL COMMENT '实体名称',
        `reset_count` int DEFAULT '0' COMMENT '重试次数',
        `version` int DEFAULT '0' COMMENT '更新版本号',
        `error_message` varchar(500) DEFAULT NULL COMMENT '人工错误原因',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='导入导出任务表';

-- ----------------------------
-- Table structure for t_c_statistics
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_c_statistics` (
        `id` bigint NOT NULL COMMENT '主键',
        `exam_id` bigint NOT NULL COMMENT '考试id',
        `exam_task_detail_ids` mediumtext COMMENT '命题任务明细id',
        `exam_detail_course_ids` mediumtext COMMENT '考务数据课程id',
        `college_id` bigint NOT NULL COMMENT '学院id',
        `college_name` varchar(100) NOT NULL COMMENT '学院名称',
        `course_name` varchar(100) NOT NULL COMMENT '课程名称',
        `course_code` varchar(100) NOT NULL COMMENT '课程代码',
        `teaching_room_id` bigint NOT NULL COMMENT '教研室id',
        `teaching_room_name` varchar(100) NOT NULL COMMENT '教研室名称',
        `teacher_name` mediumtext NOT NULL COMMENT '任课老师名称',
        `clazz_ids` varchar(100) NOT NULL COMMENT '班级ids',
        `clazz_name` varchar(100) NOT NULL COMMENT '班级名称',
        `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷编号',
        `print_plan_id` bigint DEFAULT NULL COMMENT '印刷计划id',
        `batch_no` varchar(100) NOT NULL COMMENT '批次号',
        `create_id` bigint DEFAULT NULL COMMENT '创建人id',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人id',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        `print_sum` varchar(100) DEFAULT NULL COMMENT '总印份数',
        `status` varchar(50) DEFAULT NULL COMMENT '完成状态，FINISH：已完成，UN_FINISH：未完成',
        `exam_detail_id` bigint DEFAULT NULL COMMENT '命题明细id',
        `clazz_content` mediumtext COMMENT '班级详细信息内容',
        PRIMARY KEY (`id`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='命题统计表';

-- ----------------------------
-- Table structure for t_c_statistics_temp
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_c_statistics_temp` (
        `id` bigint NOT NULL COMMENT '主键',
        `exam_id` bigint NOT NULL COMMENT '考试id',
        `exam_task_detail_id` bigint DEFAULT NULL COMMENT '命题任务明细id',
        `exam_detail_course_id` bigint DEFAULT NULL COMMENT '考务数据课程id',
        `college_id` bigint NOT NULL COMMENT '学院id',
        `college_name` varchar(100) NOT NULL COMMENT '学院名称',
        `course_name` varchar(100) NOT NULL COMMENT '课程名称',
        `course_code` varchar(100) NOT NULL COMMENT '课程代码',
        `teaching_room_id` bigint NOT NULL COMMENT '教研室id',
        `teaching_room_name` varchar(100) NOT NULL COMMENT '教研室名称',
        `teacher_name` mediumtext NOT NULL COMMENT '任课老师名称',
        `clazz_ids` varchar(100) NOT NULL COMMENT '班级ids',
        `clazz_name` varchar(100) NOT NULL COMMENT '班级名称',
        `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷编号',
        `print_plan_id` bigint DEFAULT NULL COMMENT '印刷计划id',
        `batch_no` varchar(100) NOT NULL COMMENT '批次号',
        `create_id` bigint DEFAULT NULL COMMENT '创建人id',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人id',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        `data_source` varchar(50) DEFAULT NULL COMMENT '数据来源，IMPORT：导入，JOIN：关联查询的数据',
        `exam_detail_id` bigint DEFAULT NULL COMMENT '命题明细id',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='命题统计表';

-- ----------------------------
-- Table structure for t_f_custom_flow
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_f_custom_flow` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `name` varchar(100) NOT NULL COMMENT '流程名称',
        `model_type` varchar(50) NOT NULL COMMENT '模式类型，USER_FIXED：指定人员,APPROVE_SET：发起人指定',
        `type` varchar(100) NOT NULL COMMENT '流程类型，ELECTRON_FLOW：电子交卷审核，QUESTION_ELECTRON_FLOW：题库试题提交审核，PAPER_ELECTRON_FLOW：题库试卷审核',
        `enable` tinyint DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
        `publish` tinyint NOT NULL DEFAULT '1' COMMENT '是否发布，0：否，1：是',
        `object_data` mediumtext NOT NULL COMMENT '自定义流程源数据',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        `version` bigint NOT NULL DEFAULT '1' COMMENT '版本号',
        `flow_key` varchar(100) DEFAULT NULL COMMENT '流程key',
        `act_flow_id` varchar(100) DEFAULT NULL COMMENT '流程版本id',
        `flow_process_var` mediumtext COMMENT '流程过程变量数据',
        PRIMARY KEY (`id`) USING BTREE,
        UNIQUE KEY `custom_flow_schoolId_type_idx` (`school_id`,`type`,`enable`,`publish`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='自定义流程表';

-- ----------------------------
-- Table structure for t_f_custom_flow_entity
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_f_custom_flow_entity` (
        `id` bigint NOT NULL COMMENT '主键',
        `t_f_custom_flow_id` bigint NOT NULL COMMENT '自定义流程id',
        `flow_id` bigint DEFAULT NULL COMMENT '流程id',
        `flow_process_var` mediumtext NOT NULL COMMENT '流程过程变量',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE,
        KEY `flow_entity_flowId_idx` (`flow_id`) USING BTREE,
        KEY `flow_entity_custom_flowId_idx` (`t_f_custom_flow_id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='自定义流程实体数据';


-- ----------------------------
-- Table structure for t_f_flow_approve
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_f_flow_approve` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `flow_id` bigint NOT NULL COMMENT '流程id',
        `approve_id` bigint DEFAULT NULL COMMENT '审核id',
        `status` varchar(50) NOT NULL COMMENT '状态，START：已开始，AUDITING：审核中，REJECT：已驳回，END：已终止，FINISH：已结束',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        `setup` tinyint NOT NULL COMMENT '计划步骤',
        `approve_setup` int DEFAULT NULL COMMENT '审批步骤',
        `approve_max_setup` int DEFAULT NULL COMMENT '最大审批步骤',
        `transfer_assignee` mediumtext COMMENT '流程转办',
        `all_assignee` mediumtext COMMENT '会签审批人',
        `reject_link` mediumtext COMMENT '驳回路线',
        `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
        PRIMARY KEY (`id`) USING BTREE,
        KEY `flow_approve_idx` (`school_id`,`flow_id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='流程审核表';


-- ----------------------------
-- Table structure for t_f_flow_join
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_f_flow_join` (
        `id` bigint NOT NULL COMMENT '主键',
        `flow_entity_id` bigint NOT NULL COMMENT '自定义流程实体id',
        `object_table` varchar(100) NOT NULL COMMENT '实体表',
        `object_id` bigint NOT NULL COMMENT '实体id',
        `flow_id` bigint NOT NULL COMMENT '流程id',
        `level` int NOT NULL COMMENT '流程等级',
        `type` varchar(50) NOT NULL COMMENT '类型，NEW：新增，RESTART：重新提交',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        PRIMARY KEY (`id`) USING BTREE,
        KEY `task_flowId_idx` (`flow_id`) USING BTREE,
        KEY `task_objectId_idx` (`object_id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='实体和流程关联表';


-- ----------------------------
-- Table structure for t_f_flow_log
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_f_flow_log` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `flow_id` bigint NOT NULL COMMENT '流程id',
        `object_table` varchar(100) DEFAULT NULL COMMENT '实体表',
        `object_id` bigint NOT NULL COMMENT '实体id',
        `approve_id` bigint DEFAULT NULL COMMENT '审核人id',
        `approve_setup` tinyint DEFAULT NULL COMMENT '审核步骤',
        `approve_remark` mediumtext COMMENT '审核意见',
        `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        `approve_operation` varchar(50) NOT NULL COMMENT '审批操作，SUBMIT：提交，APPROVE：审批，REJECT：驳回，END：终止',
        `pend_approve_id` mediumtext COMMENT '待审核人id',
        `paper_attachment_id` mediumtext COMMENT '试卷附件',
        `approve_multi_setup` int DEFAULT NULL COMMENT '审批步骤',
        `approve_user_approve_type` varchar(30) DEFAULT NULL COMMENT '审批用户类别，ORDER：依次审批，ALL：会签（所有人必须审批），SOME：或签（一名审批人同意或拒绝即可）',
        `task_id` bigint DEFAULT NULL COMMENT '流程节点id',
        PRIMARY KEY (`id`) USING BTREE,
        KEY `flow_log_idx` (`school_id`,`flow_id`) USING BTREE,
        KEY `flow_log_objectId_idx` (`object_id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='流程流水表';


-- ----------------------------
-- Table structure for t_g_error
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_g_error` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint DEFAULT NULL COMMENT '学校id',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `obj_id` bigint DEFAULT NULL COMMENT '实体id',
        `summary` mediumtext COMMENT '实时摘要信息',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='全局异常错误信息表';


-- ----------------------------
-- Table structure for t_m_mq_message
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_m_mq_message` (
        `id` varchar(50) NOT NULL COMMENT '主键',
        `school_id` bigint DEFAULT NULL COMMENT '学校id',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `topic` varchar(100) NOT NULL COMMENT '消息topic',
        `tag` varchar(100) NOT NULL COMMENT '消息tag',
        `body` mediumtext COMMENT '消息内容',
        `type` varchar(50) NOT NULL COMMENT '消息类型',
        `obj_id` varchar(50) NOT NULL COMMENT '关联业务id',
        `obj_name` varchar(100) DEFAULT NULL COMMENT '关联业务名称',
        `ack` tinyint NOT NULL COMMENT '消息ack',
        `sequence` int DEFAULT NULL COMMENT '消息序号',
        `properties` varchar(1000) DEFAULT NULL COMMENT '扩展类型',
        `create_time` bigint NOT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        `timestamp` bigint NOT NULL COMMENT '时间戳',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='mq消息';


-- ----------------------------
-- Table structure for t_s_auth
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_s_auth` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint DEFAULT NULL COMMENT '学校id',
        `access_key` varchar(255) DEFAULT NULL COMMENT 'accessKey',
        `access_secret` varchar(255) DEFAULT NULL COMMENT 'accessSecret',
        `description` text COMMENT '离线授权证书',
        `type` varchar(16) NOT NULL COMMENT '授权类型',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        `expire_time` bigint DEFAULT NULL COMMENT '过期时间',
        `file` mediumblob COMMENT '文件数据',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='激活授权配置表';


-- ----------------------------
-- Table structure for t_school_privilege
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_school_privilege` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `privilege_id` bigint NOT NULL COMMENT '菜单权限id',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='学校菜单权限表';


-- ----------------------------
-- Table structure for t_school_role
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_school_role` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `role_id` bigint NOT NULL COMMENT '菜单权限id',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='学校角色权限表';


-- ----------------------------
-- Table structure for t_sync_exam_log
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_sync_exam_log` (
        `id` bigint NOT NULL,
        `school_id` bigint DEFAULT NULL COMMENT '学校ID',
        `semester_id` bigint DEFAULT NULL COMMENT '学期ID',
        `exam_id` bigint DEFAULT NULL COMMENT '云阅卷考试ID',
        `third_relate_id` int DEFAULT NULL COMMENT '云阅卷考试ID',
        `exam_name` varchar(100) DEFAULT NULL COMMENT '云阅卷考试名称',
        `status` varchar(20) DEFAULT NULL COMMENT '任务状态，INIT：未开始，RUNNING：进行中，FINISH：已完成',
        `result` varchar(20) DEFAULT NULL COMMENT '数据结果，SUCCESS：成功，ERROR：失败',
        `start_time` bigint DEFAULT NULL COMMENT '同步开始时间',
        `end_time` bigint DEFAULT NULL COMMENT '结束同步时间',
        `summary` mediumtext,
        `total_courses` int DEFAULT NULL COMMENT '总课程数',
        `total_exam_tasks` int DEFAULT NULL COMMENT '总命题任务数',
        `total_students` int DEFAULT NULL COMMENT '总考生数',
        `success_students` int DEFAULT NULL COMMENT '成功人数',
        `fail_students` int DEFAULT NULL COMMENT '失败人数',
        `error_file_path` varchar(500) DEFAULT NULL COMMENT '错误日志文件地址',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='同步考试日志表';


-- ----------------------------
-- Table structure for t_sync_exam_student_score
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_sync_exam_student_score` (
        `id` bigint NOT NULL COMMENT '主键',
        `school_id` bigint DEFAULT NULL COMMENT '学校id',
        `org_id` bigint DEFAULT NULL COMMENT '机构id',
        `semester_id` bigint DEFAULT NULL COMMENT '学期id',
        `major_name` varchar(100) DEFAULT NULL COMMENT '专业id',
        `clazz_id` bigint DEFAULT NULL COMMENT '班级id',
        `exam_id` bigint DEFAULT NULL COMMENT '云阅卷考试id',
        `third_relate_id` int DEFAULT NULL COMMENT '云阅卷考试ID',
        `exam_code` varchar(100) DEFAULT NULL COMMENT '云阅卷考试标识',
        `exam_number` varchar(100) DEFAULT NULL COMMENT '准考证号',
        `student_code` varchar(100) DEFAULT NULL COMMENT '学号',
        `name` varchar(100) NOT NULL COMMENT '姓名',
        `paper_number` varchar(45) DEFAULT NULL COMMENT '试卷编号',
        `subject_code` varchar(100) DEFAULT NULL COMMENT '科目代码',
        `subject_name` varchar(100) DEFAULT NULL COMMENT '科目名称',
        `paper_type` varchar(50) DEFAULT NULL COMMENT '试卷类型',
        `college` varchar(100) DEFAULT NULL COMMENT '学院',
        `clazz_name` varchar(100) DEFAULT NULL COMMENT '班级',
        `teacher` varchar(100) DEFAULT NULL COMMENT '教师',
        `status` tinyint DEFAULT NULL COMMENT '考生状态，1-正常，2-缺考(包含未上传)，3-违纪',
        `sheet_urls` mediumtext COMMENT '原卷url',
        `local_sheet_urls` mediumtext COMMENT '从云阅卷下载下来的原图url',
        `total_score` double DEFAULT NULL COMMENT '总分',
        `objective_score` double DEFAULT NULL COMMENT '客观题分数',
        `subjective_score` double DEFAULT NULL COMMENT '主观题分数',
        `sync_data` mediumtext COMMENT '同步数据json',
        `trajectory_urls` varchar(500) DEFAULT NULL COMMENT '轨迹图地址',
        `create_id` bigint DEFAULT NULL COMMENT '创建人id',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        PRIMARY KEY (`id`) USING BTREE,
        KEY `score_schoolId_idx` (`school_id`,`semester_id`,`org_id`,`major_name`,`clazz_id`,`student_code`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='同步考生成绩表';


-- ----------------------------
-- Table structure for t_sync_stmms_exam
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_sync_stmms_exam` (
        `id` bigint NOT NULL,
        `school_id` bigint DEFAULT NULL COMMENT '学校ID',
        `org_code` varchar(45) DEFAULT NULL COMMENT '机构代码',
        `exam_id` int DEFAULT NULL COMMENT '云阅卷ID',
        `exam_name` varchar(45) DEFAULT NULL COMMENT '云阅卷考试名称',
        `create_id` bigint DEFAULT NULL,
        `create_time` bigint DEFAULT NULL,
        PRIMARY KEY (`id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='云阅卷考试表';


-- ----------------------------
-- Table structure for teach_clazz
-- ----------------------------
CREATE TABLE IF NOT EXISTS `teach_clazz` (
        `id` bigint NOT NULL COMMENT '教学班表主键',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `teach_clazz_code` varchar(20) DEFAULT NULL COMMENT '教学班编号',
        `teach_clazz_name` varchar(50) NOT NULL COMMENT '教学班名称',
        `user_id` bigint NOT NULL COMMENT '用户id',
        `basic_course_id` bigint NOT NULL COMMENT '基础课程主键',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE,
        UNIQUE KEY `teach_clazz_unique` (`school_id`,`basic_course_id`,`teach_clazz_name`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='教学班表';


-- ----------------------------
-- Table structure for teach_course
-- ----------------------------
CREATE TABLE IF NOT EXISTS `teach_course` (
        `id` bigint NOT NULL,
        `school_id` bigint NOT NULL COMMENT '学校id',
        `basic_course_id` bigint NOT NULL COMMENT '基础课程id',
        `user_id` bigint NOT NULL COMMENT '创建此教学课程的用户id',
        `enable` tinyint(1) DEFAULT NULL COMMENT '是否删除',
        `create_id` bigint DEFAULT NULL COMMENT '创建人id',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人id',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE,
        UNIQUE KEY `teach_course_unique` (`school_id`,`basic_course_id`,`user_id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='教学课程表';


-- ----------------------------
-- Table structure for teach_student
-- ----------------------------
CREATE TABLE IF NOT EXISTS `teach_student` (
        `id` bigint NOT NULL COMMENT '教学学生关系表主键',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `student_id` bigint NOT NULL COMMENT '学生id',
        `teach_clazz_id` bigint NOT NULL COMMENT '教学班级id',
        `basic_course_id` bigint NOT NULL COMMENT '基础课程id（冗余）',
        `user_id` bigint NOT NULL COMMENT '用户id（冗余）',
        `create_id` bigint DEFAULT NULL COMMENT '创建人',
        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
        `update_id` bigint DEFAULT NULL COMMENT '更新人',
        `update_time` bigint DEFAULT NULL COMMENT '更新时间',
        PRIMARY KEY (`id`) USING BTREE,
        UNIQUE KEY `teach_student_unique` (`school_id`,`student_id`,`teach_clazz_id`) USING BTREE
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='教学学生关系表';


-- ----------------------------
-- Function structure for getRootChildId
-- ----------------------------
DROP FUNCTION IF EXISTS `getRootChildId`;
delimiter ;;
CREATE FUNCTION `getRootChildId`(rootId bigint)
    RETURNS mediumtext CHARSET utf8mb4
  DETERMINISTIC
BEGIN
    DECLARE ptemp MEDIUMTEXT;
    DECLARE ctemp MEDIUMTEXT;
    SET ptemp = '#';
    SET ctemp = cast(rootId as CHAR);
    WHILE ctemp IS NOT NULL DO
      SET ptemp = concat(ptemp, ',', ctemp);
SELECT group_concat(id) INTO ctemp FROM sys_org WHERE FIND_IN_SET(parent_id, ctemp) > 0;
END WHILE;
RETURN ptemp;
END
;;
delimiter ;
